﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;

using MathNet.Numerics.LinearAlgebra;

using Agora.Model.Utils;

namespace Agora.Model.ImageAnalysis
{
    public class CImageAnalysis
    {
        private static CCache _Cache = null;
        public static CCache Cache
        {
            get { return _Cache; }
            set { _Cache = value; }
        }

        private System.Drawing.Bitmap _CurrentImage = null;
        public System.Drawing.Bitmap CurrentImage
        {
            get { return _CurrentImage; }
        }

        private String _FileName = null;
        public String FileName
        {
            get { return _FileName; }
        }

        private CGraph _Graph = new CGraph();
        public CGraph Graph
        {
            get { return _Graph; }
        }

        private List<CEOCType> _EOCTypes = new List<CEOCType>();
        public List<CEOCType> EOCTypes
        {
            get { return _EOCTypes; }
            set { _EOCTypes = value; }
        }

        public CImageAnalysis()
        {
        }
        public CImageAnalysis Clone()
        {
            CImageAnalysis result = new CImageAnalysis();
            result._CurrentImage = this._CurrentImage;
            result._FileName = this._FileName;
            result._Graph = this._Graph.Clone();
            //result.SetDimensions(this.GetDimensions());

            Dictionary<CEOCType, CEOCType> eocTypeDic = new Dictionary<CEOCType, CEOCType>();
            foreach (CEOCType type in _EOCTypes)
            {
                CEOCType newType = new CEOCType(type.Type, type.TypeName);
                result._EOCTypes.Add(newType);
                eocTypeDic.Add(type, newType);
            }
            CEOC eoc;
            CBFS bfs = new CBFS();
            bfs.Start(result._Graph.GetRoot());
            while ((eoc = bfs.GetNext()) != null)
            {
                CEOCType oldType = eoc.Eoctype;
                CEOCType newType = eocTypeDic[oldType];
                eoc.Eoctype = newType;
            }

            return result;
        }

        public void CreateEOCType(string name)
        {
            // find an free id, then use it for creating a new EOC Type
            int id = 0;
            while (this.GetEOCTypeByID(id) != null)
            {
                id++;
            }
            this.CreateEOCType(id, name.ToUpper());
        }
        public void CreateEOCType(int id, string name)
        {
            bool found = false;
            foreach (CEOCType t in EOCTypes)
            {
                if (t.Type == id)
                {
                    found = true;
                    break;
                }
            }
            if (found == false)
            {
                CEOCType type = new CEOCType(id, name.ToUpper());
                EOCTypes.Add(type);
            }
        }
        public void DeleteEOCType(CEOCType type)
        {
            while (EOCTypes.Remove(type) == true) ;
        }
        public CEOCType GetEOCTypeByID(int id)
        {
            foreach (CEOCType t in EOCTypes)
            {
                if (t.Type == id)
                {
                    return t;
                }
            }
            return null;
        }
        public CEOCType GetEOCTypeByName(string name)
        {
            foreach (CEOCType t in EOCTypes)
            {
                if (t.TypeName.ToUpper() == name.ToUpper())
                {
                    return t;
                }
            }
            return null;
        }

        public List<string> GetEOCTypeNames()
        {
            List<string> res = new List<string>();
            foreach (CEOCType t in EOCTypes)
            {
                res.Add(String.Copy(t.TypeName.ToUpper()));
            }
            return res;
        }

        public void SetDimensions(CDimensionType[] dimensions)
        {
            CEOC eoc = _Graph.GetRoot();
            eoc.SetDimensions(dimensions);
            eoc.Measure(CEOCType.EOCDocument);
        }
        public CDimensionType[] GetDimensions()
        {
            CEOC eoc = _Graph.GetRoot();
            CDimensionType[] result = new CDimensionType[eoc.Dimensions.Length];
            for (int i = 0; i < eoc.Dimensions.Length; i++)
            {
                CDimension dim = eoc.Dimensions[i];
                result[i] = dim.Type;
            }
            return result;
        }
        public int GetDimensionIndex(CDimensionType dim)
        {
            int result = -1;
            CDimensionType[] dimensions = this.GetDimensions();
            for (int i = 0; i < dimensions.Length; i++)
            {
                if (dimensions[i] == dim)
                {
                    result = i;
                    break;
                }
            }
            return result;
        }

        public void LoadImage(String fn)
        {
            Bitmap bm;
            this._FileName = fn;
            try
            {
                bm = new Bitmap(_FileName);
                bm = AForge.Imaging.Image.Clone(bm, PixelFormat.Format24bppRgb);
            }
            catch (Exception)
            {
                throw new ApplicationException("Failed loading image from file" + _FileName);
            }

            _Cache = new CCache(_FileName);

            _CurrentImage = bm;
            //_Graph = new CGraph();
            //_EOCTypes = new List<CEOCType>();
            this.CreateEOCType(CEOCType.EOCNoType, CEOCType.EOCNoTypeName);
            this.CreateEOCType(CEOCType.EOCDocument, CEOCType.EOCDocumentName);

            CEOC eoc = new CEOC(
                /*_Graph,*/
                bm,
                new Rectangle(0, 0, bm.Width, bm.Height),
                this.GetEOCTypeByID(CEOCType.EOCDocument)
                );
            _Graph.SetRoot(eoc);

        }


    }
}
