﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Agora.Model.Utils;

namespace Agora.Model.ImageAnalysis
{

    public class CPattern
    {
        private CEOCType _PatternType;
        public CEOCType PatternType
        {
            get { return _PatternType; }
            set { _PatternType = value; }
        }

        private CGraph _Graph = null;
        public CGraph Graph
        {
            get { return _Graph; }
            //set { _graph = value; }
        }

        private bool _IsRecurse;
        public bool IsRecurse
        {
            get { return _IsRecurse; }
            set { _IsRecurse = value; }
        }

        public CPattern(CEOCType t)
        {
            _Graph = new CGraph();
            _PatternType = t;
        }

        public void LoadPatternNChilds(
            int nbChilds,
            CEOCType rootType,
            CEOCType[] childTypes,
            CFeatures[] childsFeatures
            )
        {
            CEOC root = new CEOC(rootType);
            this.Graph.SetRoot(root);
            for (int i = 0; i < nbChilds; i++)
            {
                CEOC child = new CEOC(childTypes[i]);
                root.AddChild(child);
                child.Features = null;
                if (childsFeatures != null)
                {
                    if (childsFeatures[i] != null)
                    {
                        child.Features = childsFeatures[i].Clone();
                    }
                }
            }
        }

        public void LoadPatternSingle(CEOCType singleType)
        {
            CEOC root = new CEOC(singleType);
            this.Graph.SetRoot(root);
        }
        
        /*
        public void LoadPattern1Child(
            CEOCType rootType, CEOCType leafType,
            CFeatures leafFeatures
            )
        {
            CEOC root = new CEOC(rootType);
            this.Graph.SetRoot(root);
            CEOC leaf = new CEOC(leafType);
            root.AddChild(leaf);
            leaf.Features = leafFeatures;
        }

        /// <summary>
        /// Loads a 2 childs pattern with features for each child.
        /// </summary>
        /// <param name="rootType"></param>
        /// <param name="leftType"></param>
        /// <param name="rightType"></param>
        /// <param name="features">Distance between the 2 points</param>
        /// <param name="coeffs"></param>
        /// <param name="scores"></param>
        /// <param name="mode"></param>
        public void LoadPattern2Childs(
            CEOCType rootType, CEOCType leftType, CEOCType rightType,
            double[] spacing,
            double[][] coeffs,             // foreach proj -> coeffs for each dim
            double[] scores,                // scores foreach proj    
            CPattern.SetPointMode mode
            )
        {
            int nbDim = coeffs[0].Length;

            CEOC root = new CEOC(rootType);
            this.Graph.SetRoot(root);

            Vector p;

            CEOC left = new CEOC(leftType);
            p = new Vector(nbDim); // all set to 0.0
            left.Points.Add(p);
            root.AddChild(left);

            CEOC right = new CEOC(rightType);
            p = new Vector(nbDim);
            for (int i = 0; i < nbDim; i++) p[i] = spacing[i];
            right.Points.Add(p);
            root.AddChild(right);

            this.MatchingScores = scores;
            this.Projections = new List<Matrix>();
            for (int numProj = 0; numProj < scores.Length; numProj++)
            {
                Matrix m = Matrix.Identity(nbDim, nbDim);
                for (int numCoeff = 0; numCoeff < coeffs[numProj].Length; numCoeff++)
                {
                    m[numCoeff, numCoeff] = coeffs[numProj][numCoeff];
                }
                this.Projections.Add(m);
            }
            this._EnableTranslation = true;
            this._EnableScaling = false;
            this._EnableRotation = false;
            this._PointMode = mode;
        }

        /// <summary>
        /// Loads a 2 childs pattern with N points per child
        /// </summary>
        /// <param name="rootType"></param>
        /// <param name="leftType"></param>
        /// <param name="rightType"></param>
        /// <param name="leftPoints"></param>
        /// <param name="rightPoints"></param>
        /// <param name="coeffs"></param>
        /// <param name="scores"></param>
        /// <param name="mode"></param>
        public void LoadPattern2ChildsNPoints(
            CEOCType rootType, CEOCType leftType, CEOCType rightType,
            double[][] leftPoints, double[][] rightPoints,
            double[][] coeffs,             // foreach proj -> coeffs for each dim
            double[] scores,                // scores foreach proj    
            CPattern.SetPointMode mode
            )
        {
            int nbDim = coeffs[0].Length;

            CEOC root = new CEOC(rootType);
            this.Graph.SetRoot(root);

            Vector p;

            CEOC left = new CEOC(leftType);
            for (int numPoint = 0; numPoint < leftPoints.Length; numPoint++)
            {
                p = new Vector(leftPoints[numPoint]);
                left.Points.Add(p);
            }
            root.AddChild(left);

            CEOC right = new CEOC(rightType);
            for (int numPoint = 0; numPoint < rightPoints.Length; numPoint++)
            {
                p = new Vector(rightPoints[numPoint]);
                right.Points.Add(p);
            }
            root.AddChild(right);

            this.MatchingScores = scores;
            this.Projections = new List<Matrix>();
            for (int numProj = 0; numProj < scores.Length; numProj++)
            {
                Matrix m = Matrix.Identity(nbDim, nbDim);
                for (int numCoeff = 0; numCoeff < coeffs[numProj].Length; numCoeff++)
                {
                    m[numCoeff, numCoeff] = coeffs[numProj][numCoeff];
                }
                this.Projections.Add(m);
            }
            this._EnableTranslation = true;
            this._EnableScaling = false;
            this._EnableRotation = false;
            this._PointMode = mode;
        }
        */
    }
}
