﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Drawing.Imaging;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;

using MathNet.Numerics.LinearAlgebra;

namespace Agora.Model.OperatorLevel0
{
    public class CFindCC : COperatorExpand
    {
        public int BinarizationBasicThreshold = 150;
        public int BinarizationSauvolaThresholdWindow = 12;
        public float BinarizationSauvolaThresholdKValue = 0.5F;
        public float BinarizationSauvolaThresholdRValue = 128.0F;
        public bool BinarizationUseSauvola = false;
        public int RangeMapLookX = 1;
        public int RangeMapLookY = 1;
        public bool ExtendedBoundingBox = false;

        public CFindCC(CPattern p)
            : base(p)
        {
        }

        override public int GetHashCode()
        {
            return (
                BinarizationBasicThreshold.GetHashCode() ^
                BinarizationSauvolaThresholdWindow.GetHashCode() ^
                BinarizationSauvolaThresholdKValue.GetHashCode() ^
                BinarizationSauvolaThresholdRValue.GetHashCode() ^
                BinarizationUseSauvola.GetHashCode() ^
                RangeMapLookX.GetHashCode() ^
                RangeMapLookY.GetHashCode() ^
                ExtendedBoundingBox.GetHashCode()
                );
        }

        override public void Expand(CEOC eoc)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            Trace.WriteLine("Expand...", name);
            Bitmap binImage;

            FastBitmap fastBmp = new FastBitmap(eoc.GetImage());
            fastBmp.LockImage();
            byte[] bitmapHashCode = fastBmp.GetHashCode();
            fastBmp.UnlockImage();
            fastBmp = null;
            Guid guid = new Guid(bitmapHashCode);

            string id = base.ToString() + "." + this.GetHashCode().ToString() + "." + guid.ToString() + "." + eoc.BoundingBox.ToString();
            if (!CImageAnalysis.Cache.Exist(id))
            {
                Bitmap tmpImage = eoc.GetImage().Clone(eoc.BoundingBox, eoc.GetImage().PixelFormat);

                Trace.WriteLine("Grayscale...");
                Bitmap grayImage;
                grayImage = AForge.Imaging.Filters.Grayscale.CommonAlgorithms.BT709.Apply(tmpImage);
                Trace.WriteLine("Grayscale OK.");

                // CImageAnalysis.Cache.StoreInCache(id + "_debug01.jpg", grayImage);

                binImage = grayImage;
                AForge.Imaging.Filters.Threshold thresholdFilter =
                    new AForge.Imaging.Filters.Threshold(BinarizationBasicThreshold);
                AForge.Imaging.Filters.SauvolaThreshold sauvolaFilter =
                    new AForge.Imaging.Filters.SauvolaThreshold(
                        BinarizationSauvolaThresholdWindow,
                        BinarizationSauvolaThresholdKValue,
                        BinarizationSauvolaThresholdRValue);
                if (BinarizationUseSauvola == true)
                {
                    Trace.WriteLine("Sauvola Filter...");
                    sauvolaFilter.ApplyInPlace(binImage);
                    Trace.WriteLine("Sauvola OK.");
                }
                else
                {
                    Trace.WriteLine("Threshold Filter...");
                    thresholdFilter.ApplyInPlace(binImage);
                    Trace.WriteLine("Threshold OK.");
                }

                //CImageAnalysis.Cache.StoreInCache(id + "_debug02.jpg", binImage);

                AForge.Imaging.Filters.Closing closingFilter = new AForge.Imaging.Filters.Closing();
                AForge.Imaging.Filters.Opening openingFilter = new AForge.Imaging.Filters.Opening();
                //dilatationFilter.ApplyInPlace(binImage);
                //erosionFilter.ApplyInPlace(binImage);
                //closingFilter.ApplyInPlace(binImage);

                //AForge.Imaging.Filters.Opening openingFilter = new AForge.Imaging.Filters.Opening();
                //openingFilter.ApplyInPlace(binImage);

                //CImageAnalysis.Cache.StoreInCache(id + "_debug03.jpg", binImage);

                Trace.WriteLine("Invert...");
                AForge.Imaging.Filters.Invert invertFilter = new AForge.Imaging.Filters.Invert();
                invertFilter.ApplyInPlace(binImage);
                Trace.WriteLine("Invert OK.");
                //binImage.Save("debug00.jpg");
                CImageAnalysis.Cache.StoreInCache(id, binImage);
            }
            else
            {
                binImage = CImageAnalysis.Cache.LoadFromCache(id);
            }

            Trace.WriteLine("Detect CC...");
            AForge.Imaging.BlobCounter bc = new AForge.Imaging.BlobCounter();
            bc.ProcessImage(binImage);
            Trace.WriteLine("Detect CC OK.");

            Rectangle[] rects = bc.GetObjectsRectangles();
            Trace.WriteLine("Adding " + rects.Length.ToString() + " CC to graph...");
            CEOC cc = null;
            int saveRangeMapLookX = GLOBALDATA.RANGEMAP_LOOK_X;
            int saveRangeMapLookY = GLOBALDATA.RANGEMAP_LOOK_Y;
            GLOBALDATA.RANGEMAP_LOOK_X = RangeMapLookX;
            GLOBALDATA.RANGEMAP_LOOK_Y = RangeMapLookY;
            foreach (Rectangle r in rects)
            {
                Rectangle r2 = new Rectangle(
                        eoc.BoundingBox.X + r.X,
                        (ExtendedBoundingBox == false) ? eoc.BoundingBox.Y + r.Y : eoc.BoundingBox.Y,
                        r.Width,
                        (ExtendedBoundingBox == false) ? r.Height : eoc.BoundingBox.Height);
                cc = new CEOC(/*eoc.Graph,*/ eoc.GetImage(), r2, Pattern.PatternType);
                eoc.AddChild(cc);
                cc.Measure(CEOCType.EOCCC);
            }
            GLOBALDATA.RANGEMAP_LOOK_X = saveRangeMapLookX;
            GLOBALDATA.RANGEMAP_LOOK_Y = saveRangeMapLookY;

            eoc.ComputeFeatures();

            Trace.WriteLine("Adding CC OK.");

            Trace.WriteLine("OK.", name);
        }

        public override string ToString()
        {
            return base.ToString() +
                "BinarizationBasicThreshold=" + BinarizationBasicThreshold.ToString() + "." +
                "BinarizationSauvolaThresholdKValue=" + BinarizationSauvolaThresholdKValue.ToString() + "." +
                "BinarizationSauvolaThresholdRValue=" + BinarizationSauvolaThresholdRValue.ToString() + "." +
                "BinarizationSauvolaThresholdWindow=" + BinarizationSauvolaThresholdWindow.ToString() + "." +
                "BinarizationUseSauvola=" + BinarizationUseSauvola.ToString() + "." +
                "RangeMapLookX=" + RangeMapLookX.ToString() + "." +
                "RangeMapLookY=" + RangeMapLookY.ToString();
        }
    }
}
