﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using MathNet.Numerics.LinearAlgebra;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;

//TODO Comparator w/ Parent

namespace Agora.Model.OperatorLevel0
{
    public class COperatorInsert : COperator
    {
        public COperatorInsert(CPattern p)
            : base(p)
        {
        }

        protected override bool EvaluatePairing()
        {
            bool result = this.PairingEvaluator.EvaluatePairing(this.Pattern);
            return result;
        }

        // Modify graph structure : insert new eoc with pattern's type
        // 0. create new eoc with type = pattern's type
        // 1. graph's childs (for pairing BACKUP only) --> new eoc childs
        // 3. add new eoc to graph eoc childs
        // 4. compute boudingbox of new eoc
        // 5. compute points of new eoc
        override protected void UpdateGraph()
        {
            CEOC rootPattern = this.Pattern.Graph.GetRoot();
            CEOC eocGraph = rootPattern.PairedWith.Value;

            // 0. create new EOC
            CEOC newEOC = new CEOC(this.Pattern.PatternType);

            // 1. remove graph eoc childs (for pairing backup only)
            //    and add them to new EOC
            LinkedListNode<CEOC> childPattern = rootPattern.Childs.First;
            LinkedListNode<CEOC> childGraph;
            while (childPattern != null)
            {
                childGraph = childPattern.Value.PairedWith;
                // 2 possible paths toward EOC :
                // - either by actual node in the childs list
                // - or by the candidate node in the temporary candidates list
                // anyway, in each case, we must remove the ACTUAL node !
                eocGraph.RemoveChild(childGraph.Value.Node);
                newEOC.AddChild(childGraph.Value.Node);
                childPattern = childPattern.Next;
            }

            // 3. Add child "new eoc" to graph's eoc
            eocGraph.AddChild(newEOC);

            // 4. compute boundingbox of new eoc
            newEOC.ComputeBoundingBox();

            // 5. compute features of new eoc
            newEOC.FeatureModes = CFeatureMode.CloneArray(this.FeatureModes);
            newEOC.ComputeFeatures();
        }
    }
}
