﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;

namespace Agora.Model.OperatorLevel0.Pairing
{
    public class CFeatureValuesComparator
    {
        //public string Name { get; set; }
        public bool SignedDistance { get; set; }
        public double MaxDistance { get; set; }

        /// <summary>
        /// Default values : "DONTCARE" (ie false/MaxValue)
        /// </summary>
        public CFeatureValuesComparator()
        {
            //Name = String.Empty;
            SignedDistance = false;
            MaxDistance = Double.PositiveInfinity; // Double.MaxValue;
        }
        /*
        public CFeatureValuesComparator(string name)
        {
            //Name = String.Copy(name);
            SignedDistance = false;
            MaxDistance = Double.PositiveInfinity;
        }
        */
        public CFeatureValuesComparator(/*string name,*/ bool signeddistance, double maxdistance)
        {
            //Name = String.Copy(name);
            SignedDistance = signeddistance;
            MaxDistance = maxdistance;
        }
        public CFeatureValuesComparator Clone()
        {
            return new CFeatureValuesComparator(/*Name,*/ SignedDistance, MaxDistance);
        }
        public override string ToString()
        {
            string s = "{" + /*Name + "|" +*/ SignedDistance.ToString() + "|" + MaxDistance.ToString() + "}";
            return s;
        }
        public bool Compare(double x0, double x1)
        {
            bool result = false;
            double distance = x0 - x1;
            if (SignedDistance == false)
            {
                if (Math.Abs(distance) <= Math.Abs(MaxDistance))
                    result = true;
            }
            else
            {
                if (MaxDistance >= 0.0)
                {
                    if ((distance >= 0.0) && (distance <= MaxDistance))
                        result = true;
                }
                else
                {
                    if ((distance <= 0.0) && (distance >= MaxDistance))
                        result = true;
                }
            }
            return result;
        }
        public bool Compare(CFeatureValuesComparator fvc)
        {
            bool result =
                ((this.SignedDistance == fvc.SignedDistance) && (this.MaxDistance == fvc.MaxDistance)) ?
                true : false;
            return result;
        }
    }

    public class CFeatureMatrixComparator
    {
        /*
        public CFeatureValuesComparator[][] ComparisonMatrix { get; set;}
        */

        public CFeatureValuesComparator a0a1 { get; set; }
        public CFeatureValuesComparator a0b1 { get; set; }
        public CFeatureValuesComparator b0a1 { get; set; }
        public CFeatureValuesComparator b0b1 { get; set; }

        /// <summary>
        /// Uses default values (ie DONTCARE)
        /// </summary>
        public CFeatureMatrixComparator()
        {
            this.SetDefaultValues();
        }
        public CFeatureMatrixComparator(
            CFeatureValuesComparator a0a1,
            CFeatureValuesComparator a0b1,
            CFeatureValuesComparator b0a1,
            CFeatureValuesComparator b0b1)
        {
            /*
            ComparisonMatrix = new CFeatureValuesComparator[2][];
            ComparisonMatrix[0] = new CFeatureValuesComparator[2];
            ComparisonMatrix[0][0] = a0a1.Clone();
            ComparisonMatrix[0][1] = a0b1.Clone();
            ComparisonMatrix[1] = new CFeatureValuesComparator[2];
            ComparisonMatrix[1][0] = b0a1.Clone();
            ComparisonMatrix[1][1] = b0b1.Clone();
            */
            this.a0a1 = a0a1.Clone();
            this.a0b1 = a0b1.Clone();
            this.b0a1 = b0a1.Clone();
            this.b0b1 = b0b1.Clone();
        }
        public CFeatureMatrixComparator Clone()
        {
            return new CFeatureMatrixComparator(
                /*
                this.ComparisonMatrix[0][0],
                this.ComparisonMatrix[0][1],
                this.ComparisonMatrix[1][0],
                this.ComparisonMatrix[1][1]
                */
                this.a0a1, this.a0b1, this.b0a1, this.b0b1
                );
        }
        public void SetDefaultValues()
        {
            /*
            ComparisonMatrix = new CFeatureValuesComparator[2][];
            ComparisonMatrix[0] = new CFeatureValuesComparator[2];
            ComparisonMatrix[0][0] = new CFeatureValuesComparator("a0a1");
            ComparisonMatrix[0][1] = new CFeatureValuesComparator("a0b1");
            ComparisonMatrix[1] = new CFeatureValuesComparator[2];
            ComparisonMatrix[1][0] = new CFeatureValuesComparator("b0a1");
            ComparisonMatrix[1][1] = new CFeatureValuesComparator("b0b1");
            */
            a0a1 = new CFeatureValuesComparator();
            a0b1 = new CFeatureValuesComparator();
            b0a1 = new CFeatureValuesComparator();
            b0b1 = new CFeatureValuesComparator();
        }
        public override string ToString()
        {
            string result = "{";
            /*
            result += this.ComparisonMatrix[0][0].ToString() + ",";
            result += this.ComparisonMatrix[0][1].ToString() + ",";
            result += this.ComparisonMatrix[1][0].ToString() + ",";
            result += this.ComparisonMatrix[1][1].ToString() + ",";
            result += "}";
            */
            result += this.a0a1.ToString() + ",";
            result += this.a0b1.ToString() + ",";
            result += this.b0a1.ToString() + ",";
            result += this.b0b1.ToString() + ",";
            return result;
        }
        public bool Compare(CFeatureMatrixComparator m)
        {
            bool result = true;
            /*
            result &= ComparisonMatrix[0][0].Compare(m.ComparisonMatrix[0][0]);
            result &= ComparisonMatrix[0][1].Compare(m.ComparisonMatrix[0][1]);
            result &= ComparisonMatrix[1][0].Compare(m.ComparisonMatrix[1][0]);
            result &= ComparisonMatrix[1][1].Compare(m.ComparisonMatrix[1][1]);
            */
            result &= this.a0a1.Compare(m.a0a1);
            result &= this.a0b1.Compare(m.a0b1);
            result &= this.b0a1.Compare(m.b0a1);
            result &= this.b0b1.Compare(m.b0b1);
            return result;
        }
        public bool Compare(double a0, double b0, double a1, double b1)
        {
            bool result = true;
            //result &= ComparisonMatrix[0][0].Compare(a0, a1);
            result &= this.a0a1.Compare(a0, a1);
            if (result == false) return result;
            //result &= ComparisonMatrix[0][1].Compare(a0, b1);
            result &= this.a0b1.Compare(a0, b1);
            if (result == false) return result;
            //result &= ComparisonMatrix[1][0].Compare(b0, a1);
            result &= this.b0a1.Compare(b0, a1);
            if (result == false) return result;
            //result &= ComparisonMatrix[1][1].Compare(b0, b1);
            result &= this.b0b1.Compare(b0, b1);
            return result;
        }
    }

    public class CFeaturesComparator
    {
        public CFeatureMatrixComparator[] Comparators { get; set; }
        public CFeaturesComparator()
        {
        }
        /// <summary>
        /// Builds a n-list of DONTCARE comparison criteria
        /// </summary>
        /// <param name="n"></param>
        public CFeaturesComparator(int nbFeatures)
        {
            Comparators = new CFeatureMatrixComparator[nbFeatures];
            for (int i = 0; i < nbFeatures; i++)
            {
                Comparators[i] = new CFeatureMatrixComparator();
            }
        }
        public CFeaturesComparator(CFeatureMatrixComparator[] comparators)
        {
            int nbFeatures = comparators.Length;
            Comparators = new CFeatureMatrixComparator[nbFeatures];
            for (int i = 0; i < nbFeatures; i++)
            {
                Comparators[i] = comparators[i].Clone();
            }
        }
        public CFeaturesComparator Clone()
        {
            CFeaturesComparator result = new CFeaturesComparator(this.Comparators);
            return result;
        }

        static public CFeaturesComparator[] CloneArray(CFeaturesComparator[] array)
        {
            CFeaturesComparator[] result = null;
            if (array != null)
            {
                result = new CFeaturesComparator[array.Length];
                for (int i = 0; i < array.Length; i++)
                {
                    result[i] = array[i].Clone();
                }
            }
            return result;
        }

        static public CFeaturesComparator[][] CloneArrayArray(CFeaturesComparator[][] array)
        {
            CFeaturesComparator[][] result = null;
            if (array != null)
            {
                result = new CFeaturesComparator[array.Length][];
                for (int i = 0; i < array.Length; i++)
                {
                    result[i] = CFeaturesComparator.CloneArray(array[i]);
                }
            }
            return result;
        }

        public override string ToString()
        {
            string result = "{";
            for (int i = 0; i < this.Comparators.Length; i++)
            {
                result += i.ToString() + "=" + this.Comparators[i].ToString() + ",";
            }
            result += "}";
            return result;
        }
        /// <summary>
        /// Build an array of nbComparators comparators with nbFeatures each.
        /// Default values = DONTCARE everywhere
        /// </summary>
        /// <param name="nbComparators"></param>
        /// <param name="nbFeatures"></param>
        /// <returns></returns>
        static public CFeaturesComparator[] BuildArray(int nbComparators, int nbFeatures)
        {
            CFeaturesComparator[] a = new CFeaturesComparator[nbComparators];
            for (int i = 0; i < nbComparators; i++)
            {
                a[i] = new CFeaturesComparator(nbFeatures);
            }
            return a;
        }
        /// <summary>
        /// Build a matrix (array of arrays) of nbElem X nbElem comparators
        /// with nbFeatures each.
        /// Default values = DONTCARE everywhere
        /// </summary>
        /// <param name="nbElements"></param>
        /// <param name="nbFeatures"></param>
        /// <returns></returns>
        static public CFeaturesComparator[][] BuildMatrix(int nbElements, int nbFeatures)
        {
            CFeaturesComparator[][] m = new CFeaturesComparator[nbElements][];
            for (int i = 0; i < nbElements; i++)
            {
                m[i] = CFeaturesComparator.BuildArray(nbElements, nbFeatures);
            }
            return m;
        }
        public bool Compare(CFeatures f0, CFeatures f1)
        {
            bool result = true;
            for (int i = 0; i < f0.a.Length; i++)
            {
                result &= Comparators[i].Compare(f0.a[i], f0.b[i], f1.a[i], f1.b[i]);
                if (result == false) break;
            }
            return result;
        }
    }
}
