﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;

namespace Agora.Model.OperatorLevel1
{
    public class CDelete : Agora.Model.Project.CStepOperator
    {
        public string EOCTypeParent { get; set; }
        public string EOCTypeChild { get; set; }
        public bool DeleteAll { get; set; }
        public bool Loop { get; set; }

        public CDelete()
        {
            EOCTypeParent = String.Empty;
            EOCTypeChild = String.Empty;
            DeleteAll = false;
            Loop = true;
        }

        public CDelete(string eocTypeParent, string eocTypeChild, bool deleteAll, bool loop)
        {
            EOCTypeParent = String.Copy(eocTypeParent);
            EOCTypeChild = String.Copy(eocTypeChild);
            DeleteAll = deleteAll;
            Loop = loop;
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CDelete(EOCTypeParent, EOCTypeChild, DeleteAll, Loop);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = false;
            CPattern p;
            OperatorLevel0.COperator op;
            
            if (Trace.IndentLevel == 0) Trace.WriteLine("Delete child " + EOCTypeChild + "...", name);
            Trace.Indent();

            p = new CPattern(null);
            p.LoadPatternNChilds(1,
                analysis.GetEOCTypeByName(EOCTypeParent),
                new CEOCType[] { analysis.GetEOCTypeByName(EOCTypeChild) },
                null);

            op = new COperatorDelete(p, DeleteAll);

            op.PairingEvaluator = null;
            op.FeatureModes = null;

            bool lastResult;
            do
            {
                lastResult = op.Apply(analysis.Graph.GetRoot());
                result |= lastResult;
            } while ((lastResult == true) && (Loop == true));

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("Delete child OK.", name);

            return result;
        }
        public override string ToString()
        {
            return "Delete:" +
                "Parent=" + EOCTypeParent + "," +
                "Child=" + EOCTypeChild + "," +
                "DeleteAll=" + DeleteAll.ToString() + "," +
                "Loop=" + Loop.ToString();
        }
    }
}
