﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;

namespace Agora.Model.OperatorLevel1
{
    public class CFindCC : Agora.Model.Project.CStepOperator
    {
        public string EOCTypeParent { get; set; }
        public string EOCTypeNewCC { get; set; }
        public int BinarizationBasicThreshold { get; set; }
        public int BinarizationSauvolaThresholdWindow { get; set; }
        public float BinarizationSauvolaThresholdKValue { get; set; }
        public float BinarizationSauvolaThresholdRValue { get; set; }
        public bool BinarizationUseSauvola { get; set; }
        public int RangeMapLookX { get; set; }
        public int RangeMapLookY { get; set; }
        public bool ExtendedBoundingBox { get; set; }

        private OperatorLevel0.CFindCC _FindCC = null;

        public CFindCC()
        {
            BinarizationBasicThreshold = 150;
            BinarizationSauvolaThresholdWindow = 12;
            BinarizationSauvolaThresholdKValue = 0.5F;
            BinarizationSauvolaThresholdRValue = 128.0F;
            BinarizationUseSauvola = false;
            RangeMapLookX = 1;
            RangeMapLookY = 1;
            ExtendedBoundingBox = false;
        }
        public CFindCC(string eocTypeParent, string eocTypeNewCC,
            int binarizationBasicThreshold,
            int binarizationSauvolaThresholdWindow,
            float binarizationSauvolaThresholdKValue, float binarizationSauvolaThresholdRValue,
            bool binarizationUseSauvola,
            int rangeMapLookX, int rangeMapLookY,
            bool extendedBoundingBox)
        {
            this.EOCTypeParent = String.Copy(eocTypeParent);
            this.EOCTypeNewCC = String.Copy(eocTypeNewCC);
            this.BinarizationBasicThreshold = binarizationBasicThreshold;
            this.BinarizationSauvolaThresholdWindow = binarizationSauvolaThresholdWindow;
            this.BinarizationSauvolaThresholdKValue = binarizationSauvolaThresholdKValue;
            this.BinarizationSauvolaThresholdRValue = binarizationSauvolaThresholdRValue;
            this.BinarizationUseSauvola = binarizationUseSauvola;
            this.RangeMapLookX = rangeMapLookX;
            this.RangeMapLookY = rangeMapLookY;
            this.ExtendedBoundingBox = extendedBoundingBox;
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CFindCC(EOCTypeParent, EOCTypeNewCC,
                BinarizationBasicThreshold,
                BinarizationSauvolaThresholdWindow,
                BinarizationSauvolaThresholdKValue, BinarizationSauvolaThresholdRValue,
                BinarizationUseSauvola,
                RangeMapLookX, RangeMapLookY,
                ExtendedBoundingBox);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result;
            CPattern p;
            
            if (Trace.IndentLevel == 0) Trace.WriteLine("Find CC...", name);
            Trace.Indent();

            Trace.Assert(analysis.GetEOCTypeByName(EOCTypeNewCC) != null);
            Trace.Assert(analysis.GetEOCTypeByName(EOCTypeParent) != null);
            p = new CPattern(analysis.GetEOCTypeByName(EOCTypeNewCC));
            p.LoadPatternSingle(analysis.GetEOCTypeByName(EOCTypeParent));
            _FindCC = new OperatorLevel0.CFindCC(p);
            _FindCC.BinarizationBasicThreshold = this.BinarizationBasicThreshold;
            _FindCC.BinarizationSauvolaThresholdKValue = this.BinarizationSauvolaThresholdKValue;
            _FindCC.BinarizationSauvolaThresholdRValue = this.BinarizationSauvolaThresholdRValue;
            _FindCC.BinarizationSauvolaThresholdWindow = this.BinarizationSauvolaThresholdWindow;
            _FindCC.BinarizationUseSauvola = this.BinarizationUseSauvola;
            _FindCC.RangeMapLookX = this.RangeMapLookX;
            _FindCC.RangeMapLookY = this.RangeMapLookY;
            _FindCC.ExtendedBoundingBox = this.ExtendedBoundingBox;
            result = _FindCC.Apply(analysis.Graph.GetRoot());

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("Find CC OK.", name);

            return result;
        }
        public override string ToString()
        {
            return "FindCC:" +
                "Parent=" + EOCTypeParent + "," +
                "New=" + EOCTypeNewCC + "," +
                "BinarizationBasicThreshold=" + BinarizationBasicThreshold.ToString() + "," +
                "BinarizationSauvolaThresholdKValue=" + BinarizationSauvolaThresholdKValue.ToString() + "," +
                "BinarizationSauvolaThresholdRValue=" + BinarizationSauvolaThresholdRValue.ToString() + "," +
                "BinarizationSauvolaThresholdWindow=" + BinarizationSauvolaThresholdWindow.ToString() + "," +
                "BinarizationUseSauvola=" + BinarizationUseSauvola.ToString() +
                "RangeMapLookX=" + RangeMapLookX.ToString() + "," +
                "RangeMapLookY=" + RangeMapLookY.ToString() + "," +
                "ExtendedBB=" + ExtendedBoundingBox.ToString();
        }
    }
}
