﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;

namespace Agora.Model.OperatorLevel1
{
    public class CInsert : Agora.Model.Project.CStepOperator
    {
        public string EOCTypeParent { get; set; }
        public string[] EOCTypeChilds { get; set; }
        public string EOCTypeNewEOC { get; set; }

        /// <summary>
        /// Features of childs (pattern)
        /// </summary>
        public CFeatures[] Features { get; set; }

        public CPairingEvaluator Evaluator { get; set; }

        public double[] Neighbouroud { get; set; }
        public CFeatureMode[] Modes { get; set; }
        public bool Loop { get; set; }

        public CInsert()
        {
            EOCTypeParent = String.Empty;
            EOCTypeChilds = null;
            EOCTypeNewEOC = String.Empty;
            Features = null;
            Evaluator = new CPairingEvaluator();
            Neighbouroud = null;
            Modes = null;
            Loop = true;
        }
        public CInsert(
            string eoctypeparent, string[] eoctypechilds, string eoctypeneweoc,
            CFeatures[] features,
            CPairingEvaluator evaluator,
            double[] neighbouroud, CFeatureMode[] modes, bool loop)
        {

            EOCTypeParent = String.Copy(eoctypeparent);
            EOCTypeChilds = new string[eoctypechilds.Length];
            for (int i = 0; i < eoctypechilds.Length; i++)
                EOCTypeChilds[i] = String.Copy(eoctypechilds[i]);
            EOCTypeNewEOC = String.Copy(eoctypeneweoc);

            Features = CFeatures.CloneArray(features);
            Evaluator = evaluator.Clone();

            Neighbouroud = null;
            if (neighbouroud != null)
            {
                Neighbouroud = (double[])neighbouroud.Clone();
            }

            Modes = CFeatureMode.CloneArray(modes);
            Loop = loop;
        }

        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CInsert(
                EOCTypeParent, EOCTypeChilds, EOCTypeNewEOC,
                Features, 
                Evaluator,
                Neighbouroud, Modes, Loop);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = false;
            CPattern p;
            OperatorLevel0.COperator op;
            string s = "Insert : " + EOCTypeNewEOC + " <- { ";
            for (int i = 0; i < EOCTypeChilds.Length; i++)
            {
                s = s + EOCTypeChilds[i] + ((i == EOCTypeChilds.Length-1) ? "" : " + ");
            }
            s = s + " }";
            if (Trace.IndentLevel == 0) Trace.WriteLine(s, name);
            Trace.Indent();

            Trace.Assert(analysis.GetEOCTypeByName(EOCTypeParent) != null);

            CEOCType[] childTypes = new CEOCType[EOCTypeChilds.Length];
            for (int i = 0; i < EOCTypeChilds.Length; i++)
            {
                Trace.Assert(analysis.GetEOCTypeByName(EOCTypeChilds[i]) != null);
                childTypes[i] = analysis.GetEOCTypeByName(EOCTypeChilds[i]);
            }

            p = new CPattern(analysis.GetEOCTypeByName(EOCTypeNewEOC));
            p.LoadPatternNChilds(
                EOCTypeChilds.Length,
                analysis.GetEOCTypeByName(EOCTypeParent),
                childTypes,
                Features);

            op = new COperatorInsert(p);
            op.FeatureModes = Modes;
            op.PairingEvaluator = Evaluator;

            if (Neighbouroud != null)
            {
                op.EnableNeighbourhood = true;
                op.Neighbourhood = Neighbouroud;
            }

            bool lastResult;
            do
            {
                lastResult = op.Apply(analysis.Graph.GetRoot());
                result |= lastResult;
            }
            while ((lastResult == true) && (Loop == true));

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("Insert OK.", name);

            return result;
        }
        public override string ToString()
        {
            string childsFeatures = "{";
            if (Features != null)
            {
                foreach (CFeatures f in this.Features)
                {
                    childsFeatures += f.ToString() + ",";
                }
            }
            childsFeatures += "}";

            string evaluator = Evaluator.ToString();

            string childsEOCTypes = "{";
            foreach (string type in EOCTypeChilds)
            {
                childsEOCTypes += type + ",";
            }
            childsEOCTypes += "}";

            string neigh = "{";
            if (Neighbouroud != null)
            {
                foreach (double d in Neighbouroud)
                {
                    neigh = neigh + d.ToString() + ",";
                }
            }
            neigh += "}";
            
            string modes = "{";
            foreach (CFeatureMode m in Modes)
            {
                modes += m.ToString() + ",";
            }
            modes += "}";
            
            return "Insert:" +
                "Parent=" + EOCTypeParent + "," +
                "Childs=" + childsEOCTypes + "," +
                "New=" + EOCTypeNewEOC + "," +
                "Features=" + childsFeatures + "," +
                evaluator + "," +
                "Neighbouroud=" + neigh + "," +
                "Modes=" + modes +
                "Loop=" + Loop.ToString() +
                "";
            ;
        }
    }
}
