﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;

namespace Agora.Model.OperatorLevel1
{
    public class CSetFeatures : Agora.Model.Project.CStepOperator
    {
        public string EOCTypeParent { get; set; }
        public string EOCTypeChild { get; set; }
        public CFeatureMode[] Modes { get; set; }
        public bool Loop { get; set; }

        public CSetFeatures()
        {
            EOCTypeParent = String.Empty;
            EOCTypeChild = String.Empty;
            Modes = null;
            Loop = true;
        }
        public CSetFeatures(
            string eocTypeParent, string eocTypeChild,
            CFeatureMode[] modes, bool loop)
        {
            EOCTypeParent = String.Copy(eocTypeParent);
            EOCTypeChild = String.Copy(eocTypeChild);
            Modes = CFeatureMode.CloneArray(modes);
            Loop = loop;
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CSetFeatures(EOCTypeParent, EOCTypeChild, Modes, Loop);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = false;
            CPattern p;
            OperatorLevel0.COperator op;
            
            if (Trace.IndentLevel == 0) Trace.WriteLine(name);
            Trace.Indent();

            p = new CPattern(analysis.GetEOCTypeByID(CEOCType.EOCNoType));
            p.LoadPatternNChilds(1,
                analysis.GetEOCTypeByName(EOCTypeParent),
                new CEOCType[] { analysis.GetEOCTypeByName(EOCTypeChild) },
                null);

            op = new COperatorSetFeatures(p);
            op.PairingEvaluator = null;
            op.FeatureModes = this.Modes;

            bool lastResult;
            do
            {
                lastResult = op.Apply(analysis.Graph.GetRoot());
                result |= lastResult;
            }
            while ((lastResult == true) && (Loop == true));

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("SetFeatures OK.", name);

            return result;
        }
        public override string ToString()
        {
            string modes = "{";
            foreach (CFeatureMode m in Modes)
            {
                modes += m.ToString() + ",";
            }
            modes += "}";

            return "SetFeatures:" +
                "Parent=" + EOCTypeParent + "," +
                "Child=" + EOCTypeChild + "," +
                "Mode=" + modes + "," +
                "Loop=" + Loop.ToString();
            ;
        }
    }
}
