﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;

namespace Agora.Model.OperatorLevel2
{
    public class CFusion : Agora.Model.Project.CStepOperator
    {
        /// <summary>
        /// Context
        /// </summary>
        public string EOCTypeParent { get; set; }

        /// <summary>
        /// Childs type to fusion
        /// </summary>
        public string EOCTypeChilds { get; set; }

        /// <summary>
        /// Type of resulting EOC
        /// </summary>
        public string EOCTypeNewEOC { get; set; }

        /// <summary>
        /// Comparator between 2 childs (graph/graph)
        /// </summary>
        public CFeaturesComparator Comparator { get; set; }

        public double[] Neighbouroud { get; set; }
        public CFeatureMode[] Modes { get; set; }

        public CFusion()
        {
        }
        public CFusion(
            string eoctypeparent, string eoctypechilds, string eoctypeneweoc,
            CFeaturesComparator comparator,
            double[] neighbouroud, CFeatureMode[] modes)
        {

            EOCTypeParent = String.Copy(eoctypeparent);
            EOCTypeChilds = String.Copy(eoctypechilds);
            EOCTypeNewEOC = String.Copy(eoctypeneweoc);

            Comparator = comparator.Clone();

            Neighbouroud = null;
            if (neighbouroud != null)
            {
                Neighbouroud = (double[])neighbouroud.Clone();
            }
            Modes = CFeatureMode.CloneArray(modes);
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CFusion(
                EOCTypeParent, EOCTypeChilds, EOCTypeNewEOC,
                Comparator, Neighbouroud, Modes);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = false;

            int nbDim = analysis.GetDimensions().Length;

            OperatorLevel1.CInsert opIns;
            OperatorLevel1.CDelete opDel;

            if (EOCTypeChilds != EOCTypeNewEOC)
            {
                if (Trace.IndentLevel == 0) Trace.WriteLine("Phase 1 : " + EOCTypeChilds + "->" + EOCTypeNewEOC + "...", name);
                Trace.Indent();

                opIns = new CInsert(
                    EOCTypeParent, new string[] { EOCTypeChilds }, EOCTypeNewEOC,
                    new CFeatures[] { new CFeatures(new double[nbDim], new double[nbDim]) },
                    new CPairingEvaluator(
                        false, false, false,
                        new double[][] { new double[nbDim] },
                        CFeaturesComparator.BuildArray(1, nbDim), null, null),
                    null, Modes, true);
                result = opIns.Apply(analysis);

                Trace.Unindent();
                if (Trace.IndentLevel == 0) Trace.WriteLine("Phase 1 : OK.", name);

                if (result == false) return result;
            }

            if (Trace.IndentLevel == 0) Trace.WriteLine("Phase 2 : Insertion & simplification...", name);
            Trace.Indent();

            CFeaturesComparator[][] fc = CFeaturesComparator.BuildMatrix(2, nbDim);
            fc[0][1] = Comparator;

            opIns = new CInsert(
            EOCTypeParent, new string[] { EOCTypeNewEOC, EOCTypeNewEOC }, EOCTypeNewEOC,
            new CFeatures[] {
                new CFeatures(new double[nbDim], new double[nbDim]),
                new CFeatures(new double[nbDim], new double[nbDim]),
                },
            new CPairingEvaluator(
                false, false, false,
                new double[][] { new double[nbDim] },
                null, null, fc),
            Neighbouroud, Modes, true);

            opDel = new CDelete(EOCTypeNewEOC, EOCTypeNewEOC, false, true);

            bool lastResult;
            do
            {
                lastResult = opIns.Apply(analysis);
                if (lastResult == true)
                {
                    lastResult = opDel.Apply(analysis);
                }
                result |= lastResult;
            } while (lastResult == true);

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("Phase 2 : OK.", name);

            return result;
        }
        public override string ToString()
        {
            string neigh = "{";
            if (Neighbouroud != null)
            {
                foreach (double d in Neighbouroud)
                {
                    neigh = neigh + d.ToString() + ",";
                }
            }
            neigh += "}";

            string modes = "{";
            foreach (CFeatureMode m in Modes)
            {
                modes += m.ToString() + ",";
            }
            modes += "}";

            return "Fusion:" +
                "Parent=" + EOCTypeParent + "," +
                "Childs=" + EOCTypeChilds + "," +
                "New=" + EOCTypeNewEOC + "," +
                "Comparator=" + Comparator.ToString() + "," +
                "Modes=" + modes +
                "Neighbouroud=" + neigh + "," +
                "";
            ;
        }
    }
}
