﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;

namespace Agora.Model.OperatorLevel2
{
    public class CIntersect : Agora.Model.Project.CStepOperator
    {
        /// <summary>
        /// Context
        /// </summary>
        public string EOCTypeParent { get; set; }

        /// <summary>
        /// Childs types to check intersection
        /// </summary>
        public string[] EOCTypeChilds { get; set; }

        /// <summary>
        /// Type of resulting EOC
        /// </summary>
        public string EOCTypeNewEOC { get; set; }

        /// <summary>
        /// N projection matrix 
        /// dim0 = nb of proj (N)
        /// dim1 = coefficients (= nb of dim, only diagonal values)
        /// </summary>
        public double[][] Coefficients { get; set; }

        public double[] Neighbouroud { get; set; }
        public CFeatureMode[] Modes { get; set; }

        public CIntersect()
        {
        }
        public CIntersect(
            string eoctypeparent, string[] eoctypechilds, string eoctypeneweoc,
            double[][] coefficients,
            double[] neighbouroud, CFeatureMode[] modes)
        {

            EOCTypeParent = String.Copy(eoctypeparent);
            EOCTypeChilds = new string[eoctypechilds.Length];
            for (int i = 0; i < eoctypechilds.Length; i++)
                EOCTypeChilds[i] = String.Copy(eoctypechilds[i]);
            EOCTypeNewEOC = String.Copy(eoctypeneweoc);

            Coefficients = CPairingEvaluator.CloneArrayArray(coefficients);

            Neighbouroud = null;
            if (neighbouroud != null)
            {
                Neighbouroud = (double[])neighbouroud.Clone();
            }

            Modes = CFeatureMode.CloneArray(modes);
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CIntersect(
                EOCTypeParent, EOCTypeChilds, EOCTypeNewEOC,
                Coefficients, Neighbouroud, Modes);
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = false;

            int nbDim = analysis.GetDimensions().Length;

            OperatorLevel1.CInsert opIns;
            OperatorLevel1.CDelete opDel;

            if (Trace.IndentLevel == 0) Trace.WriteLine(EOCTypeChilds[0] + "+" + EOCTypeChilds[1] + "->" + EOCTypeNewEOC + "...", name);
            Trace.Indent();

            bool simplification = false;
            if (EOCTypeChilds[0].Contains(EOCTypeNewEOC))
            {
                Trace.WriteLine("RECURSIVE intersection detected --> Simplification ON.", name);
                simplification = true;
            }

            opIns = new CInsert(
                EOCTypeParent, EOCTypeChilds, EOCTypeNewEOC,
                CFeatures.BuildArray(EOCTypeChilds.Length, nbDim),
                new CPairingEvaluatorIntersection(Coefficients),
                null, Modes, true);
            opDel = new CDelete(EOCTypeNewEOC, EOCTypeNewEOC, false, true);

            bool lastResult;
            do
            {
                lastResult = opIns.Apply(analysis);
                if ((lastResult == true) && (simplification == true))
                {
                    lastResult = opDel.Apply(analysis);
                }
                result |= lastResult;
            } while (lastResult == true);

            Trace.Unindent();
            if (Trace.IndentLevel == 0) Trace.WriteLine("OK.", name);

            return result;
        }
        public override string ToString()
        {
            string childsEOCTypes = "{";
            foreach (string type in EOCTypeChilds)
            {
                childsEOCTypes += type + ",";
            }
            childsEOCTypes += "}";

            string neigh = "{";
            if (Neighbouroud != null)
            {
                foreach (double d in Neighbouroud)
                {
                    neigh = neigh + d.ToString() + ",";
                }
            }
            neigh += "}";

            string modes = "{";
            foreach (CFeatureMode m in Modes)
            {
                modes += m.ToString() + ",";
            }
            modes += "}";

            string coefs = "{";
            foreach (double[] coef in Coefficients)
            {
                coefs += "{";
                foreach (double d in coef)
                {
                    coefs = coefs + d.ToString() + ",";
                }
                coefs += "},";
            }
            coefs += "}";

            return "Intersection:" +
                "Parent=" + EOCTypeParent + "," +
                "Childs=" + childsEOCTypes + "," +
                "New=" + EOCTypeNewEOC + "," +
                "Coefficients=" + coefs + "," +
                "Modes=" + modes +
                "Neighbouroud=" + neigh + "," +
                "";
            ;
        }
    }
}
