﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;
using Agora.Model.Utils;

namespace Agora.Model.OperatorLevel3
{
    public class CEuclidianFeatures : Agora.Model.Project.CStepOperator
    {

        /// <summary>
        /// Context
        /// </summary>
        public string EOCTypeParent { get; set; }

        /// <summary>
        /// Type of the only one child
        /// </summary>
        public string EOCTypeChild { get; set; }

        /// <summary>
        /// Type of the new child if classification has to be done
        /// </summary>
        public string EOCTypeNewEOC { get; set; }

        public bool UseForm { get; set; }
        public bool UsePosition { get; set; }
        public bool UseSize { get; set; }
        
        /// <summary>
        /// Position child's XY features
        /// </summary>
        public double PositionXMin { get; set; }
        public double PositionXMax { get; set; }
        public double PositionYMin { get; set; }
        public double PositionYMax { get; set; }

        /// <summary>
        /// Size child's XY features
        /// </summary>
        public enum SizeClassificationType { Maximum, Minimum, Exact }

        public SizeClassificationType SizeType { get; set; }
        public double SizeX { get; set; }
        public double SizeXTolerance { get; set; }
        public double SizeY { get; set; }
        public double SizeYTolerance { get; set; }

        /// <summary>
        /// Form child's XY features
        /// </summary>
        public double FormXMin { get; set; }
        public double FormXMax { get; set; }
        public double FormYMin { get; set; }
        public double FormYMax { get; set; }

        /// <summary>
        /// If new EOC is inserted, then replace child EOC
        /// instead of just insert it. This will call "Delete"
        /// </summary>
        public bool Replace { get; set; }

        public CEuclidianFeatures()
        {
            EOCTypeChild = null;
            EOCTypeNewEOC = null;
            EOCTypeParent = null;
            UseForm = false;
            UsePosition = false;
            UseSize = false;
            FormXMin = 0;
            FormXMax = 0;
            FormYMin = 0;
            FormYMax = 0;
            PositionXMin = 0;
            PositionXMax = 0;
            PositionYMin = 0;
            PositionYMax = 0;
            SizeType = SizeClassificationType.Exact;
            SizeX = 0;
            SizeXTolerance = 0;
            SizeY = 0;
            SizeYTolerance = 0;
            Replace = false;
        }

        public CEuclidianFeatures(
            string eocTypeParent, string eocTypeChild, string eocTypeNewEOC,
            bool useForm, bool usePosi, bool useSize,
            double formxmin, double formxmax, double formymin, double formymax,
            double posixmin, double posixmax, double posiymin, double posiymax,
            SizeClassificationType sizetype, double sizex, double sizextol, double sizey, double sizeytol,
            bool replace
            )
        {
            EOCTypeParent = String.Copy(eocTypeParent);
            EOCTypeChild = String.Copy(eocTypeChild);
            EOCTypeNewEOC = String.Copy(eocTypeNewEOC);
            UseForm = useForm;
            UsePosition = usePosi;
            UseSize = useSize;
            FormXMin = formxmin;
            FormXMax = formxmax;
            FormYMin = formymin;
            FormYMax = formymax;
            PositionXMin = posixmin;
            PositionXMax = posixmax;
            PositionYMin = posiymin;
            PositionYMax = posiymax;
            SizeType = sizetype;
            SizeX = sizex;
            SizeXTolerance = sizextol;
            SizeY = sizey;
            SizeYTolerance = sizeytol;
            Replace = replace;
        }

        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CEuclidianFeatures(
                EOCTypeParent, EOCTypeChild, EOCTypeNewEOC,
                UseForm, UsePosition, UseSize,
                FormXMin, FormXMax, FormYMin, FormYMax,
                PositionXMin, PositionXMax, PositionYMin, PositionYMax,
                SizeType, SizeX, SizeXTolerance, SizeY, SizeYTolerance,
                Replace);
        }

        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result;
            Trace.WriteLine("Euclidian features...", name);

            int nbDim = analysis.GetDimensions().Length;
            int euclidian_X_num = 0;
            int euclidian_Y_num = 1;

            double[][] coefficients = CPairingEvaluator.CreateStandardProjection(nbDim);
            for (int i = 0; i < nbDim; i++)
            {
                if (analysis.GetDimensions()[i] == CDimensionType.EUCLIDIAN_X)
                {
                    euclidian_X_num = i;
                }
                else if (analysis.GetDimensions()[i] == CDimensionType.EUCLIDIAN_Y)
                {
                    euclidian_Y_num = i;
                }
                else
                {
                    coefficients[0][i] = 0.0;
                }
            }

            result = false;

            Trace.WriteLine("Phase 0 : Temporary labels creations...", name);
            Trace.Indent();

            string LabelNameForm = "LABELFORM_" + Guid.NewGuid().ToString().Substring(0,4);
            string LabelNamePosi = "LABELPOSI_" + Guid.NewGuid().ToString().Substring(0, 4);
            string LabelNameSize = "LABELSIZE_" + Guid.NewGuid().ToString().Substring(0, 4);

            LabelNameForm = LabelNameForm.ToUpper();
            LabelNamePosi = LabelNamePosi.ToUpper();
            LabelNameSize = LabelNameSize.ToUpper();

            OperatorMisc.CCreateLabel opCreate = new Agora.Model.OperatorMisc.CCreateLabel();

            opCreate.EOCType = 0;
            opCreate.EOCTypeName = LabelNameForm;
            opCreate.Apply(analysis);
            opCreate.EOCType = 0;
            opCreate.EOCTypeName = LabelNamePosi;
            opCreate.Apply(analysis);
            opCreate.EOCType = 0;
            opCreate.EOCTypeName = LabelNameSize;
            opCreate.Apply(analysis);

            OperatorLevel1.CDelete opDel;
            OperatorLevel2.CClassifyByFeatures opClassify;

            Trace.Unindent();
            Trace.WriteLine("Phase 0 : OK.", name);

            Trace.WriteLine("Phase 1 : Classification by Position...", name);
            Trace.Indent();

            bool resultPosi = false;
            if (UsePosition == true)
            {
                CFeatures featPosi = new CFeatures(nbDim);
                CFeaturesComparator compPosi = new CFeaturesComparator(nbDim);
                featPosi.a[euclidian_X_num] = PositionXMin;
                featPosi.b[euclidian_X_num] = PositionXMax;
                featPosi.a[euclidian_Y_num] = PositionYMin;
                featPosi.b[euclidian_Y_num] = PositionYMax;

                // Set comparators for CDimensionType.EUCLIDIAN_X and CDimensionType.EUCLIDIAN_Y
                for (int i = 0; i < nbDim; i++)
                {
                    if ((i == euclidian_X_num) || (i == euclidian_Y_num))
                    {
                        // a0,b0=pattern=position of zone, a1,b1=eoc
                        // if eoc is included in zone then we want a0<a1<b0 and a0<b1<b0
                        compPosi.Comparators[i].a0a1.SignedDistance = true;
                        compPosi.Comparators[i].a0a1.MaxDistance = Double.NegativeInfinity;
                        compPosi.Comparators[i].b0a1.SignedDistance = true;
                        compPosi.Comparators[i].b0a1.MaxDistance = Double.PositiveInfinity;
                        compPosi.Comparators[i].a0b1.SignedDistance = true;
                        compPosi.Comparators[i].a0b1.MaxDistance = Double.NegativeInfinity;
                        compPosi.Comparators[i].b0b1.SignedDistance = true;
                        compPosi.Comparators[i].b0b1.MaxDistance = Double.PositiveInfinity;
                    }
                }

                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, EOCTypeChild, LabelNamePosi,
                    featPosi, coefficients, compPosi,
                    CClassifyByFeatures.ClassificationType.ClassifyByPosition,
                    false);
                resultPosi = opClassify.Apply(analysis);
            }
            else
            {
                // Inserts new EOC for ALL matching childs
                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, EOCTypeChild, LabelNamePosi,
                    new CFeatures(nbDim), // default features
                    coefficients,
                    new CFeaturesComparator(nbDim), // default comparators
                    CClassifyByFeatures.ClassificationType.ClassifyByPosition,
                    false);
                resultPosi = opClassify.Apply(analysis);
            }


            Trace.Unindent();
            Trace.WriteLine("Phase 1 : OK.", name);

            Trace.WriteLine("Phase 2 : Classification by Size...", name);
            Trace.Indent();

            bool resultSize = false;
            if (UseSize == true)
            {
                CFeatures featSize = new CFeatures(nbDim);
                CFeaturesComparator compSize = new CFeaturesComparator(nbDim);
                featSize.a[euclidian_X_num] = 0;
                featSize.b[euclidian_X_num] = SizeX;
                featSize.a[euclidian_Y_num] = 0;
                featSize.b[euclidian_Y_num] = SizeY;

                switch (SizeType)
                {
                    case SizeClassificationType.Maximum :
                        // Set comparators for CDimensionType.EUCLIDIAN_X and CDimensionType.EUCLIDIAN_Y
                        for (int i = 0; i < nbDim; i++)
                        {
                            if ((i == euclidian_X_num) || (i == euclidian_Y_num))
                            {
                                // a0,b0=pattern=size of eoc, a1,b1=eoc
                                // if eoc is smaller than pattern then we want a0<a1<b1 and a1<b1<b0
                                compSize.Comparators[i].a0a1.SignedDistance = true;
                                compSize.Comparators[i].a0a1.MaxDistance = Double.NegativeInfinity;
                                compSize.Comparators[i].a0b1.SignedDistance = true;
                                compSize.Comparators[i].a0b1.MaxDistance = Double.NegativeInfinity;
                                compSize.Comparators[i].b0a1.SignedDistance = true;
                                compSize.Comparators[i].b0a1.MaxDistance = Double.PositiveInfinity;
                                compSize.Comparators[i].b0b1.SignedDistance = true;
                                compSize.Comparators[i].b0b1.MaxDistance = Double.PositiveInfinity;
                            }
                        }
                        break;
                    case SizeClassificationType.Minimum :
                        // Set comparators for CDimensionType.EUCLIDIAN_X and CDimensionType.EUCLIDIAN_Y
                        for (int i = 0; i < nbDim; i++)
                        {
                            if ((i == euclidian_X_num) || (i == euclidian_Y_num))
                            {
                                // a0,b0=pattern=size of eoc, a1,b1=eoc
                                // if eoc is greater than pattern then we want a1<a0<b1 and a1<b0<b1
                                compSize.Comparators[i].a0a1.SignedDistance = true;
                                compSize.Comparators[i].a0a1.MaxDistance = Double.PositiveInfinity;
                                compSize.Comparators[i].a0b1.SignedDistance = true;
                                compSize.Comparators[i].a0b1.MaxDistance = Double.NegativeInfinity;
                                compSize.Comparators[i].b0a1.SignedDistance = true;
                                compSize.Comparators[i].b0a1.MaxDistance = Double.PositiveInfinity;
                                compSize.Comparators[i].b0b1.SignedDistance = true;
                                compSize.Comparators[i].b0b1.MaxDistance = Double.NegativeInfinity;
                            }
                        }
                        break;
                    case SizeClassificationType.Exact :
                        // Set comparators for CDimensionType.EUCLIDIAN_X and CDimensionType.EUCLIDIAN_Y
                        for (int i = 0; i < nbDim; i++)
                        {
                            if ((i == euclidian_X_num) || (i == euclidian_Y_num))
                            {
                                // a0,b0=pattern=size of eoc, a1,b1=eoc
                                // if eoc is greater than pattern then we want a0-xtol/2<a1<a0+xtol/2 and b0-xtol/2<b1<b1+xtol/2
                                compSize.Comparators[i].a0a1.SignedDistance = false;
                                compSize.Comparators[i].a0a1.MaxDistance =
                                    (i == euclidian_X_num) ? SizeXTolerance / 2.0 : SizeYTolerance / 2.0;
                                compSize.Comparators[i].a0b1.SignedDistance = true;
                                compSize.Comparators[i].a0b1.MaxDistance = Double.NegativeInfinity;
                                compSize.Comparators[i].b0a1.SignedDistance = true;
                                compSize.Comparators[i].b0a1.MaxDistance = Double.PositiveInfinity;
                                compSize.Comparators[i].b0b1.SignedDistance = false;
                                compSize.Comparators[i].b0b1.MaxDistance =
                                   (i == euclidian_X_num) ? SizeXTolerance / 2.0 : SizeYTolerance / 2.0;
                            }
                        }
                        break;
                }

                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, LabelNamePosi, LabelNameSize,
                    featSize, coefficients, compSize,
                    CClassifyByFeatures.ClassificationType.ClassifyBySize,
                    false);
                resultSize = opClassify.Apply(analysis);
            }
            else
            {
                // Inserts new EOC for ALL matching childs
                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, LabelNamePosi, LabelNameSize,
                    new CFeatures(nbDim), // default features
                    coefficients,
                    new CFeaturesComparator(nbDim), // default comparators
                    CClassifyByFeatures.ClassificationType.ClassifyBySize,
                    false);
                resultSize = opClassify.Apply(analysis);
            }

            Trace.Unindent();
            Trace.WriteLine("Phase 2 : OK.", name);

            Trace.WriteLine("Phase 3 : Classification by Form...", name);
            Trace.Indent();

            bool resultForm = false;
            if (UseForm == true)
            {
                /*
                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, EOCTypeChild, EOCTypeNewEOC, FeaturesForm,
                    coefficients, ComparatorForm,
                    CClassifyByFeatures.ClassificationType.ClassifyByForm,
                    Replace);
                result |= opClassify.Apply(analysis);
                */
            }
            else
            {
                // Inserts new EOC for ALL matching childs
                opClassify = new CClassifyByFeatures(
                    EOCTypeParent, LabelNameSize, LabelNameForm,
                    new CFeatures(nbDim), // default features
                    coefficients,
                    new CFeaturesComparator(nbDim), // default comparators
                    CClassifyByFeatures.ClassificationType.ClassifyByPosition,
                    false);
                resultForm = opClassify.Apply(analysis);
            }

            Trace.Unindent();
            Trace.WriteLine("Phase 3 : OK.", name);

            Trace.WriteLine("Phase 4 : Finally, inserts " + EOCTypeNewEOC + "...", name);
            Trace.Indent();

            // Inserts new EOC for ALL matching childs
            opClassify = new CClassifyByFeatures(
                EOCTypeParent, LabelNameForm, EOCTypeNewEOC,
                new CFeatures(nbDim), // default features
                coefficients,
                new CFeaturesComparator(nbDim), // default comparators
                CClassifyByFeatures.ClassificationType.ClassifyByPosition,
                false);
            result = opClassify.Apply(analysis);

            Trace.Unindent();
            Trace.WriteLine("Phase 4 : OK.", name);

            Trace.WriteLine("Phase 5 : Simplifications...", name);
            Trace.Indent();

            // Delete temporary EOC(LABELPOSI/LABELSIZE/LABELFORM) which are childs of EOC(EOCTypeNewEOC)
            opDel = new CDelete(EOCTypeNewEOC, LabelNameForm, false, true);
            opDel.Apply(analysis);
            opDel = new CDelete(EOCTypeNewEOC, LabelNameSize, false, true);
            opDel.Apply(analysis);
            opDel = new CDelete(EOCTypeNewEOC, LabelNamePosi, false, true);
            opDel.Apply(analysis);

            Trace.Unindent();
            Trace.WriteLine("Phase 5 : OK.", name);

            Trace.WriteLine("Phase 6 : Cleaning...", name);
            Trace.Indent();

            // Delete temporary EOC(LABELPOSI/LABELSIZE/LABELFORM) which are childs of EOC(Parent)
            // inserted by previous operations
            opDel = new CDelete(EOCTypeParent, LabelNameForm, false, true);
            opDel.Apply(analysis);
            opDel = new CDelete(EOCTypeParent, LabelNameSize, false, true);
            opDel.Apply(analysis);
            opDel = new CDelete(EOCTypeParent, LabelNamePosi, false, true);
            opDel.Apply(analysis);

            // Delete temporary labels
            OperatorMisc.CDeleteLabel opDelLabel = new Agora.Model.OperatorMisc.CDeleteLabel();
            opDelLabel.EOCTypeName = LabelNameForm;
            opDelLabel.Apply(analysis);
            opDelLabel.EOCTypeName = LabelNameSize;
            opDelLabel.Apply(analysis);
            opDelLabel.EOCTypeName = LabelNamePosi;
            opDelLabel.Apply(analysis);

            Trace.Unindent();
            Trace.WriteLine("Phase 6 : OK.", name);
            Trace.WriteLine("Euclidian features OK.", name);

            return result;
        }
        public override string ToString()
        {
            return "EuclidianFeatures:" +
                "Parent=" + EOCTypeParent + "," +
                "Child=" + EOCTypeChild + "," +
                "New=" + EOCTypeNewEOC + "," +
                "UseForm=" + UseForm.ToString() + "," +
                "UsePosi=" + UsePosition.ToString() + "," +
                "UseSize=" + UseSize.ToString() + "," +
                "PosiXMin=" + PositionXMin.ToString() + "," +
                "PosiXMax=" + PositionXMax.ToString() + "," +
                "PosiYMin=" + PositionYMin.ToString() + "," +
                "PosiYMax=" + PositionYMax.ToString() + "," +
                "SizeType=" + SizeType.ToString() + "," +
                "SizeX=" + SizeX.ToString() + "," +
                "SizeXTol=" + SizeXTolerance.ToString() + "," +
                "SizeY=" + SizeY.ToString() + "," +
                "SizeYTol=" + SizeYTolerance.ToString() + "," +
                "FormXMin=" + FormXMin.ToString() + "," +
                "FormXMax=" + FormXMax.ToString() + "," +
                "FormYMin=" + FormYMin.ToString() + "," +
                "FormYMax=" + FormYMax.ToString() + "," +
                "Replace=" + Replace.ToString() + "," +
                "";
        }
    }

}
