﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace Agora.Model.OperatorLevel3
{
    public class CTextSegmentation : Agora.Model.Project.CStepOperator
    {
        private const string LABEL_NOISE = "NOISE";
        private const string LABEL_ACCENT = "ACCENT";
        private const string LABEL_CHAR = "CHAR";
        private const string LABEL_CHAR_AD = "CHAR_AD";
        private const string LABEL_IMAGE = "IMAGE";
        private const string LABEL_WORD = "WORD";
        private const string LABEL_LWORD = "LEFTWORD";
        private const string LABEL_RWORD = "RIGHTWORD";
        private const string LABEL_TMPLINE = "TMPLINE";
        private const string LABEL_LINE = "LINE";
        private const string LABEL_TMPBLOCK = "TMPBLOCK";
        private const string LABEL_BLOCK = "BLOCK";

        public string EOCTypeParent { get; set; }
        public string EOCTypeCC { get; set; }
        public string EOCTypeNewEOCPrefix { get; set; }
        public double NoiseWidthMax { get; set; }
        public double NoiseHeightMax { get; set; }
        public double AccentWidthMax { get; set; }
        public double AccentHeightMax { get; set; }
        public double CharWidthMax { get; set; }
        public double CharHeightMax { get; set; }
        public double CharADWidthMax { get; set; }
        public double CharADHeightMax { get; set; }
        public double CharAccentYSpacingMax { get; set; }
        public double WordCharSpacingXMax { get; set; }
        public double WordLeftRangemapXMax { get; set; }
        public double WordRightRangemapXMin { get; set; }
        public double BlockLineSpacingYMax { get; set; }
        public double DocumentBlockSpacingXMin { get; set; }

        public CTextSegmentation()
        {
            EOCTypeParent = String.Empty;
            EOCTypeCC = String.Empty;
            EOCTypeNewEOCPrefix = String.Empty;
        }

        public CTextSegmentation(string eocTypeParent, string eocTypeCC, string eocTypeNewEOCPrefix,
            double noiseWidthMax, double noiseHeightMax,
            double accentWidthMax, double accentHeightMax,
            double charWidthMax, double charHeightMax,
            double charADWidthMax, double charADHeightMax,
            double charAccentYSpacingMax,
            double wordCharSpacingXMax,
            double wordLeftRangemapXMax, double wordRightRangemapXMin,
            double blockLineSpacingYMax,
            double documentBlockSpacingXMin)
        {
            this.EOCTypeParent = String.Copy(eocTypeParent);
            this.EOCTypeCC = String.Copy(eocTypeCC);
            this.EOCTypeNewEOCPrefix = String.Copy(eocTypeNewEOCPrefix);
            this.NoiseWidthMax = noiseWidthMax;
            this.NoiseHeightMax = noiseHeightMax;
            this.AccentWidthMax = accentWidthMax;
            this.AccentHeightMax = accentHeightMax;
            this.CharWidthMax = charWidthMax;
            this.CharHeightMax = charHeightMax;
            this.CharADWidthMax = charADWidthMax;
            this.CharADHeightMax = charADHeightMax;
            this.CharAccentYSpacingMax = charAccentYSpacingMax;
            this.WordCharSpacingXMax = wordCharSpacingXMax;
            this.WordLeftRangemapXMax = wordLeftRangemapXMax;
            this.WordRightRangemapXMin = wordRightRangemapXMin;
            this.BlockLineSpacingYMax = blockLineSpacingYMax;
            this.DocumentBlockSpacingXMin = documentBlockSpacingXMin;
        }

        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CTextSegmentation(EOCTypeParent, EOCTypeCC, EOCTypeNewEOCPrefix,
                NoiseWidthMax, NoiseHeightMax, AccentWidthMax, AccentHeightMax,
                CharWidthMax, CharHeightMax, CharADWidthMax, CharADHeightMax, CharAccentYSpacingMax,
                WordCharSpacingXMax, WordLeftRangemapXMax, WordRightRangemapXMin,
                BlockLineSpacingYMax,
                DocumentBlockSpacingXMin);
        }

        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            bool result = true;
            Trace.WriteLine("Segmentation...", name);

            int nbDim = analysis.GetDimensions().Length;

            // #########
            // NEVER REUSE SAME OPERATOR (because "new" will clone parameters !)
            // ALWAYS USE NEW OPERATOR OR UPDATE ITS PROPERTIES
            // #########

            double[][] coefficients;
            CFeatures features;
            CFeatureMode[] modes;
            CFeaturesComparator comparator;
            CFeaturesComparator[][] ggComparators;
            CPairingEvaluator evaluator;
            OperatorMisc.CCreateLabel opCreateLabel;
            OperatorLevel1.CSetFeatures opSetFeat;
            OperatorLevel1.CInsert opInsert;
            OperatorLevel1.CDelete opDel;
            OperatorLevel2.CClassifyByFeatures opFeat;
            OperatorLevel2.CFusion opFusion;
            OperatorLevel2.CIntersect opInter;

            // Create all labels

            string ACCENT = EOCTypeNewEOCPrefix + LABEL_ACCENT;
            string BLOCK = EOCTypeNewEOCPrefix + LABEL_BLOCK;
            string CHAR = EOCTypeNewEOCPrefix + LABEL_CHAR;
            string CHAR_AD = EOCTypeNewEOCPrefix + LABEL_CHAR_AD;
            string IMAGE = EOCTypeNewEOCPrefix + LABEL_IMAGE;
            string LINE = EOCTypeNewEOCPrefix + LABEL_LINE;
            string LWORD = EOCTypeNewEOCPrefix + LABEL_LWORD;
            string NOISE = EOCTypeNewEOCPrefix + LABEL_NOISE;
            string RWORD = EOCTypeNewEOCPrefix + LABEL_RWORD;
            string TMPBLOCK = EOCTypeNewEOCPrefix + LABEL_TMPBLOCK;
            string TMPLINE = EOCTypeNewEOCPrefix + LABEL_TMPLINE;
            string WORD = EOCTypeNewEOCPrefix + LABEL_WORD;

            opCreateLabel = new OperatorMisc.CCreateLabel(ACCENT); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(BLOCK); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(TMPBLOCK); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(CHAR); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(CHAR_AD); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(IMAGE); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(TMPLINE); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(LINE); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(LWORD); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(NOISE); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(RWORD); opCreateLabel.Apply(analysis);
            opCreateLabel = new OperatorMisc.CCreateLabel(WORD); opCreateLabel.Apply(analysis);

            // Classify Noise and delete

            features = new CFeatures(nbDim);
            features.b[0] = NoiseWidthMax;
            features.b[1] = NoiseHeightMax;

            coefficients = new double[1][];
            coefficients[0] = new double[nbDim];
            coefficients[0][0] = 1.0;
            coefficients[0][1] = 1.0;

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[0].a0a1.MaxDistance = double.NegativeInfinity;
            comparator.Comparators[0].a0a1.SignedDistance = true;
            comparator.Comparators[0].b0b1.MaxDistance = double.PositiveInfinity;
            comparator.Comparators[0].b0b1.SignedDistance = true;
            comparator.Comparators[1].a0a1.MaxDistance = double.NegativeInfinity;
            comparator.Comparators[1].a0a1.SignedDistance = true;
            comparator.Comparators[1].b0b1.MaxDistance = double.PositiveInfinity;
            comparator.Comparators[1].b0b1.SignedDistance = true;

            opFeat = new CClassifyByFeatures(
                EOCTypeParent, EOCTypeCC, NOISE,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyBySize, false
                );
            opFeat.Apply(analysis);

            opDel = new CDelete(EOCTypeParent, NOISE, true, true);
            // opDel.Apply(analysis);

            // Classify Accent/Point

            features.b[0] = AccentWidthMax;
            features.b[1] = AccentHeightMax;
            opFeat = new CClassifyByFeatures(
                EOCTypeParent, EOCTypeCC, ACCENT,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyBySize, false
                );
            opFeat.Apply(analysis);

            // Classify Char

            features.b[0] = CharWidthMax;
            features.b[1] = CharHeightMax;

            opFeat = new CClassifyByFeatures(
                EOCTypeParent, EOCTypeCC, CHAR,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyBySize, false
                );
            opFeat.Apply(analysis);

            // Classify Char Ascender/Descender

            features.b[0] = CharADWidthMax;
            features.b[1] = CharADHeightMax;
            opFeat = new CClassifyByFeatures(
                EOCTypeParent, EOCTypeCC, CHAR_AD,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyBySize, false
                );
            opFeat.Apply(analysis);

            // Classify Image = remaining CC

            features = new CFeatures(nbDim);
            comparator = new CFeaturesComparator(nbDim);
            opFeat = new CClassifyByFeatures(
                EOCTypeParent, EOCTypeCC, IMAGE,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyByPosition, false
                );
            opFeat.Apply(analysis);

            // Group Char and Accent (accents, points over "i", etc.)
            // 1. Average X feature for Accent
            // 2. Insert if (Char.X.a0 < Average < Char.X.b0) and distance < CharAccentYSpacingMax
            // 3. Delete remaining Accent

            modes = CFeatureMode.BuildArray(nbDim);
            modes[0].aMode = CFeatureMode.SetFeatureMode.AverageAll;
            modes[0].bMode = CFeatureMode.SetFeatureMode.AverageAll;

            opSetFeat = new CSetFeatures(EOCTypeParent, ACCENT, modes, true);
            opSetFeat.Apply(analysis);

            coefficients = new double[1][] { new double[nbDim] };
            ggComparators = CFeaturesComparator.BuildMatrix(2, nbDim);
            // Child0.X.a0 < Child1.X.a0 and Child0.X.b0 < Child1.X.b0 (remember Child1.X = AverageAll !)
            ggComparators[0][1].Comparators[0].a0a1.MaxDistance = double.NegativeInfinity;
            ggComparators[0][1].Comparators[0].a0a1.SignedDistance = true;
            ggComparators[0][1].Comparators[0].b0b1.MaxDistance = double.PositiveInfinity;
            ggComparators[0][1].Comparators[0].b0b1.SignedDistance = true;
            // Child0.Y.a0 > (CharNoiseYSpacingMax + Child1.Y.b0) (ie Child0 is "under" Child1)
            ggComparators[0][1].Comparators[1].a0b1.MaxDistance = CharAccentYSpacingMax;
            ggComparators[0][1].Comparators[1].a0b1.SignedDistance = true;

            evaluator = new CPairingEvaluator(
                false, false, false, coefficients, null, null, ggComparators); 

            opInsert = new CInsert(
                EOCTypeParent,
                new string[] { CHAR, ACCENT },
                CHAR_AD,
                CFeatures.BuildArray(2, nbDim), evaluator, null, CFeatureMode.BuildArray(nbDim), false);
            opInsert.Apply(analysis);

            opDel = new CDelete(EOCTypeParent, ACCENT, true, true);
            // opDel.Apply(analysis);

            // Build LINEs of center Y aligned CHAR

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[1].a0a1.MaxDistance = CharHeightMax;
            comparator.Comparators[1].a0a1.SignedDistance = false;

            modes = CFeatureMode.BuildArray(nbDim);
            modes[1].aMode = CFeatureMode.SetFeatureMode.AverageAll;
            modes[1].bMode = CFeatureMode.SetFeatureMode.AverageAll;

            opFusion = new CFusion(
                EOCTypeParent, CHAR, TMPLINE,
                comparator, null, modes);
            opFusion.Apply(analysis);

            // Add AD Chars to existing lines
            // 1. A Chars -> bottom of line
            // 2. D Chars -> top of line
            
            modes = CFeatureMode.BuildArray(nbDim);
            modes[1].aMode = CFeatureMode.SetFeatureMode.Average;
            modes[1].bMode = CFeatureMode.SetFeatureMode.Average;
            opSetFeat = new CSetFeatures(EOCTypeParent, TMPLINE, modes, true);
            opSetFeat.Apply(analysis);

            ggComparators = CFeaturesComparator.BuildMatrix(2, nbDim);

            // (Line.Y.b - CharAD.Y.b) < CharHeightMax
            ggComparators[0][1].Comparators[1].b0b1.MaxDistance = CharHeightMax;
            ggComparators[0][1].Comparators[1].b0b1.SignedDistance = false;

            evaluator = new CPairingEvaluator(false, false, false, null, null, null, ggComparators);

            opInsert = new CInsert(
                EOCTypeParent,
                new string[] { TMPLINE, CHAR_AD },
                TMPLINE,
                CFeatures.BuildArray(2, nbDim), evaluator, null, modes, true);
            opInsert.Apply(analysis);

            opDel = new CDelete(TMPLINE, TMPLINE, false, true);
            opDel.Apply(analysis);

            // (Line.Y.a - CharAD.Y.a) < CharHeightMax
            ggComparators[0][1].Comparators[1] = new CFeatureMatrixComparator();
            ggComparators[0][1].Comparators[1].a0a1.MaxDistance = CharHeightMax;
            ggComparators[0][1].Comparators[1].a0a1.SignedDistance = false;

            evaluator = new CPairingEvaluator(false, false, false, null, null, null, ggComparators);

            modes = CFeatureMode.BuildArray(nbDim);
            modes[1].aMode = CFeatureMode.SetFeatureMode.Average;
            modes[1].bMode = CFeatureMode.SetFeatureMode.Average;

            opInsert = new CInsert(
                EOCTypeParent,
                new string[] { TMPLINE, CHAR_AD }, TMPLINE,
                CFeatures.BuildArray(2, nbDim), evaluator, null, modes, true);
            opInsert.Apply(analysis);

            opDel.Apply(analysis);

            // regroup lines that overlap

            coefficients = new double[1][];
            coefficients[0] = new double[nbDim];
            coefficients[0][0] = 1.0;
            coefficients[0][1] = 1.0;
            opInter = new CIntersect(
                EOCTypeParent, new string[] { TMPLINE, TMPLINE }, TMPLINE,
                coefficients, null, modes);
            opInter.Apply(analysis);

            opDel.Apply(analysis);

            // TODO Build LINEs of center Y aligned CHAR_AD : isolated CHAR_AD or
            // textlines with only CHAR_AD letters (example : capitals titles)
            
            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[1].a0a1.MaxDistance = CharHeightMax;
            comparator.Comparators[1].a0a1.SignedDistance = false;
            comparator.Comparators[1].b0b1.MaxDistance = CharHeightMax;
            comparator.Comparators[1].b0b1.SignedDistance = false;

            modes = CFeatureMode.BuildArray(nbDim);
            modes[1].aMode = CFeatureMode.SetFeatureMode.Average;
            modes[1].bMode = CFeatureMode.SetFeatureMode.Average;

            opFusion = new CFusion(
                EOCTypeParent, CHAR_AD, TMPLINE,
                comparator, null, modes);
            opFusion.Apply(analysis);
            

            // Build WORDs : fusion CHAR into WORD (inside TMPLINEs !)

            features = new CFeatures(nbDim);
            comparator = new CFeaturesComparator(nbDim);
            opFeat = new CClassifyByFeatures(
                TMPLINE, CHAR_AD, CHAR,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyByPosition, false
                );
            opFeat.Apply(analysis);

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[0].a0b1.MaxDistance = WordCharSpacingXMax;
            comparator.Comparators[0].a0b1.SignedDistance = false;
            
            modes = CFeatureMode.BuildArray(nbDim);

            opFusion = new CFusion(
                TMPLINE, CHAR, WORD,
                comparator, null, modes);
            opFusion.Apply(analysis);
            
            // regroup words that overlap in a line

            modes = CFeatureMode.BuildArray(nbDim);
            opSetFeat = new CSetFeatures(WORD, CHAR, modes, true);
            opSetFeat.Apply(analysis);

            coefficients = new double[1][];
            coefficients[0] = new double[nbDim];
            coefficients[0][0] = 1.0;
            coefficients[0][1] = 1.0;
            opInter = new CIntersect(
                TMPLINE, new string[] { WORD, WORD }, WORD,
                coefficients, null, modes);
            opInter.Apply(analysis);

            opDel = new CDelete(WORD, WORD, false, true);
            opDel.Apply(analysis);

            // Classify Left and Right words

            features = new CFeatures(nbDim);
            features.a[2] = WordLeftRangemapXMax;
            features.b[2] = WordRightRangemapXMin;

            coefficients = new double[1][];
            coefficients[0] = new double[nbDim];
            coefficients[0][2] = 1.0;

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[2].a0a1.MaxDistance = double.PositiveInfinity;
            comparator.Comparators[2].a0a1.SignedDistance = true;

            opFeat = new CClassifyByFeatures(
                TMPLINE, WORD, LWORD,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyByPosition, false
                );
            opFeat.Apply(analysis);

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[2].b0b1.MaxDistance = double.NegativeInfinity;
            comparator.Comparators[2].b0b1.SignedDistance = true;

            opFeat = new CClassifyByFeatures(
                TMPLINE, WORD, RWORD,
                features, coefficients, comparator,
                CClassifyByFeatures.ClassificationType.ClassifyByPosition, false
                );
            opFeat.Apply(analysis);

            // Build sublines only from "normal" words. Maximum spacing between words
            // is computed from BlockSpacingMin & WordCharSpacingMax

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[0].b0a1.MaxDistance = 2 * WordCharSpacingXMax + DocumentBlockSpacingXMin;
            comparator.Comparators[0].b0a1.SignedDistance = false;

            modes = CFeatureMode.BuildArray(nbDim);

            opFusion = new CFusion(
                TMPLINE, WORD, LINE,
                comparator, null, modes);
            opFusion.Apply(analysis);

            // Add Left word to subline

            ggComparators = CFeaturesComparator.BuildMatrix(2, nbDim);
            // Child0.X.b0 < Child1.X.a1 + Distance, Distance = WordSpacing+BlockSpacing
            //ggComparators[0][1].Comparators[0].b0a1.MaxDistance = WordCharSpacingXMax + DocumentBlockSpacingXMin;
            ggComparators[0][1].Comparators[0].b0a1.MaxDistance = DocumentBlockSpacingXMin;
            ggComparators[0][1].Comparators[0].b0a1.SignedDistance = false;

            evaluator = new CPairingEvaluator(
                false, false, false, null, null, null, ggComparators);

            modes = CFeatureMode.BuildArray(nbDim);

            opInsert = new CInsert(
                TMPLINE, new string[] { LWORD, LINE }, LINE,
                CFeatures.BuildArray(2, nbDim), evaluator, null, modes, false);
            opInsert.Apply(analysis);

            // Add Right word to subline

            opInsert = new CInsert(
                TMPLINE, new string[] { LINE, RWORD }, LINE,
                CFeatures.BuildArray(2, nbDim), evaluator, null, modes, false);
            opInsert.Apply(analysis);

            // Delete sublines included in sublines

            opDel = new CDelete(LINE, LINE, false, true);
            opDel.Apply(analysis);

            // Remove tmplines

            opDel = new CDelete(EOCTypeParent, TMPLINE, false, true);
            opDel.Apply(analysis);

            // gather isolated word, lword, rword -> line

            opFeat = new CClassifyByFeatures(
                EOCTypeParent, WORD, LINE,
                new CFeatures(nbDim), coefficients, new CFeaturesComparator(nbDim),
                CClassifyByFeatures.ClassificationType.ClassifyBySize, false
                );
            opFeat.Apply(analysis);

            opFeat.EOCTypeChild = LWORD;
            opFeat.Apply(analysis);

            opFeat.EOCTypeChild = RWORD;
            opFeat.Apply(analysis);

            // Prepare features for block building : average for lines

            modes = CFeatureMode.BuildArray(nbDim);
            modes[1].aMode = CFeatureMode.SetFeatureMode.AverageAll;
            modes[1].bMode = CFeatureMode.SetFeatureMode.AverageAll;
            opSetFeat.Modes = modes;

            opSetFeat.EOCTypeParent = WORD;
            opSetFeat.EOCTypeChild = CHAR;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = LWORD;
            opSetFeat.EOCTypeChild = WORD;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = RWORD;
            opSetFeat.EOCTypeChild = WORD;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = LINE;
            opSetFeat.EOCTypeChild = WORD;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = LINE;
            opSetFeat.EOCTypeChild = LWORD;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = LINE;
            opSetFeat.EOCTypeChild = RWORD;
            opSetFeat.Apply(analysis);

            opSetFeat.EOCTypeParent = EOCTypeParent;
            opSetFeat.EOCTypeChild = LINE;
            opSetFeat.Apply(analysis);

            // Fusion of lines into blocks due to left and right align

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[0].a0a1.MaxDistance = DocumentBlockSpacingXMin;
            comparator.Comparators[0].a0a1.SignedDistance = false;
            comparator.Comparators[1].b0a1.MaxDistance = BlockLineSpacingYMax;
            comparator.Comparators[1].b0a1.SignedDistance = false;

            modes = CFeatureMode.BuildArray(nbDim);
            //modes[0].aMode = CFeatureMode.SetFeatureMode.Average;
            //modes[0].bMode = CFeatureMode.SetFeatureMode.Average;

            opFusion = new CFusion(
                EOCTypeParent, LINE, TMPBLOCK,
                comparator, null, modes);
            opFusion.Apply(analysis);

            comparator = new CFeaturesComparator(nbDim);
            comparator.Comparators[0].b0b1.MaxDistance = DocumentBlockSpacingXMin;
            comparator.Comparators[0].b0b1.SignedDistance = false;
            comparator.Comparators[1].b0a1.MaxDistance = BlockLineSpacingYMax;
            comparator.Comparators[1].b0a1.SignedDistance = false;

            opFusion = new CFusion(
                EOCTypeParent, TMPBLOCK, BLOCK,
                comparator, null, modes);
            opFusion.Apply(analysis);

            Trace.WriteLine("OK.", name);
            return result;
        }

        public override string ToString()
        {
            return "TextSegmentation:" +
                "Parent=" + EOCTypeParent + "," +
                "CC=" + EOCTypeCC + "," +
                "Prefix=" + EOCTypeNewEOCPrefix + "," +
                "NoiseWidthMax=" + NoiseWidthMax.ToString() + "," +
                "NoiseHeightMax=" + NoiseHeightMax.ToString() + "," +
                "AccentWidthMax=" + AccentWidthMax.ToString() + "," +
                "AccentHeightMax=" + AccentHeightMax.ToString() + "," +
                "CharWidthMax=" + CharWidthMax.ToString() + "," +
                "CharHeightMax=" + CharHeightMax.ToString() + "," +
                "CharADWidthMax=" + CharADWidthMax.ToString() + "," +
                "CharADHeightMax=" + CharADHeightMax.ToString() + "," +
                "CharAccentYSpacingMax=" + CharAccentYSpacingMax + "," +
                "WordCharSpacingXMax=" + WordCharSpacingXMax.ToString() + "," +
                "WordLeftRangemapXMax=" + WordLeftRangemapXMax.ToString() + "," +
                "WordRightRangemapXMin=" + WordRightRangemapXMin.ToString() + "," +
                "BlockLineSpacingYMax=" + BlockLineSpacingYMax.ToString() + "," +
                "DocumentBlockSpacingXMin=" + DocumentBlockSpacingXMin.ToString() + "," +
                "";
        }
    }
}
