﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;

namespace Agora.Model.OperatorMisc
{
    public class CDeleteLabel : Agora.Model.Project.CStepOperator
    {
        private int _EOCType = 0;
        public int EOCType
        {
            get { return _EOCType; }
            set { _EOCType = value; }
        }
        private string _EOCTypeName = "";
        public string EOCTypeName
        {
            get { return _EOCTypeName; }
            set { _EOCTypeName = value; }
        }

        public CDeleteLabel()
        {
        }
        public CDeleteLabel(string name)
        {
            _EOCTypeName = name.ToUpper();
        }
        public CDeleteLabel(int type, string name)
        {
            _EOCType = type;
            _EOCTypeName = name.ToUpper();
        }

        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CDeleteLabel(_EOCType, _EOCTypeName.ToUpper());
        }
        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            Trace.WriteLine("Delete label '" + _EOCTypeName + "' ... ", name);
            _EOCTypeName = _EOCTypeName.ToUpper();
            CEOCType type = null;
            if (_EOCTypeName != "")
            {
                type = analysis.GetEOCTypeByName(_EOCTypeName);
            }
            else if (_EOCType != 0)
            {
                type = analysis.GetEOCTypeByID(_EOCType);
            }
            if (type == null)
            {
                Trace.WriteLine(type.ToString() + " NOT FOUND !", name);
                return false;
            }

            bool foundExisting = false;
            CEOC eoc = null;
            CBFS bfs = new CBFS();
            bfs.Start(analysis.Graph.GetRoot());
            while ((eoc = bfs.GetNext()) != null)
            {
                if (eoc.Eoctype == type)
                {
                    foundExisting = true;
                    break;
                }
            }

            if (foundExisting == true)
            {
                Trace.WriteLine(type.ToString() + " USED BY EOC !", name);
                return false;
            }

            analysis.DeleteEOCType(type);

            Trace.WriteLine("OK.", name);
            return true;
        }
        /*
        public override bool Undo(CImageAnalysis analysis)
        {
            return true;
        }
        */
        public override string ToString()
        {
            return "DeleteLabel:" +
                "Type=" + EOCTypeName;
        }
    }
}
