﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Xml.Serialization;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;

namespace Agora.Model.OperatorMisc
{
    /// <summary>
    /// This class has 2 goals :
    /// 1. offering general services for preparing exportation
    /// of the results (creating directory structure, etc.) of a whole project,
    /// thanks to static methods and properties
    /// 2. offering specific services that can be called within a scenario
    /// </summary>
    public class CExportResults : Agora.Model.Project.CStepOperator
    {
        private List<string> _AltoTextblockEOCTypes = new List<string>();
        [XmlArray("AltoTextblockEOCTypes")]
        [XmlArrayItem("EOCType", typeof(string))]
        public List<string> AltoTextblockEOCTypes
        {
            get { return _AltoTextblockEOCTypes; }
            set { _AltoTextblockEOCTypes = value; }
        }

        private List<string> _AltoTextlineEOCTypes = new List<string>();
        [XmlArray("AltoTextlineEOCTypes")]
        [XmlArrayItem("EOCType", typeof(string))]
        public List<string> AltoTextlineEOCTypes
        {
            get { return _AltoTextlineEOCTypes; }
            set { _AltoTextlineEOCTypes = value; }
        }

        private List<string> _AltoWordEOCTypes = new List<string>();
        [XmlArray("AltoWordEOCTypes")]
        [XmlArrayItem("EOCType", typeof(string))]
        public List<string> AltoWordEOCTypes
        {
            get { return _AltoWordEOCTypes; }
            set { _AltoWordEOCTypes = value; }
        }

        private List<string> _AltoStringEOCTypes = new List<string>();
        [XmlArray("AltoStringEOCTypes")]
        [XmlArrayItem("EOCType", typeof(string))]
        public List<string> AltoStringEOCTypes
        {
            get { return _AltoStringEOCTypes; }
            set { _AltoStringEOCTypes = value; }
        }

        private List<string> _AltoIllustrationEOCTypes = new List<string>();
        [XmlArray("AltoIllustrationEOCTypes")]
        [XmlArrayItem("EOCType", typeof(string))]
        public List<string> AltoIllustrationEOCTypes
        {
            get { return _AltoIllustrationEOCTypes; }
            set { _AltoIllustrationEOCTypes = value; }
        }

        private bool _AltoGenerateThumbnails = false;
        [XmlElement("AltoGenerateThumbnails")]
        public bool AltoGenerateThumbnails
        {
            get { return _AltoGenerateThumbnails; }
            set { _AltoGenerateThumbnails = value; }
        }

        private bool _AltoThumbnailsJPEGFormat = false;
        [XmlElement("AltoThumbnailsJPEGFormat")]
        public bool AltoThumbnailsJPEGFormat
        {
            get { return _AltoThumbnailsJPEGFormat; }
            set { _AltoThumbnailsJPEGFormat = value; }
        }

        private bool _AltoThumbnailsPrefixOriginalFilename = false;
        [XmlElement("AltoThumbnailsPrefixOriginalFilename")]
        public bool AltoThumbnailsPrefixOriginalFilename
        {
            get { return _AltoThumbnailsPrefixOriginalFilename; }
            set { _AltoThumbnailsPrefixOriginalFilename = value; }
        }

        private string _OriginalImageBasename = null;
        private string _OriginalImageID = null;
        private string _AltoImageFilename = null;
        private string _AltoImageBasename = null;
        private string _AltoImageID = null;
        private AgoraAlto.alto _AltoData = null;

        public CExportResults()
        {
        }
        public CExportResults(
            List<string> textblockEOCTypes,
            List<string> textlineEOCTypes,
            List<string> wordEOCTypes,
            List<string> stringEOCTypes,
            List<string> illustrationEOCTypes,
            bool generateThumbnails,
            bool jpegFormat,
            bool filenamePrefix)
        {
            _AltoTextblockEOCTypes.Clear();
            foreach (string typeName in textblockEOCTypes)
            {
                string s = new string(typeName.ToCharArray());
                _AltoTextblockEOCTypes.Add(s);
            }
            _AltoTextlineEOCTypes.Clear();
            foreach (string typeName in textlineEOCTypes)
            {
                string s = new string(typeName.ToCharArray());
                _AltoTextlineEOCTypes.Add(s);
            }
            _AltoWordEOCTypes.Clear();
            foreach (string typeName in wordEOCTypes)
            {
                string s = new string(typeName.ToCharArray());
                _AltoWordEOCTypes.Add(s);
            }
            _AltoStringEOCTypes.Clear();
            foreach (string typeName in stringEOCTypes)
            {
                string s = new string(typeName.ToCharArray());
                _AltoStringEOCTypes.Add(s);
            }
            _AltoIllustrationEOCTypes.Clear();
            foreach (string typeName in illustrationEOCTypes)
            {
                string s = new string(typeName.ToCharArray());
                _AltoIllustrationEOCTypes.Add(s);
            }
            AltoGenerateThumbnails = generateThumbnails;
            AltoThumbnailsJPEGFormat = jpegFormat;
            AltoThumbnailsPrefixOriginalFilename = filenamePrefix;
        }
        public override Agora.Model.Project.CStepOperator Clone()
        {
            return new CExportResults(
                _AltoTextblockEOCTypes,
                _AltoTextlineEOCTypes,
                _AltoWordEOCTypes,
                _AltoStringEOCTypes,
                _AltoIllustrationEOCTypes,
                _AltoGenerateThumbnails,
                _AltoThumbnailsJPEGFormat,
                _AltoThumbnailsPrefixOriginalFilename);
        }

        public override bool Apply(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            Trace.WriteLine("Start export...", name);
            bool result = true;
            result &= GLOBALDATA.CheckDestinationDirectories();
            result &= this.RetrieveAndRenameImage();
            result &= this.CreateAltoStructure();
            result &= this.FillAltoStructure(analysis);
            result &= this.SaveAltoStructure();
            Trace.WriteLine("OK.", name);
            return result;
        }
        /*
        public override bool Undo(CImageAnalysis analysis)
        {
            bool result = true;

            return result;
        }
        */
        public override string ToString()
        {
            string result = "Export:";
            result += "AltoStringType={";
            foreach (string s in _AltoStringEOCTypes)
            {
                result += s + ",";
            }
            result += "},AltoTextlineType={";
            foreach (string s in _AltoTextlineEOCTypes)
            {
                result += s + ",";
            }
            result += "},AltoWordType={";
            foreach (string s in _AltoWordEOCTypes)
            {
                result += s + ",";
            }
            result += "},AltoTextblockType={";
            foreach (string s in _AltoTextblockEOCTypes)
            {
                result += s + ",";
            }
            result += "},AltoIllustrationType={";
            foreach (string s in _AltoIllustrationEOCTypes)
            {
                result += s + ",";
            }
            result += "}";
            return result;
        }

        static private int CompareEOCByBoundingBoxLeft(CEOC x, CEOC y)
        {
            return x.BoundingBox.Left - y.BoundingBox.Left;
        }
        static private int CompareEOCByBoundingBoxTop(CEOC x, CEOC y)
        {
            return x.BoundingBox.Top - y.BoundingBox.Top;
        }

        private bool RetrieveAndRenameImage()
        {
            bool result = true;
            Trace.Assert(File.Exists(GLOBALDATA.CurrentImageFilename) == true, "Cannot open file " + GLOBALDATA.CurrentImageFilename);
            _OriginalImageBasename = Path.GetFileName(GLOBALDATA.CurrentImageFilename);
            _OriginalImageID = _OriginalImageBasename.Substring(0, _OriginalImageBasename.LastIndexOf("."));

            _AltoImageID = String.Format(GLOBALDATA.IMAGE_FILE_ID, GLOBALDATA.CurrentImageNum);
            _AltoImageBasename = _AltoImageID + "." + GLOBALDATA.IMAGE_OUTPUT_FILE_EXT;
            //if (_AltoThumbnailsPrefixOriginalFilename == true)
            //{
            //    _AltoImageBasename = _OriginalImageBasename + "." + _AltoImageBasename;
            //}
            _AltoImageFilename = Path.Combine(GLOBALDATA.ProjectDirectory, Path.Combine(GLOBALDATA.DIR_IMAGES, _AltoImageBasename));
            if (File.Exists(_AltoImageFilename) == true)
            {
                try
                {
                    File.Delete(_AltoImageFilename);
                    // File.Copy(GLOBALDATA.CurrentImageFilename, _AltoImageFilename);
                    Bitmap bmp = new Bitmap(GLOBALDATA.CurrentImageFilename);
                    bmp.Save(_AltoImageFilename, ImageFormat.Png);
                }
                catch (System.Exception e)
                {
                }
            }
            else
            {
                //File.Copy(GLOBALDATA.CurrentImageFilename, _AltoImageFilename);
                Bitmap bmp = new Bitmap(GLOBALDATA.CurrentImageFilename);
                bmp.Save(_AltoImageFilename, ImageFormat.Png);
            }

            return result;
        }

        private bool CreateAltoStructure()
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            Trace.WriteLine("Creating ALTO structure...", name);
            AgoraAlto.fileIdentifierType imagesLocation = new AgoraAlto.fileIdentifierType();
            imagesLocation.fileIdentifierLocation = "../" + GLOBALDATA.DIR_IMAGES;
            Trace.WriteLine("Alto structure for " + _OriginalImageBasename + "(" + _AltoImageBasename + ")");
            _AltoData = new AgoraAlto.alto();
            _AltoData.Description = new AgoraAlto.altoDescription();
            _AltoData.Description.MeasurementUnit = AgoraAlto.altoDescriptionMeasurementUnit.pixel;
            _AltoData.Description.MeasurementUnitSpecified = true;
            _AltoData.Description.sourceImageInformation = new AgoraAlto.sourceImageInformationType();
            _AltoData.Description.sourceImageInformation.fileName = _AltoImageBasename;
            _AltoData.Description.sourceImageInformation.fileIdentifier =
                new AgoraAlto.fileIdentifierType[] { imagesLocation };
            _AltoData.Description.OCRProcessing = new AgoraAlto.altoDescriptionOCRProcessing[1 + GLOBALDATA.ScenarioSteps.Count];
            // Step 0 is original image renaming
            _AltoData.Description.OCRProcessing[0] = new AgoraAlto.altoDescriptionOCRProcessing();
            _AltoData.Description.OCRProcessing[0].ID = "OCR.0";
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep = new AgoraAlto.processingStepType();
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep.processingSoftware = new AgoraAlto.processingSoftwareType();
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep.processingSoftware.softwareName = "AGORA";
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep.processingSoftware.softwareVersion = "3.0";
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep.processingStepDescription = new string[] { "RenameOriginalImage" };
            _AltoData.Description.OCRProcessing[0].ocrProcessingStep.processingStepSettings =
                Path.Combine(GLOBALDATA.ImagesSubDirectory, _OriginalImageBasename) +
                "," +
                Path.Combine(GLOBALDATA.DIR_IMAGES, _AltoImageBasename);
            // Following OCR steps are the Scenario steps
            for (int numStep = 1; numStep < (1 + GLOBALDATA.ScenarioSteps.Count); numStep++)
            {
                _AltoData.Description.OCRProcessing[numStep] = new AgoraAlto.altoDescriptionOCRProcessing();
                _AltoData.Description.OCRProcessing[numStep].ID = "OCR." + numStep.ToString();
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep = new AgoraAlto.processingStepType();
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep.processingSoftware = new AgoraAlto.processingSoftwareType();
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep.processingSoftware.softwareName = "AGORA";
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep.processingSoftware.softwareVersion = "3.0";
                string[] scenarioStep = GLOBALDATA.ScenarioSteps[numStep - 1].Split(new char[] { ':' });
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep.processingStepDescription =
                    new string[] { scenarioStep[0] };
                _AltoData.Description.OCRProcessing[numStep].ocrProcessingStep.processingStepSettings =
                    scenarioStep[1];
            }
            _AltoData.Styles = new AgoraAlto.altoStyles();
            _AltoData.Styles.ParagraphStyle = new AgoraAlto.altoStylesParagraphStyle[
                _AltoTextblockEOCTypes.Count +
                _AltoTextlineEOCTypes.Count +
                _AltoStringEOCTypes.Count];
            int numStyle = 0;
            foreach (string eocType in _AltoTextblockEOCTypes)
            {
                _AltoData.Styles.ParagraphStyle[numStyle] = new AgoraAlto.altoStylesParagraphStyle();
                _AltoData.Styles.ParagraphStyle[numStyle].ID = eocType;
                numStyle++;
            }
            foreach (string eocType in _AltoTextlineEOCTypes)
            {
                _AltoData.Styles.ParagraphStyle[numStyle] = new AgoraAlto.altoStylesParagraphStyle();
                _AltoData.Styles.ParagraphStyle[numStyle].ID = eocType;
                numStyle++;
            }
            foreach (string eocType in _AltoStringEOCTypes)
            {
                _AltoData.Styles.ParagraphStyle[numStyle] = new AgoraAlto.altoStylesParagraphStyle();
                _AltoData.Styles.ParagraphStyle[numStyle].ID = eocType;
                numStyle++;
            }
            _AltoData.Layout = new AgoraAlto.altoLayout();
            _AltoData.Layout.Page = new AgoraAlto.altoLayoutPage[1];
            _AltoData.Layout.Page[0] = new AgoraAlto.altoLayoutPage();
            _AltoData.Layout.Page[0].ID = _AltoImageID;
            _AltoData.Layout.Page[0].PrintSpace = new AgoraAlto.PageSpaceType();
            _AltoData.Layout.Page[0].PrintSpace.HPOS = 0.0F; // updated after each new bloc
            _AltoData.Layout.Page[0].PrintSpace.VPOS = 0.0F; // updated after each new bloc
            _AltoData.Layout.Page[0].PrintSpace.WIDTH = 0.0F; // updated after each new bloc
            _AltoData.Layout.Page[0].PrintSpace.HEIGHT = 0.0F; // updated after each new bloc
            _AltoData.Layout.Page[0].PrintSpace.TextBlock = null; // created later
            _AltoData.Layout.Page[0].PrintSpace.Illustration = null; // created later
            Trace.WriteLine("Creating ALTO structure OK.", name);
            return true;
        }

        private bool FillAltoStructure(CImageAnalysis analysis)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            Trace.WriteLine("Filling alto structure...", name);

            if (AltoGenerateThumbnails == true)
            {
                Trace.WriteLine("Thumbnails generation is ON : may be quite long...");
                _ThumbnailsCounter = 0;
            }
            // Scan all graph vertices for textblocks and illustrations and store a reference to them
            List<CEOC> textblocks = new List<CEOC>();
            List<CEOC> illustrations = new List<CEOC>();
            Agora.Model.Utils.CBFS bfs = new Agora.Model.Utils.CBFS();
            bfs.Start(analysis.Graph.GetRoot());
            CEOC eoc;
            while ((eoc = bfs.GetNext()) != null)
            {
                // If eoc = DOCUMENT --> updates page width and height
                if (eoc.Eoctype == analysis.GetEOCTypeByName("DOCUMENT"))
                {
                    _AltoData.Layout.Page[0].HEIGHTSpecified = true;
                    _AltoData.Layout.Page[0].WIDTHSpecified = true;
                    _AltoData.Layout.Page[0].HEIGHT = eoc.BoundingBox.Height;
                    _AltoData.Layout.Page[0].WIDTH = eoc.BoundingBox.Width;
                }
                // Is this eoc must be written as a text block in Alto file ?
                if (_AltoTextblockEOCTypes.Contains(eoc.Eoctype.TypeName) == true)
                {
                    textblocks.Add(eoc);
                }
                // Is this eoc must be written as an illustration block in Alto file ?
                else if (_AltoIllustrationEOCTypes.Contains(eoc.Eoctype.TypeName) == true)
                {
                    illustrations.Add(eoc);
                }
            }
            if (textblocks.Count > 0)
            {
                _AltoData.Layout.Page[0].PrintSpace.TextBlock = new AgoraAlto.TextBlockType[textblocks.Count];
                // For each found textblock, create an alto textblock
                for (int blockNum = 0; blockNum < textblocks.Count; blockNum++)
                {
                    CEOC textblock = textblocks[blockNum];
                    Trace.WriteLine("Processing textblock " + textblock.Eoctype.TypeName + textblock.BoundingBox.ToString());
                    AgoraAlto.TextBlockType altoBlock = new AgoraAlto.TextBlockType();
                    altoBlock.ID = _AltoData.Layout.Page[0].ID + "." + blockNum.ToString();
                    altoBlock.STYLEREFS = textblock.Eoctype.TypeName;
                    altoBlock.HPOS = textblock.BoundingBox.Left;
                    altoBlock.VPOS = textblock.BoundingBox.Top;
                    altoBlock.WIDTH = textblock.BoundingBox.Width;
                    altoBlock.HEIGHT = textblock.BoundingBox.Height;

                    GenerateThumbnailText(altoBlock.ID, textblock.BoundingBox);

                    // Update PrintSpace.HPOS/VPOS/WIDTH/HEIGHT
                    System.Drawing.Rectangle recNewPrintSpace;
                    System.Drawing.Rectangle recBlock = new System.Drawing.Rectangle(
                        altoBlock.HPOS, altoBlock.VPOS,
                        altoBlock.WIDTH, altoBlock.HEIGHT);
                    if ((_AltoData.Layout.Page[0].PrintSpace.WIDTH == 0.0F) &&
                        (_AltoData.Layout.Page[0].PrintSpace.HEIGHT == 0.0F))
                    {
                        recNewPrintSpace = recBlock;
                    }
                    else
                    {
                        System.Drawing.Rectangle recPrintSpace = new System.Drawing.Rectangle(
                            (int)_AltoData.Layout.Page[0].PrintSpace.HPOS,
                            (int)_AltoData.Layout.Page[0].PrintSpace.VPOS,
                            (int)_AltoData.Layout.Page[0].PrintSpace.WIDTH,
                            (int)_AltoData.Layout.Page[0].PrintSpace.HEIGHT);
                        recNewPrintSpace = System.Drawing.Rectangle.Union(recBlock, recPrintSpace);
                    }
                    _AltoData.Layout.Page[0].PrintSpace.HPOS = (float)recNewPrintSpace.Left;
                    _AltoData.Layout.Page[0].PrintSpace.VPOS = (float)recNewPrintSpace.Top;
                    _AltoData.Layout.Page[0].PrintSpace.WIDTH = (float)recNewPrintSpace.Width;
                    _AltoData.Layout.Page[0].PrintSpace.HEIGHT = (float)recNewPrintSpace.Height;

                    // Scan all childs for textlines and store a reference to them
                    List<CEOC> textlines = new List<CEOC>();
                    foreach (CEOC textblockChild in textblock.Childs)
                    {
                        // Is this child must be written as a textline in Alto file ?
                        if (_AltoTextlineEOCTypes.Contains(textblockChild.Eoctype.TypeName) == true)
                        {
                            textlines.Add(textblockChild);
                        }
                    }
                    if (textlines.Count > 0)
                    {
                        textlines.Sort(CompareEOCByBoundingBoxTop);
                        altoBlock.TextLine = new AgoraAlto.TextBlockTypeTextLine[textlines.Count];
                        // For each found textline, create an alto textline
                        for (int lineNum = 0; lineNum < textlines.Count; lineNum++)
                        {
                            CEOC textline = textlines[lineNum];
                            AgoraAlto.TextBlockTypeTextLine altoLine = new AgoraAlto.TextBlockTypeTextLine();
                            altoLine.ID = altoBlock.ID + "." + lineNum.ToString();
                            altoLine.STYLEREFS = textline.Eoctype.TypeName;
                            altoLine.HPOS = textline.BoundingBox.Left;
                            altoLine.VPOS = textline.BoundingBox.Top;
                            altoLine.WIDTH = textline.BoundingBox.Width;
                            altoLine.HEIGHT = textline.BoundingBox.Height;

                            GenerateThumbnailText(altoLine.ID, textline.BoundingBox);

                            // Scan all childs for words and store a reference to them
                            List<CEOC> textwords = new List<CEOC>();
                            foreach (CEOC textlineChild in textline.Childs)
                            {
                                // Is this child must be written as a string in Alto file ?
                                if (_AltoWordEOCTypes.Contains(textlineChild.Eoctype.TypeName) == true)
                                {
                                    textwords.Add(textlineChild);
                                }
                            }
                            if (textwords.Count > 0)
                            {
                                textwords.Sort(CompareEOCByBoundingBoxLeft);
                                // Prepare a string list for final storage in altoLine.String
                                // because Alto doesn't know the "word" structure
                                List<AgoraAlto.StringType> altoStringList = new List<AgoraAlto.StringType>();

                                for (int wordNum = 0; wordNum < textwords.Count; wordNum++)
                                {
                                    CEOC textword = textwords[wordNum];
                                    string wordID = altoLine.ID + "." + wordNum.ToString();

                                    GenerateThumbnailText(wordID, textword.BoundingBox);

                                    // Scan all childs for strings and store a reference to them
                                    List<CEOC> textstrings = new List<CEOC>();
                                    foreach (CEOC textwordChild in textword.Childs)
                                    {
                                        // Is this child must be written as a string in Alto file ?
                                        if (_AltoStringEOCTypes.Contains(textwordChild.Eoctype.TypeName) == true)
                                        {
                                            textstrings.Add(textwordChild);
                                        }
                                    }
                                    if (textstrings.Count > 0)
                                    {
                                        textstrings.Sort(CompareEOCByBoundingBoxLeft);
                                        // For each found textstring, create an alto textstring
                                        for (int stringNum = 0; stringNum < textstrings.Count; stringNum++)
                                        {
                                            CEOC textstring = textstrings[stringNum];

                                            AgoraAlto.StringType altoString = new AgoraAlto.StringType();
                                            altoString.ID = wordID + "." + stringNum.ToString();
                                            altoString.STYLEREFS = textstring.Eoctype.TypeName;
                                            altoString.CONTENT = "*";
                                            altoString.HPOS = textstring.BoundingBox.Left;
                                            altoString.VPOS = textstring.BoundingBox.Top;
                                            altoString.WIDTH = textstring.BoundingBox.Width;
                                            altoString.HEIGHT = textstring.BoundingBox.Height;
                                            altoString.HPOSSpecified = true;
                                            altoString.VPOSSpecified = true;
                                            altoString.HEIGHTSpecified = true;
                                            altoString.WIDTHSpecified = true;

                                            GenerateThumbnailText(altoString.ID, textstring.BoundingBox);

                                            altoStringList.Add(altoString);
                                        }
                                    }
                                }
                                if (altoStringList.Count > 0)
                                {
                                    altoLine.String = altoStringList.ToArray();
                                }
                            }
                            altoBlock.TextLine[lineNum] = altoLine;
                        }
                    }
                    _AltoData.Layout.Page[0].PrintSpace.TextBlock[blockNum] = altoBlock;
                }
            }
            if (illustrations.Count > 0)
            {
                _AltoData.Layout.Page[0].PrintSpace.Illustration =
                    new AgoraAlto.IllustrationType[illustrations.Count];
                for (int illusNum = 0; illusNum < illustrations.Count; illusNum++)
                {
                    Trace.WriteLine("Processing illustration " + illustrations[illusNum].Eoctype.TypeName + illustrations[illusNum].BoundingBox.ToString());

                    AgoraAlto.IllustrationType altoIllustration = new AgoraAlto.IllustrationType();
                    int altoIllustrationNum = illusNum + textblocks.Count;
                    altoIllustration.ID = _AltoData.Layout.Page[0].ID + "." + altoIllustrationNum.ToString();
                    altoIllustration.STYLEREFS = illustrations[illusNum].Eoctype.TypeName;
                    altoIllustration.HPOS = illustrations[illusNum].BoundingBox.Left;
                    altoIllustration.VPOS = illustrations[illusNum].BoundingBox.Top;
                    altoIllustration.WIDTH = illustrations[illusNum].BoundingBox.Width;
                    altoIllustration.HEIGHT = illustrations[illusNum].BoundingBox.Height;

                    GenerateThumbnailIllustration(altoIllustration.ID, illustrations[illusNum].BoundingBox);

                    _AltoData.Layout.Page[0].PrintSpace.Illustration[illusNum] = altoIllustration;
                }
            }
            Trace.WriteLine("Filling alto structure OK.", name);
            return true;
        }

        private bool SaveAltoStructure()
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;

            Trace.WriteLine("Saving ALTO structure...", name);

            string altoBasename = _AltoImageID + ".xml";
            string altoFileName = Path.Combine(GLOBALDATA.ProjectDirectory, Path.Combine(GLOBALDATA.DIR_ALTO, altoBasename));
            Trace.WriteLine("Creating ALTO file " + altoFileName + "...");
            if (File.Exists(altoFileName))
            {
                Trace.WriteLine("Backuping file " + altoFileName + "... ");
                File.Copy(altoFileName, altoFileName + ".bak", true);
                Trace.WriteLine("Backuping OK.");
            }

            XmlSerializer xs = new XmlSerializer(typeof(AgoraAlto.alto));
            StreamWriter wr = new StreamWriter(altoFileName);
            xs.Serialize(wr, _AltoData);
            Trace.WriteLine("Saving ALTO structure OK.", name);
            return true;
        }

        private System.Drawing.Bitmap _SrcImage = null;
        private string _SrcImageFilename = null;
        private int _ThumbnailsCounter = 0;
        private void _GenerateThumbnail(string dstFilename, System.Drawing.Rectangle rect)
        {
            string srcFilename = Path.Combine(
                Path.Combine(GLOBALDATA.ProjectDirectory, GLOBALDATA.DIR_IMAGES),
                _AltoData.Description.sourceImageInformation.fileName);
            if (srcFilename != _SrcImageFilename)
            {
                _SrcImageFilename = new string(srcFilename.ToCharArray());
                _SrcImage = new Bitmap(_SrcImageFilename);
                _SrcImage = _SrcImage.Clone(
                    new Rectangle(0, 0, _SrcImage.Width, _SrcImage.Height),
                    PixelFormat.Format32bppArgb);
                //_SrcImage = AForge.Imaging.Image.Clone(
                //    _SrcImage,
                //    PixelFormat.Format24bppRgb);
            }
            /*
            Bitmap bmpCrop = _SrcImage.Clone(rect, _SrcImage.PixelFormat);
            */
            FastBitmap bmpFast = new FastBitmap(_SrcImage);
            bmpFast.LockImage();
            Bitmap bmpCrop = bmpFast.Clone(rect.X, rect.Y, rect.Width, rect.Height);
            bmpFast.UnlockImage();
            bmpCrop.Save(dstFilename, (_AltoThumbnailsJPEGFormat == true) ? ImageFormat.Jpeg : ImageFormat.Png);
            
            //this.saveJpeg(dstFilename, bmpCrop, 75L);
        }
        private void saveJpeg(string path, Bitmap img, long quality)
        {
            // Encoder parameter for image quality
            EncoderParameter qualityParam = new EncoderParameter(
                System.Drawing.Imaging.Encoder.Quality, quality);

            // Jpeg image codec
            ImageCodecInfo jpegCodec = this.getEncoderInfo("image/jpeg");

            if (jpegCodec == null)
                return;

            EncoderParameters encoderParams = new EncoderParameters(1);
            encoderParams.Param[0] = qualityParam;

            img.Save(path, jpegCodec, encoderParams);
        }
        private ImageCodecInfo getEncoderInfo(string mimeType)
        {
            // Get image codecs for all image formats
            ImageCodecInfo[] codecs = ImageCodecInfo.GetImageEncoders();

            // Find the correct image codec
            for (int i = 0; i < codecs.Length; i++)
                if (codecs[i].MimeType == mimeType)
                    return codecs[i];
            return null;
        }

        private string BuildThumbnailFilename(string ID)
        {
            string prefix = "";
            if (_AltoThumbnailsPrefixOriginalFilename == true)
            {
                prefix = _OriginalImageBasename + ".";
            }
            string extension = GLOBALDATA.IMAGE_OUTPUT_FILE_EXT;
            if (_AltoThumbnailsJPEGFormat == true)
            {
                extension = "jpg";
            }
            string thumbnailFilename = prefix + ID + "." + extension;
            return thumbnailFilename;
        }
        private void GenerateThumbnailText(string ID, Rectangle rect)
        {
            if (AltoGenerateThumbnails == false) return;
            _ThumbnailsCounter++;
            if ((_ThumbnailsCounter % 100) == 0) Trace.WriteLine(_ThumbnailsCounter);
            string subDir = ID.Split(new char[] { '.' })[0];
            string dstDir = Path.Combine(Path.Combine(GLOBALDATA.ProjectDirectory, GLOBALDATA.DIR_ALTO_IMAGES_TEXT), subDir);
            if (!Directory.Exists(dstDir))
            {
                Directory.CreateDirectory(dstDir);
            }
            string filename = Path.Combine(dstDir, this.BuildThumbnailFilename(ID));
            this._GenerateThumbnail(filename, rect);
        }
        private void GenerateThumbnailIllustration(string ID, Rectangle rect)
        {
            if (AltoGenerateThumbnails == false) return;
            string subDir = ID.Split(new char[] { '.' })[0];
            string dstDir = Path.Combine(Path.Combine(GLOBALDATA.ProjectDirectory, GLOBALDATA.DIR_ALTO_IMAGES_ILLUSTRATION), subDir);
            if (!Directory.Exists(dstDir))
            {
                Directory.CreateDirectory(dstDir);
            }
            string filename = Path.Combine(dstDir, this.BuildThumbnailFilename(ID));
            this._GenerateThumbnail(filename, rect);
        }

        //private bool CreateTEIStructure()
        //{
        //}
    }
}
