﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.IO;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;

namespace Agora.Model.Project
{
    public class CProject
    {
        [XmlIgnore]
        public bool NotSaved { get; set; }

        static private string _Directory = null;
        [XmlIgnore]
        static public string Directory
        {
            get { return _Directory; }
            set { _Directory = value; }
        }

        private string _ImagesSubDirectory = null;
        public string ImagesSubDirectory
        {
            get { return _ImagesSubDirectory; }
            set { _ImagesSubDirectory = value; }
        }

        private List<string> _ImageFilenames = new List<string>();
        [XmlIgnore]
        public List<string> ImageFilenames
        {
            get { return _ImageFilenames; }
        }

        private int _CurrentImageNum = -1;
        public int CurrentImageNum
        {
            get { return _CurrentImageNum; }
            set { _CurrentImageNum = value; }
        }

        private string _ScenarioFilename = "";
        public string ScenarioFilename
        {
            get { return _ScenarioFilename; }
            set { _ScenarioFilename = value; }
        }

        private CScenario _Scenario = new CScenario();
        [XmlIgnore]
        public CScenario Scenario
        {
            get { return _Scenario; }
            set { _Scenario = value; }
        }

        public CProject()
        {
        }

        public void Save(string filename)
        {
            XmlSerializer xs = new XmlSerializer(typeof(CProject));
            StreamWriter wr = new StreamWriter(filename);
            xs.Serialize(wr, this);
            wr.Close();
        }
        public static CProject Load(string filename)
        {
            CProject result;
            XmlSerializer xs = new XmlSerializer(typeof(CProject));
            StreamReader rd = new StreamReader(filename);
            result = (CProject)xs.Deserialize(rd);
            rd.Close();
            result.SetDirectory(System.IO.Path.GetDirectoryName(filename));
            result.SetImagesSubDirectory(result.ImagesSubDirectory);
            result.SetCurrentImage(result.ImageFilenames[result.CurrentImageNum]);
            result.ScenarioLoad(result.ScenarioFilename);
            return result;
        }

        public void SetDirectory(string dir)
        {
            _Directory = dir;
            GLOBALDATA.ProjectDirectory = _Directory;
        }
        public string GetDirectory()
        {
            return _Directory;
        }
        public void SetImagesSubDirectory(string dir)
        {
            _ImagesSubDirectory = dir;
            GLOBALDATA.ImagesSubDirectory = _ImagesSubDirectory;
            List<string> filenamesList = new List<string>();
            string[] extensionsList = GLOBALDATA.IMAGE_INPUT_FILE_EXT.Split(new char[] { ',' });
            foreach (string extension in extensionsList)
            {
                string[] filenames =
                    System.IO.Directory.GetFiles(
                        System.IO.Path.Combine(_Directory, _ImagesSubDirectory),
                        "*." + extension);
                filenamesList.AddRange(filenames);
            }
            this.SetImages(filenamesList);
        }
        public string GetImagesSubDirectory()
        {
            return _ImagesSubDirectory;
        }
        public void SetImages(List<string> filenames)
        {
            _ImageFilenames.Clear();
            _ImageFilenames.AddRange(filenames);
        }

        public void SetCurrentImage(string filename)
        {
            if (_ImageFilenames.Contains(filename) == true)
            {
                this.SetCurrentImage(_ImageFilenames.IndexOf(filename));
            }
        }

        /// <summary>
        /// Start a new scenario with an image.
        /// </summary>
        /// <param name="numImage"></param>
        public void SetCurrentImage(int numImage)
        {
            _CurrentImageNum = numImage;
            GLOBALDATA.CurrentImageNum = _CurrentImageNum;
            GLOBALDATA.CurrentImageFilename = _ImageFilenames[_CurrentImageNum];
            this.ScenarioInit();
        }

        /// <summary>
        /// Save scenario in the same directory as project
        /// </summary>
        /// <param name="filename"></param>
        public void ScenarioSave(string filename)
        {
            _ScenarioFilename = System.IO.Path.GetFileName(filename);
            _Scenario.Save(System.IO.Path.Combine(_Directory, filename));
        }
        /// <summary>
        /// Load scenario from the same directory as project
        /// </summary>
        /// <param name="filename"></param>
        public void ScenarioLoad(string filename)
        {
            string basename = System.IO.Path.GetFileName(filename);
            string scenarioFile = System.IO.Path.Combine(_Directory, basename);
            if (File.Exists(scenarioFile) == true)
            {
                _ScenarioFilename = basename;
                _Scenario = CScenario.Load(scenarioFile);
                this.ScenarioInit();
                this.ScenarioApply();
            }
        }
        public void ScenarioInit()
        {
            _Scenario.Init(_ImageFilenames[_CurrentImageNum]);
        }
        public void ScenarioClear()
        {
            _Scenario.ClearSteps();
        }
        public void ScenarioApply()
        {
            //this.ScenarioInit();
            _Scenario.Apply();
        }

        /// <summary>
        /// Apply scenario to all the images
        /// </summary>
        public void Apply()
        {
            GLOBALDATA.CleanDestinationDirectories();
            for (int imageNum = 0; imageNum < _ImageFilenames.Count; imageNum++)
            {
                this.SetCurrentImage(imageNum);
                Trace.WriteLine("Apply scenario to " + _ImageFilenames[_CurrentImageNum] + " (" + System.DateTime.Now.ToString() + ")");
                this.ScenarioApply();
            }
        }
    }
}
