﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;

namespace Agora.Model.Utils
{
    // Class for enumeration of all pairing between
    // srcNode.childs and dstNode.childs
    // ALL srcNode.childs --> SOME of dstNode.childs
    public class CArrangement
    {
        private CEOC _SrcNode, _DstNode;
        private LinkedList<CEOC> _Candidates;
        private CNeighbourMap _Map;

        public CEOC SrcNode
        {
            get { return _SrcNode; }
        }
        public CEOC DstNode
        {
            get { return _DstNode; }
        }

        public CArrangement(CEOC srcNode, CEOC dstNode, CNeighbourMap map)
        {
            _SrcNode = srcNode; _DstNode = dstNode;
            _Candidates = null;
            _Map = map;
            // Remove old pairings if they exist
            for (LinkedListNode<CEOC> v = _SrcNode.Childs.First; v != null; v = v.Next)
            {
                v.Value.UnpairCandidate();
                v.Value.PairingRemove();
            }
        }

        public void PairingStore()
        {
            _SrcNode.PairingStore();
            LinkedListNode<CEOC> src = _SrcNode.Childs.First;
            while (src != null)
            {
                // stores the ACTUAL node (not candidates' node which is only temporary
                // if we used neighbors map) of paired EOC
                LinkedListNode<CEOC> dst = src.Value.PairedWithCandidate.Value.Node;
                src.Value.PairingStore(dst);
                src = src.Next;
            }
        }
        public void PairingRemove()
        {
            //LinkedListNode<CEOC> src = _SrcNode.Childs.First.Next;
            LinkedListNode<CEOC> src = _SrcNode.Childs.First;
            while (src != null)
            {
                src.Value.PairingRemove();
                src = src.Next;
            }
            _SrcNode.PairingRemove();
        }

        public bool GetFirst()
        {
            _Candidates = _DstNode.Childs;

            LinkedListNode<CEOC> src, dst;
            src = _SrcNode.Childs.First;
            while (src != null)
            {
                if (src.Value.PairedWithCandidate == null)
                {
                    // src not paired yet -> start a the beginning
                    // of the candidates
                    dst = _Candidates.First;
                }
                else
                {
                    // unpair src and dst candidate
                    dst = src.Value.PairedWithCandidate;
                    src.Value.UnpairCandidate();

                    // get next candidate
                    dst = dst.Next;
                }

                // dst is now the next candidate to try
                while (dst != null)
                {
                    if ((dst.Value.PairedWithCandidate == null) && 
                        (dst.Value.PairedWith == null) &&
                        (src.Value.IsMatching(dst.Value) == true))
                    {
                        // ok candidate is pairable : pair it
                        src.Value.PairWithCandidate(dst);
                        // if neighbour map is used, then adjust candidates list
                        if ((_Map != null) && (src == _SrcNode.Childs.First))
                        {
                            _Candidates = _Map.GetNeighbors(dst.Value);
                        }
                        break;
                    }
                    else
                    {
                        // src and dst doesn't match -> try with next candidate
                        dst = dst.Next;
                    }
                }

                if (src.Value.PairedWithCandidate != null)
                {
                    // a pairing could be found -> try to pair next src
                    src = src.Next;
                    if (src != null)
                    {
                        // for restarting at the beginning of candidates
                        src.Value.UnpairCandidate();// .PairedWithCandidate = null;
                    }
                }
                else
                {
                    // a pairing could not be found -> must modify pairing for previous src
                    src = src.Previous;
                    if (src == null)
                    {
                        // Problem : all possibilities exhausted -> no pairing found
                        return false;
                    }
                }
            }
            // we are here because we succeded in pairing all src --> ok
            return true;
        }

        public bool GetNext()
        {
            LinkedListNode<CEOC> src, dst;
            src = _SrcNode.Childs.Last;
            while (src != null)
            {
                if (src.Value.PairedWithCandidate == null)
                {
                    dst = _Candidates.First;
                }
                else
                {
                    // unpair src and dst
                    dst = src.Value.PairedWithCandidate;
                    src.Value.UnpairCandidate();

                    // get next candidate dst
                    dst = dst.Next;
                }

                while (dst != null)
                {
                    if ((dst.Value.PairedWithCandidate == null) && 
                        (dst.Value.PairedWith == null) &&
                        (src.Value.IsMatching(dst.Value) == true))
                    {
                        // ok candidate is pairable : pair it
                        src.Value.PairWithCandidate(dst);
                        // if neighbour map is used, then adjust candidates list
                        if ((_Map != null) && (src == _SrcNode.Childs.First))
                        {
                            _Candidates = _Map.GetNeighbors(dst.Value);
                        }
                        break;
                    }
                    else
                    {
                        // src and dst doesn't match -> try with next dst
                        dst = dst.Next;
                    }
                }
                if (src.Value.PairedWithCandidate != null)
                {
                    // a pairing was found -> go to next src
                    src = src.Next;
                    if (src != null)
                    {
                        // for restarting at the beginning of candidates
                        src.Value.UnpairCandidate(); // .PairedWithCandidate = null;
                    }
                }
                else
                {
                    // exhausted DstNode's childs, must try other combinations for previous pairing
                    src = src.Previous;
                    if (src == null)
                    {
                        // all possibilities exhausted -> no pairing found
                        return false;
                    }
                }
            }
            return true;
        }
    }
}
