﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using MathNet.Numerics.LinearAlgebra;

namespace Agora.Model.Utils
{
    public class CLinearRegressionResults
    {
        public double a = 0.0;
        public double b = 0.0;
        public double r2 = 0.0;
        public CLinearRegressionResults()
        {
        }
        public CLinearRegressionResults(double a, double b, double r2)
        {
            this.a = a;
            this.b = b;
            this.r2 = r2;
        }
    }

    public class CLinearRegression
    {
        public CLinearRegression()
        {
        }
        static public CLinearRegressionResults Solve(
            Vector y, Vector x, out Vector b)
        {
            Vector ones = Vector.Ones(x.Length);
            Matrix X = Matrix.CreateFromColumns(new List<Vector> { ones, x });
            //Trace.Assert((X.RowCount > 1) && (X.Determinant() != 0.0));
            
            Matrix Y = Matrix.CreateFromColumns(new List<Vector> { y });
            Matrix B = X.Solve(Y);
            b = B.GetColumnVector(0);
            // compute coefficient de determination R2
            double yMean = y.Average();
            Vector yEstim = Vector.Create(x);
            yEstim.ScaleInplace(b[1]);
            yEstim.AddInplace(b[0]);
            yEstim.SubtractInplace(yMean);
            Vector yTrue = Vector.Create(y);
            yTrue.SubtractInplace(yMean);
            double SSe = yEstim.SquaredNorm();
            double SSt = yTrue.SquaredNorm();
            double r2;
            if (SSt == 0.0)
                r2 = 1.0;
            else
                r2 = SSe / SSt;
            CLinearRegressionResults results =
                new CLinearRegressionResults(b[1], b[0], r2);
            
            return (results);
        }
    }
}
