﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;

namespace Agora.Model.Utils
{
    /// <summary>
    /// This class has 2 goals :
    /// 1. offering general services for preparing exportation
    /// of the results (creating directory structure, etc.) of a whole project,
    /// thanks to static methods and properties
    /// 2. storing various global data of AGORA during execution
    /// </summary>
    public class GLOBALDATA
    {
        public const string DIR_IMAGES = "images";
        public const string DIR_ALTO = "alto";
        public const string DIR_ALTO_IMAGES_TEXT = DIR_ALTO + "\\images_text";
        public const string DIR_ALTO_IMAGES_ILLUSTRATION = DIR_ALTO + "\\images_illustration";
        public const string DIR_TEI = "tei";
        public const string DIR_METS = "mets";
        public const string IMAGE_FILE_ID = "{0:00000}";
        public const string IMAGE_INPUT_FILE_EXT = "png,jpg,jpeg,tif,tiff";
        public const string IMAGE_OUTPUT_FILE_EXT = "png";
        public const string NOID = "#";

        public static string ProjectDirectory = null;
        public static string ImagesSubDirectory = null;
        public static string CurrentImageFilename = null;
        public static int CurrentImageNum = -1;
        public static List<string> ScenarioSteps = new List<string>();

        public static int RANGEMAP_LOOK_X = 20;
        public static int RANGEMAP_LOOK_Y = 20;
        public static double RANGEMAP_NORMALIZE_MIN = 0.0;
        public static double RANGEMAP_NORMALIZE_MAX = 10.0;
        public static CDimensionType[] DIMENSIONS = null;

        /// <summary>
        /// Checks if dir exists (creates it otherwise)
        /// Then cleans this dir
        /// </summary>
        /// <param name="dir"></param>
        /// <returns>true = ok</returns>
        private static bool CheckCreateDirectory(string dir)
        {
            Trace.WriteLine("Checking " + dir + "...");
            bool result = true;
            if (!Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }
            result &= Directory.Exists(dir);
            return result;
        }

        /// <summary>
        /// Checks if dir exists (creates it otherwise)
        /// Then cleans this dir
        /// </summary>
        /// <param name="dir"></param>
        /// <returns>true = ok</returns>
        private static bool CleanDirectory(string dir)
        {
            Trace.WriteLine("Cleaning " + dir + "...");
            bool result = true;
            result = CheckCreateDirectory(dir);
            if (result == true)
            {
                foreach (string filename in Directory.GetFiles(dir))
                {
                    try
                    {
                        File.Delete(filename);
                    }
                    catch (IOException ex)
                    {
                        Trace.WriteLine("WARNING:" + ex.Message);
                    }
                }
                foreach (string directory in Directory.GetDirectories(dir))
                {
                    try
                    {
                        foreach (string filename in Directory.GetFiles(directory))
                        {
                            try
                            {
                                File.Delete(filename);
                            }
                            catch (IOException ex)
                            {
                                Trace.WriteLine("WARNING:" + ex.Message);
                            }
                        }
                        Directory.Delete(directory);
                    }
                    catch (IOException ex)
                    {
                        Trace.WriteLine("WARNING:" + ex.Message);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Checks directories
        /// </summary>
        /// <returns>true = ok</returns>
        public static bool CheckDestinationDirectories()
        {
            bool result = true;
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_IMAGES));
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO));
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO_IMAGES_TEXT));
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO_IMAGES_ILLUSTRATION));
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_TEI));
            result &= CheckCreateDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_METS));
            return result;
        }

        public static bool CleanDestinationDirectories()
        {
            bool result = true;
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_IMAGES));
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO));
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO_IMAGES_TEXT));
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_ALTO_IMAGES_ILLUSTRATION));
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_TEI));
            result &= CleanDirectory(Path.Combine(GLOBALDATA.ProjectDirectory, DIR_METS));
            return result;
        }
    }
}
