﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace GUI
{
    public class CFormUtilities
    {
        static public void SetComboBoxLabel(ComboBox cbbx, List<string> labels, string selected)
        {
            cbbx.Items.Clear();
            foreach (string label in labels)
            {
                cbbx.Items.Add(label);
            }
            cbbx.SelectedItem = selected;
        }
        /// <summary>
        /// Store a link to CFeatureMatrixComparator by using Tag of button
        /// </summary>
        /// <param name="dgv"></param>
        /// <param name="comparator"></param>
        /// <param name="row"></param>
        /// <param name="col"></param>
        static public void SetDGVComparatorsButtonValue(
            DataGridView dgv, CFeatureMatrixComparator comparator,
            int row, int col)
        {
            CFeatureMatrixComparator defaultMC = new CFeatureMatrixComparator();
            DataGridViewButtonCell btCell = (DataGridViewButtonCell)dgv.Rows[row].Cells[col];
            btCell.Tag = comparator;
            if (comparator.Compare(defaultMC) == true)
            {
                btCell.Value = Double.PositiveInfinity.ToString();
            }
            else
            {
                btCell.Value = "...";
            }
        }
        /// <summary>
        /// Stores links to each CFeatureMatrixComparator by using Tag of each button
        /// </summary>
        /// <param name="dgv"></param>
        /// <param name="rowHeaders"></param>
        /// <param name="comparators"></param>
        static public void SetDGVComparators(DataGridView dgv, bool rowHeaders, CFeaturesComparator[] comparators)
        {
            int nbCols = comparators.Length;
            int nbRows = comparators[0].Comparators.Length;

            DataGridViewCellStyle dgvCellStyle = new DataGridViewCellStyle();
            dgvCellStyle.Alignment = System.Windows.Forms.DataGridViewContentAlignment.MiddleCenter;
            dgvCellStyle.BackColor = System.Drawing.SystemColors.Control;
            dgvCellStyle.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)), true);
            dgvCellStyle.ForeColor = System.Drawing.SystemColors.WindowText;
            dgvCellStyle.SelectionBackColor = System.Drawing.SystemColors.Highlight;
            dgvCellStyle.SelectionForeColor = System.Drawing.SystemColors.HighlightText;
            dgvCellStyle.WrapMode = System.Windows.Forms.DataGridViewTriState.True;
            dgv.ColumnHeadersDefaultCellStyle = dgvCellStyle;
            
            dgv.RowHeadersVisible = rowHeaders;
            dgv.ColumnHeadersVisible = true;
            dgv.Rows.Clear();
            dgv.Columns.Clear();

            for (int i = 0; i < nbCols; i++)
            {
                DataGridViewButtonColumn btColumn = new DataGridViewButtonColumn();
                btColumn.HeaderText = "Ch" + i.ToString();
                btColumn.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                btColumn.FlatStyle = FlatStyle.Standard;
                btColumn.CellTemplate = new DataGridViewButtonCell();
                dgv.Columns.Add(btColumn);
            }

            dgv.AllowUserToOrderColumns = false;
            dgv.ColumnHeadersHeight = 20;
            dgv.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;

            dgv.Rows.Add(nbRows);
            dgv.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;
            dgv.AllowUserToAddRows = false;
            dgv.AllowUserToDeleteRows = false;
            if (rowHeaders == true)
                dgv.RowHeadersWidthSizeMode = DataGridViewRowHeadersWidthSizeMode.AutoSizeToAllHeaders;

            for (int i = 0; i < nbRows; i++)
            {
                dgv.Rows[i].Height = 20;
                dgv.Rows[i].DefaultCellStyle.Padding = new Padding(0);
                if (rowHeaders == true) dgv.Rows[i].HeaderCell.Value = GLOBALDATA.DIMENSIONS[i].ToString();
                for (int j = 0; j < nbCols; j++)
                {
                    CFormUtilities.SetDGVComparatorsButtonValue(dgv, comparators[j].Comparators[i], i, j);
                }
            }
        }
        /// <summary>
        /// Store a link to each double[] by using Tag of each column
        /// </summary>
        /// <param name="dgv"></param>
        /// <param name="rowHeaders"></param>
        /// <param name="data"></param>
        static public void SetDGVMatrixOfDouble(DataGridView dgv, bool rowHeaders, double[][] data)
        {
            int nbCols = data.Length;
            int nbRows = data[0].Length;

            DataGridViewCellStyle dgvCellStyle = new DataGridViewCellStyle();
            dgvCellStyle.Alignment = System.Windows.Forms.DataGridViewContentAlignment.MiddleCenter;
            dgvCellStyle.BackColor = System.Drawing.SystemColors.Control;
            dgvCellStyle.Font = new System.Drawing.Font("Microsoft Sans Serif", 8.25F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)), true);
            dgvCellStyle.ForeColor = System.Drawing.SystemColors.WindowText;
            dgvCellStyle.SelectionBackColor = System.Drawing.SystemColors.Highlight;
            dgvCellStyle.SelectionForeColor = System.Drawing.SystemColors.HighlightText;
            dgvCellStyle.WrapMode = System.Windows.Forms.DataGridViewTriState.True;
            dgv.ColumnHeadersDefaultCellStyle = dgvCellStyle;

            dgv.RowHeadersVisible = rowHeaders;
            dgv.ColumnHeadersVisible = true;
            dgv.Rows.Clear();
            dgv.Columns.Clear();

            dgv.ColumnCount = nbCols;
            dgv.AllowUserToOrderColumns = false;
            dgv.ColumnHeadersHeight = 20;
            dgv.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            dgv.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;

            dgv.Rows.Add(nbRows);
            dgv.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;
            dgv.AllowUserToAddRows = false;
            dgv.AllowUserToDeleteRows = false;
            if (rowHeaders == true) 
                dgv.RowHeadersWidthSizeMode = DataGridViewRowHeadersWidthSizeMode.AutoSizeToAllHeaders;

            for (int i = 0; i < nbCols; i++)
            {
                dgv.Columns[i].HeaderText = i.ToString();
                dgv.Columns[i].Tag = data[i];
                for (int j = 0; j < nbRows; j++)
                {
                    dgv.Rows[j].Height = 20;
                    dgv.Rows[j].DefaultCellStyle.Padding = new Padding(0);
                    if (rowHeaders == true)
                        dgv.Rows[j].HeaderCell.Value = GLOBALDATA.DIMENSIONS[j].ToString();
                    if (data[i] != null)
                    {
                        dgv.Rows[j].Cells[i].Value = data[i][j].ToString();
                    }
                    else
                    {
                        dgv.Rows[j].Cells[i].Value = 0.0;
                    }
                }
            }
        }
    }
}
