﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;

namespace GUI.L1Operations
{
    public partial class FeatureMode : Form
    {
        /// <summary>
        /// Properties to set/get
        /// </summary>
        public CFeatureMode[] Modes { get; set; }
        public int NbDimensions { get; set; }

        public FeatureMode()
        {
            InitializeComponent();
        }
        private void SetDefaultPropertiesValues()
        {
            Trace.Assert(Modes.Length == NbDimensions);
            for (int i = 0; i < NbDimensions; i++)
            {
                Modes[i] = new CFeatureMode();
            }
        }

        private void SetControlsValues()
        {
            // Modes
            dgvModes.RowHeadersVisible = true;
            dgvModes.ColumnHeadersVisible = true;
            dgvModes.Rows.Clear();
            dgvModes.Columns.Clear();

            DataGridViewComboBoxColumn cbbColumnA = new DataGridViewComboBoxColumn();
            cbbColumnA.HeaderText = "C";
            cbbColumnA.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            cbbColumnA.FlatStyle = FlatStyle.Standard;
            cbbColumnA.CellTemplate = new DataGridViewComboBoxCell();

            DataGridViewComboBoxColumn cbbColumnB = new DataGridViewComboBoxColumn();
            cbbColumnB.HeaderText = "C";
            cbbColumnB.AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
            cbbColumnB.FlatStyle = FlatStyle.Standard;
            cbbColumnB.CellTemplate = new DataGridViewComboBoxCell();

            dgvModes.Columns.Add(cbbColumnA);
            dgvModes.Columns.Add(cbbColumnB);
            //dgvModes.ColumnCount = 2;
            dgvModes.AllowUserToOrderColumns = false;
            dgvModes.ColumnHeadersHeight = 20;
            dgvModes.ColumnHeadersHeightSizeMode = DataGridViewColumnHeadersHeightSizeMode.DisableResizing;
            dgvModes.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
            dgvModes.Columns[0].HeaderText = "aMode";
            dgvModes.Columns[1].HeaderText = "bMode";

            dgvModes.Rows.Add(NbDimensions);
            dgvModes.AutoSizeRowsMode = DataGridViewAutoSizeRowsMode.None;
            dgvModes.AllowUserToAddRows = false;
            dgvModes.AllowUserToDeleteRows = false;
            dgvModes.RowHeadersWidthSizeMode = DataGridViewRowHeadersWidthSizeMode.AutoSizeToAllHeaders;
            for (int i = 0; i < NbDimensions; i++)
            {
                dgvModes.Rows[i].Height = 20;
                dgvModes.Rows[i].DefaultCellStyle.Padding = new Padding(0);
                dgvModes.Rows[i].HeaderCell.Value = GLOBALDATA.DIMENSIONS[i].ToString();
                dgvModes.Rows[i].Cells[0].Value = Modes[i].aMode.ToString();
                dgvModes.Rows[i].Cells[1].Value = Modes[i].bMode.ToString();
                 
                DataGridViewComboBoxCell cbbCell0 = (DataGridViewComboBoxCell)dgvModes.Rows[i].Cells[0];
                DataGridViewComboBoxCell cbbCell1 = (DataGridViewComboBoxCell)dgvModes.Rows[i].Cells[1];
                foreach (string item in Enum.GetNames(typeof(CFeatureMode.SetFeatureMode)))
                {
                    cbbCell0.Items.Add(item);
                    cbbCell1.Items.Add(item);
                }
            }
        }

        private void dgvModes_CurrentCellDirtyStateChanged(object sender, EventArgs e)
        {
            if (dgvModes.IsCurrentCellDirty)
            {
                dgvModes.CommitEdit(DataGridViewDataErrorContexts.Commit);
            }
        }

        private void FeatureMode_Shown(object sender, EventArgs e)
        {
            for (int i = 0; i < NbDimensions; i++)
            {
                if (Modes[i] == null) SetDefaultPropertiesValues();
            }
            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            string content;
            for (int i = 0; i < NbDimensions; i++)
            {
                content = dgvModes.Rows[i].Cells[0].Value.ToString();
                Modes[i].aMode = CFeatureMode.FromString(content);
                content = dgvModes.Rows[i].Cells[1].Value.ToString();
                Modes[i].bMode = CFeatureMode.FromString(content);
            }
            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }
    }
}
