﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;

namespace GUI.L1Operations
{
    public partial class FindCC : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public List<string> Labels { get; set; }

        /// <summary>
        /// Properties to set BEFORE showing form
        /// And to get AFTER closing form
        /// </summary>
        public Agora.Model.OperatorLevel1.CFindCC Operator { get; set; }

        public FindCC()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }

        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeParent = this.Labels[0];
            Operator.EOCTypeNewCC = this.Labels[0];
            Operator.BinarizationBasicThreshold = 150;
            Operator.BinarizationSauvolaThresholdWindow = 12;
            Operator.BinarizationSauvolaThresholdKValue = 0.5F;
            Operator.BinarizationSauvolaThresholdRValue = 128.0F;
            Operator.BinarizationUseSauvola = false;
            Operator.RangeMapLookX = 1;
            Operator.RangeMapLookY = 1;
            Operator.ExtendedBoundingBox = false;
        }

        private void SetControlsValues()
        {
            // Labels
            CFormUtilities.SetComboBoxLabel(cbNewCCLabel, Labels, Operator.EOCTypeNewCC);
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);

            // Textboxes
            tbThreshold.Text = Operator.BinarizationBasicThreshold.ToString();
            tbWindow.Text = Operator.BinarizationSauvolaThresholdWindow.ToString();
            tbKValue.Text = Operator.BinarizationSauvolaThresholdKValue.ToString();
            tbRValue.Text = Operator.BinarizationSauvolaThresholdRValue.ToString();
            tbLookX.Text = Operator.RangeMapLookX.ToString();
            tbLookY.Text = Operator.RangeMapLookY.ToString();

            // Radio buttons
            if (Operator.BinarizationUseSauvola == false)
            {
                if (rbThreshold.Checked == false) rbThreshold.Checked = true;
                if (rbSauvola.Checked == true) rbSauvola.Checked = false;
                tbThreshold.Enabled = true;
                gbParameters.Enabled = false;
            }
            else
            {
                if (rbThreshold.Checked == true) rbThreshold.Checked = false;
                if (rbSauvola.Checked == false) rbSauvola.Checked = true;
                tbThreshold.Enabled = false;
                gbParameters.Enabled = true;
            }

            // Checkboxes
            cbExtendedBoundingBox.Checked = Operator.ExtendedBoundingBox;
        }

        private void FindCC_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeNewCC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // UseSauvola Flag : set by radiobutton event

            // Values : set by textboxes events

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void cbExtendedBoundingBox_CheckedChanged(object sender, EventArgs e)
        {
            Operator.ExtendedBoundingBox = cbExtendedBoundingBox.Checked;
            this.SetControlsValues();
        }

        private void rbThreshold_CheckedChanged(object sender, EventArgs e)
        {
            Operator.BinarizationUseSauvola = !rbThreshold.Checked;
            this.SetControlsValues();
        }

        private void rbSauvola_CheckedChanged(object sender, EventArgs e)
        {
            Operator.BinarizationUseSauvola = rbSauvola.Checked;
            this.SetControlsValues();
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }

        private void cbNewCCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewCC = String.Copy(cbNewCCLabel.SelectedItem.ToString());
        }

        private void tbLookX_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbLookX.Text, out result) == true)
            {
                Operator.RangeMapLookX = (int)result;
            }
            this.SetControlsValues();
        }

        private void tbLookY_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbLookY.Text, out result) == true)
            {
                Operator.RangeMapLookY = (int)result;
            }
            this.SetControlsValues();
        }

        private void tbThreshold_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbThreshold.Text, out result) == true)
            {
                Operator.BinarizationBasicThreshold = (int)result;
            }
            this.SetControlsValues();
        }

        private void tbWindow_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbWindow.Text, out result) == true)
            {
                Operator.BinarizationSauvolaThresholdWindow = (int)result;
            }
            this.SetControlsValues();
        }

        private void tbKValue_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbKValue.Text, out result) == true)
            {
                Operator.BinarizationSauvolaThresholdKValue = (float)result;
            }
            this.SetControlsValues();
        }

        private void tbRValue_TextChanged(object sender, EventArgs e)
        {
            double result;
            if (Double.TryParse(tbRValue.Text, out result) == true)
            {
                Operator.BinarizationSauvolaThresholdRValue = (float)result;
            }
            this.SetControlsValues();
        }

    }
}
