﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace GUI.L1Operations
{
    public partial class Insert : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public List<string> Labels { get; set; }
        public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CInsert Operator { get; set; }

        public Insert()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }

        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeChilds = new string[] { this.Labels[0] };
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.EOCTypeParent = this.Labels[0];
            Operator.Evaluator.EnableRotation = false;
            Operator.Evaluator.EnableScaling = false;
            Operator.Evaluator.EnableTranslation = false;
            Operator.Features = CFeatures.BuildArray(1, NbDimensions);
            Operator.Evaluator.Coefficients = new double[1][] { new double[NbDimensions] };
            Operator.Evaluator.PatternGraphComparators = null;
            Operator.Evaluator.ParentGraphComparators = null;
            Operator.Evaluator.GraphGraphComparators = null;
            Operator.Neighbouroud = null;
            Operator.Modes = CFeatureMode.BuildArray(NbDimensions);
            Operator.Loop = false;
        }

        #region Controls' Events

        private void cbxUsePatternGraphComparators_CheckedChanged(object sender, EventArgs e)
        {
            if (cbxUsePatternGraphComparators.Checked == true)
            {
                if (Operator.Evaluator.PatternGraphComparators == null)
                {
                    int nbChilds = Operator.EOCTypeChilds.Length;
                    Operator.Features = CFeatures.BuildArray(nbChilds, NbDimensions);
                    Operator.Evaluator.PatternGraphComparators = CFeaturesComparator.BuildArray(nbChilds, NbDimensions);
                }
            }
            else
            {
                Operator.Evaluator.PatternGraphComparators = null;
            }
            this.SetControlsValues();
        }

        private void cbxUseParentGraphComparators_CheckedChanged(object sender, EventArgs e)
        {
            if (cbxUseParentGraphComparators.Checked == true)
            {
                if (Operator.Evaluator.ParentGraphComparators == null)
                {
                    int nbChilds = Operator.EOCTypeChilds.Length;
                    Operator.Evaluator.ParentGraphComparators = CFeaturesComparator.BuildArray(nbChilds, NbDimensions);
                }
            }
            else
            {
                Operator.Evaluator.ParentGraphComparators = null;
            }
            this.SetControlsValues();
        }

        private void cbxUseGraphGraphComparators_CheckedChanged(object sender, EventArgs e)
        {
            if (cbxUseGraphGraphComparators.Checked == true)
            {
                if (Operator.Evaluator.GraphGraphComparators == null)
                {
                    int nbChilds = Operator.EOCTypeChilds.Length;
                    Operator.Evaluator.GraphGraphComparators = CFeaturesComparator.BuildMatrix(nbChilds, NbDimensions);
                }
            }
            else
            {
                Operator.Evaluator.GraphGraphComparators = null;
            }
            this.SetControlsValues();
        }

        private void cbxUseNeighbouroud_CheckedChanged(object sender, EventArgs e)
        {
            if (cbxUseNeighbouroud.Checked == true)
            {
                Operator.Neighbouroud = new double[NbDimensions];
            }
            else
            {
                Operator.Neighbouroud = null;
            }
            this.SetControlsValues();
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }

        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }

        private void cbChildLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            ComboBox cbbx = (ComboBox)sender;
            int numChild = (int)cbbx.Tag;
            Operator.EOCTypeChilds[numChild] = String.Copy(cbbx.SelectedItem.ToString());
        }
        private void buttonCoefAdd_Click(object sender, EventArgs e)
        {
            int nbCoef = Operator.Evaluator.Coefficients.Length;
            double[][] coefficients = new double[nbCoef + 1][];
            for (int i = 0; i < nbCoef; i++)
            {
                coefficients[i] = new double[NbDimensions];
                for (int j = 0; j < NbDimensions; j++)
                {
                    coefficients[i][j] = Operator.Evaluator.Coefficients[i][j];
                }
            }
            coefficients[nbCoef] = new double[NbDimensions];
            Operator.Evaluator.Coefficients = coefficients;
            this.SetControlsValues();
        }

        private void buttonCoefDel_Click(object sender, EventArgs e)
        {
            int nbCoef = Operator.Evaluator.Coefficients.Length;
            if (nbCoef > 1)
            {
                double[][] coefficients = new double[nbCoef - 1][];
                for (int i = 0; i < (nbCoef - 1); i++)
                {
                    coefficients[i] = new double[NbDimensions];
                    for (int j = 0; j < NbDimensions; j++)
                    {
                        coefficients[i][j] = Operator.Evaluator.Coefficients[i][j];
                    }
                }
                Operator.Evaluator.Coefficients = coefficients;
                this.SetControlsValues();
            }
        }

        private void buttonSetFeaturesModes_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.FeatureMode frm = new GUI.L1Operations.FeatureMode();
            frm.NbDimensions = this.NbDimensions;
            frm.Modes = Operator.Modes;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
            }
        }

        private void buttonAddChild_Click(object sender, EventArgs e)
        {
            int nbChilds = Operator.EOCTypeChilds.Length;
            string[] newEOCTypeChilds = new string[nbChilds + 1];
            CFeatures[] newFeatures = CFeatures.BuildArray(nbChilds + 1, NbDimensions);
            CFeaturesComparator[] newPatternGraphComparators = null;
            if (Operator.Evaluator.PatternGraphComparators != null)
                newPatternGraphComparators = CFeaturesComparator.BuildArray(nbChilds + 1, NbDimensions);
            CFeaturesComparator[] newParentGraphComparators = null;
            if (Operator.Evaluator.ParentGraphComparators != null)
                newParentGraphComparators = CFeaturesComparator.BuildArray(nbChilds + 1, NbDimensions);
            CFeaturesComparator[][] newGraphGraphComparators = null;
            if (Operator.Evaluator.GraphGraphComparators != null)
                newGraphGraphComparators = CFeaturesComparator.BuildMatrix(nbChilds + 1, NbDimensions);
            for (int i = 0; i < nbChilds; i++)
            {
                newEOCTypeChilds[i] = Operator.EOCTypeChilds[i];
                newFeatures[i] = Operator.Features[i];
                if (newPatternGraphComparators != null)
                    newPatternGraphComparators[i] = Operator.Evaluator.PatternGraphComparators[i];
                if (newParentGraphComparators != null)
                    newParentGraphComparators[i] = Operator.Evaluator.ParentGraphComparators[i];
                for (int j = 0; j < nbChilds; j++)
                {
                    if (newGraphGraphComparators != null)
                        newGraphGraphComparators[i][j] = Operator.Evaluator.GraphGraphComparators[i][j];
                }
            }
            Operator.EOCTypeChilds = newEOCTypeChilds;
            Operator.EOCTypeChilds[nbChilds] = this.Labels[0];
            Operator.Features = newFeatures;
            Operator.Evaluator.PatternGraphComparators = newPatternGraphComparators;
            Operator.Evaluator.ParentGraphComparators = newParentGraphComparators;
            Operator.Evaluator.GraphGraphComparators = newGraphGraphComparators;
            this.SetControlsValues();
        }

        private void buttonDelChild_Click(object sender, EventArgs e)
        {
            int nbChilds = Operator.EOCTypeChilds.Length;
            string[] newEOCTypeChilds = new string[nbChilds - 1];
            CFeatures[] newFeatures = CFeatures.BuildArray(nbChilds - 1, NbDimensions);
            CFeaturesComparator[] newPatternGraphComparators = null;
            if (Operator.Evaluator.PatternGraphComparators != null)
                newPatternGraphComparators = CFeaturesComparator.BuildArray(nbChilds - 1, NbDimensions);
            CFeaturesComparator[] newParentGraphComparators = null;
            if (Operator.Evaluator.ParentGraphComparators != null)
                newParentGraphComparators = CFeaturesComparator.BuildArray(nbChilds - 1, NbDimensions);
            CFeaturesComparator[][] newGraphGraphComparators = null;
            if (Operator.Evaluator.GraphGraphComparators != null)
                newGraphGraphComparators = CFeaturesComparator.BuildMatrix(nbChilds - 1, NbDimensions);
            for (int i = 0; i < (nbChilds - 1); i++)
            {
                newEOCTypeChilds[i] = Operator.EOCTypeChilds[i];
                newFeatures[i] = Operator.Features[i];
                if (newPatternGraphComparators != null)
                    newPatternGraphComparators[i] = Operator.Evaluator.PatternGraphComparators[i];
                if (newParentGraphComparators != null)
                    newParentGraphComparators[i] = Operator.Evaluator.ParentGraphComparators[i];
                for (int j = 0; j < (nbChilds - 1); j++)
                {
                    if (newGraphGraphComparators != null)
                        newGraphGraphComparators[i][j] = Operator.Evaluator.GraphGraphComparators[i][j];
                }
            }
            Operator.EOCTypeChilds = newEOCTypeChilds;
            Operator.Features = newFeatures;
            Operator.Evaluator.PatternGraphComparators = newPatternGraphComparators;
            Operator.Evaluator.ParentGraphComparators = newParentGraphComparators;
            Operator.Evaluator.GraphGraphComparators = newGraphGraphComparators;
            this.SetControlsValues();
        }

        private void cbxLoop_CheckedChanged(object sender, EventArgs e)
        {
            Operator.Loop = cbxLoop.Checked;
        }

        private void dgvComparators_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            DataGridViewButtonCell btCell = (DataGridViewButtonCell)dgv.Rows[e.RowIndex].Cells[e.ColumnIndex];
            GUI.L1Operations.FeatureMatrixComparator frm = new GUI.L1Operations.FeatureMatrixComparator();
            frm.Matrix = (CFeatureMatrixComparator)btCell.Tag;
            frm.DimensionName = GLOBALDATA.DIMENSIONS[e.RowIndex].ToString();
            frm.ShowDialog();
            if (frm.DialogResult  == DialogResult.OK)
            {
                CFormUtilities.SetDGVComparatorsButtonValue(
                    dgv, frm.Matrix,
                    e.RowIndex, e.ColumnIndex);
            }
        }

        #region Cell validation
        private void dgvStandard_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            string content = e.FormattedValue.ToString();
            double value;
            if (double.TryParse(content, out value) != true)
            {
                dgv.Rows[e.RowIndex].ErrorText = "Please enter a number (or press ESC) !";
                e.Cancel = true;
            }
            else
            {
                DataGridViewColumn dgvCol = dgv.Columns[e.ColumnIndex];
                double[] data = (double[])dgvCol.Tag;
                data[e.RowIndex] = double.Parse(e.FormattedValue.ToString());
            }
        }

        private void dgvStandard_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            DataGridView dgv = (DataGridView)sender;
            dgv.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        private void dgvStandard_DefaultValuesNeeded(object sender, DataGridViewRowEventArgs e)
        {
            foreach (DataGridViewCell cell in e.Row.Cells)
            {
                cell.Value = ((double)(0.0)).ToString();
            }
        }
        #endregion

        #endregion

        private void SetControlsValues()
        {
            this.SuspendLayout();
            // Tabs for each childs
            tabChilds.TabPages.Clear();
            for (int numChild = 0; numChild < Operator.EOCTypeChilds.Length; numChild++)
            {
                tabChilds.TabPages.Add(numChild.ToString(), "Child " + numChild.ToString());

                // Main layout of tab
                TableLayoutPanel layout = new TableLayoutPanel();
                tabChilds.TabPages[numChild].Controls.Add(layout);
                layout.Dock = DockStyle.Fill;
                layout.BorderStyle = BorderStyle.Fixed3D;
                layout.CellBorderStyle = TableLayoutPanelCellBorderStyle.Single;
                layout.ColumnCount = 4;
                layout.RowCount = 3;
                layout.RowStyles.Add(new RowStyle(SizeType.Percent, 10.0F));
                layout.RowStyles.Add(new RowStyle(SizeType.Percent, 10.0F));
                layout.RowStyles.Add(new RowStyle(SizeType.Percent, 80.0F));
                layout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 40.0F));
                layout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 20.0F));
                layout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 20.0F));
                layout.ColumnStyles.Add(new ColumnStyle(SizeType.Percent, 20.0F));

                // 0,0 = label of child
                ComboBox cbChildLabel = new ComboBox();
                layout.Controls.Add(cbChildLabel, 0, 0);
                cbChildLabel.Name = "cbChildLabel";
                cbChildLabel.Dock = DockStyle.Top;
                cbChildLabel.Tag = numChild;
                cbChildLabel.SelectedIndexChanged += new EventHandler(cbChildLabel_SelectedIndexChanged);
                CFormUtilities.SetComboBoxLabel(cbChildLabel, Labels, Operator.EOCTypeChilds[numChild]);

                // 1,0 -> 1,2 : labels for DGV
                Label lblFeatures = new Label();
                Label lblPGComparators = new Label();
                Label lblParGComparators = new Label();
                Label lblGGComparators = new Label();
                layout.Controls.Add(lblFeatures, 0, 1);
                layout.Controls.Add(lblPGComparators, 1, 1);
                layout.Controls.Add(lblParGComparators, 2, 1);
                layout.Controls.Add(lblGGComparators, 3, 1);
                lblFeatures.Text = "Features:";
                lblPGComparators.Text = "Pattern/Graph:";
                lblParGComparators.Text = "Parent/Graph:";
                lblGGComparators.Text = "Graph/Graph:";
                lblFeatures.Dock = DockStyle.Bottom;
                lblPGComparators.Dock = DockStyle.Bottom;
                lblParGComparators.Dock = DockStyle.Bottom;
                lblGGComparators.Dock = DockStyle.Bottom;

                // 2,0 : Features of child
                DataGridView dgvFeatures = new DataGridView();
                layout.Controls.Add(dgvFeatures, 0, 2);
                dgvFeatures.Name = "dgvFeatures";
                dgvFeatures.Dock = DockStyle.Fill;
                if (Operator.Evaluator.PatternGraphComparators != null)
                {
                    CFormUtilities.SetDGVMatrixOfDouble(
                        dgvFeatures, true,
                        new double[][] { 
                            Operator.Features[numChild].a, 
                            Operator.Features[numChild].b });
                    dgvFeatures.Columns[0].HeaderText = "a0";
                    dgvFeatures.Columns[1].HeaderText = "b0";
                    dgvFeatures.CellValidating += new System.Windows.Forms.DataGridViewCellValidatingEventHandler(this.dgvStandard_CellValidating);
                    dgvFeatures.CellEndEdit += new System.Windows.Forms.DataGridViewCellEventHandler(this.dgvStandard_CellEndEdit);
                    dgvFeatures.DefaultValuesNeeded += new System.Windows.Forms.DataGridViewRowEventHandler(this.dgvStandard_DefaultValuesNeeded);
                }

                // 2,1 : Pattern/Graph comparators
                DataGridView dgvPGComparators = new DataGridView();
                layout.Controls.Add(dgvPGComparators, 1, 2);
                dgvPGComparators.Name = "dgvPGComparators";
                dgvPGComparators.Dock = DockStyle.Fill;
                if (Operator.Evaluator.PatternGraphComparators != null)
                {
                    dgvPGComparators.CellContentClick += new DataGridViewCellEventHandler(dgvComparators_CellContentClick);
                    CFormUtilities.SetDGVComparators(
                        dgvPGComparators, false,
                        new CFeaturesComparator[] { Operator.Evaluator.PatternGraphComparators[numChild] });
                }

                // 2,2 : Parent/Graph comparators
                DataGridView dgvParGComparators = new DataGridView();
                layout.Controls.Add(dgvParGComparators, 2, 2);
                dgvParGComparators.Name = "dgvParGComparators";
                dgvParGComparators.Dock = DockStyle.Fill;
                if (Operator.Evaluator.ParentGraphComparators != null)
                {
                    dgvParGComparators.CellContentClick += new DataGridViewCellEventHandler(dgvComparators_CellContentClick);
                    CFormUtilities.SetDGVComparators(
                        dgvParGComparators, false,
                        new CFeaturesComparator[] { Operator.Evaluator.ParentGraphComparators[numChild] });
                }

                // 2,3 : Graph/Graph comparators
                DataGridView dgvGGComparators = new DataGridView();
                layout.Controls.Add(dgvGGComparators, 3, 2);
                dgvGGComparators.Name = "dgvGGComparators";
                dgvGGComparators.Dock = DockStyle.Fill;
                if (Operator.Evaluator.GraphGraphComparators != null)
                {
                    dgvGGComparators.CellContentClick += new DataGridViewCellEventHandler(dgvComparators_CellContentClick);
                    CFormUtilities.SetDGVComparators(
                        dgvGGComparators, false,
                        Operator.Evaluator.GraphGraphComparators[numChild]);
                }
            }
            tabChilds.SelectedTab = tabChilds.TabPages[0];

            // Labels
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);

            // Transformation
            CFormUtilities.SetDGVMatrixOfDouble(dgvCoefficients, true, Operator.Evaluator.Coefficients);

            // Neighbouroud
            if (Operator.Neighbouroud == null)
            {
                cbxUseNeighbouroud.Checked = false;
                dgvNeighbouroud.Visible = false;
            }
            else
            {
                cbxUseNeighbouroud.Checked = true;
                dgvNeighbouroud.Visible = true;
                CFormUtilities.SetDGVMatrixOfDouble(
                    dgvNeighbouroud, false,
                    new double[][] { Operator.Neighbouroud });
            }

            // Loop
            cbxLoop.Checked = Operator.Loop;

            // Comparators checkboxes
            cbxUsePatternGraphComparators.Checked = (Operator.Evaluator.PatternGraphComparators != null) ? true : false;
            cbxUseParentGraphComparators.Checked = (Operator.Evaluator.ParentGraphComparators != null) ? true : false;
            cbxUseGraphGraphComparators.Checked = (Operator.Evaluator.GraphGraphComparators != null) ? true : false;

            this.ResumeLayout(false);
            this.PerformLayout();
        }
        

        private void Insert_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChilds == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();
            if (Operator.Evaluator.Coefficients == null) SetDefaultPropertiesValues();
            if (Operator.Features == null) SetDefaultPropertiesValues();
            if (Operator.Modes == null) SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Type : set by radio button event

            // Features : set by DGV cell event

            // Coefficients : set by DGV cell event

            // Comparators : set by clicking on buttons in DGV

            this.Close();
        }
        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }
    }
}
