﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace GUI.L2Operations
{
    public partial class ClassifyByFeatures : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public List<string> Labels { get; set; }
        public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CClassifyByFeatures Operator { get; set; }

        public ClassifyByFeatures()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeChild = this.Labels[0];
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.EOCTypeParent = this.Labels[0];
            Operator.ChildFeatures = new CFeatures(NbDimensions);
            Operator.Coefficients = CPairingEvaluator.CreateStandardProjection(NbDimensions);
            Operator.Comparator = new CFeaturesComparator(NbDimensions);
            Operator.Replace = false;
        }

        private void SetControlsValues()
        {
            // Labels
            CFormUtilities.SetComboBoxLabel(cbChildLabel, Labels, Operator.EOCTypeChild);
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);

            // Type
            switch (Operator.Type)
            {
                case CClassifyByFeatures.ClassificationType.ClassifyByForm: rbForm.Checked = true; break;
                case CClassifyByFeatures.ClassificationType.ClassifyByPosition: rbPosition.Checked = true; break;
                case CClassifyByFeatures.ClassificationType.ClassifyBySize: rbSize.Checked = true; break;
                default: rbPosition.Checked = true; break;
            }

            // Features
            CFormUtilities.SetDGVMatrixOfDouble(
                dgvFeatures, true,
                new double[][] { Operator.ChildFeatures.a, Operator.ChildFeatures.b });
            dgvFeatures.Columns[0].HeaderText = "a0";
            dgvFeatures.Columns[1].HeaderText = "b0";

            // Coefficients
            CFormUtilities.SetDGVMatrixOfDouble(
                dgvCoefficients, false,
                Operator.Coefficients);

            // Comparators
            CFormUtilities.SetDGVComparators(
                dgvComparators, false,
                new CFeaturesComparator[] { Operator.Comparator });

            // Checkbox
            if (cbxReplace.Checked != Operator.Replace)
                cbxReplace.Checked = Operator.Replace;
        }

        #region Controls' events

        private void buttonCoefAdd_Click(object sender, EventArgs e)
        {
            int nbCoef = Operator.Coefficients.Length;
            double[][] coefficients = new double[nbCoef + 1][];
            for (int i = 0; i < nbCoef; i++)
            {
                coefficients[i] = new double[NbDimensions];
                for (int j = 0; j < NbDimensions; j++)
                {
                    coefficients[i][j] = Operator.Coefficients[i][j];
                }
            }
            coefficients[nbCoef] = new double[NbDimensions];
            Operator.Coefficients = coefficients;
            this.SetControlsValues();
        }

        private void buttonCoefDel_Click(object sender, EventArgs e)
        {
            int nbCoef = Operator.Coefficients.Length;
            if (nbCoef > 1)
            {
                double[][] coefficients = new double[nbCoef - 1][];
                for (int i = 0; i < (nbCoef - 1); i++)
                {
                    coefficients[i] = new double[NbDimensions];
                    for (int j = 0; j < NbDimensions; j++)
                    {
                        coefficients[i][j] = Operator.Coefficients[i][j];
                    }
                }
                Operator.Coefficients = coefficients;
                this.SetControlsValues();
            }
        }

        private void dgvComparators_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            DataGridViewButtonCell btCell = (DataGridViewButtonCell)dgv.Rows[e.RowIndex].Cells[e.ColumnIndex];
            GUI.L1Operations.FeatureMatrixComparator frm = new GUI.L1Operations.FeatureMatrixComparator();
            frm.Matrix = (CFeatureMatrixComparator)btCell.Tag;
            frm.DimensionName = GLOBALDATA.DIMENSIONS[e.RowIndex].ToString();
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                CFormUtilities.SetDGVComparatorsButtonValue(
                    dgv, frm.Matrix,
                    e.RowIndex, e.ColumnIndex);
            }
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }

        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }

        private void cbChildLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChild = String.Copy(cbChildLabel.SelectedItem.ToString());
        }

        private void rbPosition_CheckedChanged(object sender, EventArgs e)
        {
            if (rbPosition.Checked == true) Operator.Type = CClassifyByFeatures.ClassificationType.ClassifyByPosition;
        }

        private void rbSize_CheckedChanged(object sender, EventArgs e)
        {
            if (rbSize.Checked == true) Operator.Type = CClassifyByFeatures.ClassificationType.ClassifyBySize;
        }

        private void rbForm_CheckedChanged(object sender, EventArgs e)
        {
            if (rbForm.Checked == true) Operator.Type = CClassifyByFeatures.ClassificationType.ClassifyByForm;
        }

        private void cbxReplace_CheckedChanged(object sender, EventArgs e)
        {
            Operator.Replace = cbxReplace.Checked;
        }

        #region Cell validation

        private void dgvStandard_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            string content = e.FormattedValue.ToString();
            double value;
            if (double.TryParse(content, out value) != true)
            {
                dgv.Rows[e.RowIndex].ErrorText = "Please enter a number (or press ESC) !";
                e.Cancel = true;
            }
            else
            {
                DataGridViewColumn dgvCol = dgv.Columns[e.ColumnIndex];
                double[] data = (double[])dgvCol.Tag;
                data[e.RowIndex] = double.Parse(e.FormattedValue.ToString());
            }
        }

        private void dgvStandard_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            DataGridView dgv = (DataGridView)sender;
            dgv.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        private void dgvStandard_DefaultValuesNeeded(object sender, DataGridViewRowEventArgs e)
        {
            foreach (DataGridViewCell cell in e.Row.Cells)
            {
                cell.Value = ((double)(0.0)).ToString();
            }
        }
        #endregion        
        #endregion

        private void ClassifySize_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChild == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();
            if (Operator.ChildFeatures == null) SetDefaultPropertiesValues();
            if (Operator.Coefficients == null) SetDefaultPropertiesValues();
            if (Operator.Comparator == null) SetDefaultPropertiesValues();
            if (Operator.Type == CClassifyByFeatures.ClassificationType.No) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Type : set by radio button event

            // Features : set by DGV cell event

            // Coefficients : set by DGV cell event

            // Comparators : set by clicking on buttons in DGV

            // Replace : set by checkbox event

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

    }
}
