﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace GUI.L2Operations
{
    public partial class Fusion : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public List<string> Labels { get; set; }
        public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CFusion Operator { get; set; }

        public Fusion()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeChilds = this.Labels[0];
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.EOCTypeParent = this.Labels[0];
            Operator.Modes = new CFeatureMode[NbDimensions];
            Operator.Comparator = new CFeaturesComparator(NbDimensions);
            Operator.Neighbouroud = null;
        }

        private void SetControlsValues()
        {
            // Labels
            CFormUtilities.SetComboBoxLabel(cbChildLabel, Labels, Operator.EOCTypeChilds);
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);

            // Comparators
            CFormUtilities.SetDGVComparators(
                dgvComparators, true,
                new CFeaturesComparator[] { Operator.Comparator });
            dgvComparators.Columns[0].HeaderText = "Ch0-Ch1";

            // Neighbouroud
            if (Operator.Neighbouroud == null)
            {
                cbxUseNeighbouroud.Checked = false;
                dgvNeighbouroud.Visible = false;
            }
            else
            {
                cbxUseNeighbouroud.Checked = true;
                dgvNeighbouroud.Visible = true;
                CFormUtilities.SetDGVMatrixOfDouble(
                    dgvNeighbouroud, false,
                    new double[][] { Operator.Neighbouroud });
            }
        }

        #region Controls's events

        private void dgvComparators_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            DataGridViewButtonCell btCell = (DataGridViewButtonCell)dgv.Rows[e.RowIndex].Cells[e.ColumnIndex];
            GUI.L1Operations.FeatureMatrixComparator frm = new GUI.L1Operations.FeatureMatrixComparator();
            frm.Matrix = (CFeatureMatrixComparator)btCell.Tag;
            frm.DimensionName = GLOBALDATA.DIMENSIONS[e.RowIndex].ToString();
            frm.ShowDialog();
            if (frm.DialogResult  == DialogResult.OK)
            {
                CFormUtilities.SetDGVComparatorsButtonValue(
                    dgv, frm.Matrix,
                    e.RowIndex, e.ColumnIndex);
            }
        }

        private void cbxUseNeighbouroud_CheckedChanged(object sender, EventArgs e)
        {
            dgvNeighbouroud.Visible = cbxUseNeighbouroud.Checked;
            if (cbxUseNeighbouroud.Checked == true)
            {
                Operator.Neighbouroud = new double[NbDimensions];
            }
            else
            {
                Operator.Neighbouroud = null;
            }
            this.SetControlsValues();
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }

        private void cbChildLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChilds = String.Copy(cbChildLabel.SelectedItem.ToString());
        }

        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }

        private void buttonSetFeaturesModes_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.FeatureMode frm = new GUI.L1Operations.FeatureMode();
            frm.NbDimensions = this.NbDimensions;
            frm.Modes = Operator.Modes;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
            }
        }

        #region Cell validation

        private void dgvStandard_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            string content = e.FormattedValue.ToString();
            double value;
            if (double.TryParse(content, out value) != true)
            {
                dgv.Rows[e.RowIndex].ErrorText = "Please enter a number (or press ESC) !";
                e.Cancel = true;
            }
            else
            {
                DataGridViewColumn dgvCol = dgv.Columns[e.ColumnIndex];
                double[] data = (double[])dgvCol.Tag;
                data[e.RowIndex] = double.Parse(e.FormattedValue.ToString());
            }
        }

        private void dgvStandard_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            DataGridView dgv = (DataGridView)sender;
            dgv.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        private void dgvStandard_DefaultValuesNeeded(object sender, DataGridViewRowEventArgs e)
        {
            foreach (DataGridViewCell cell in e.Row.Cells)
            {
                cell.Value = ((double)(0.0)).ToString();
            }
        }

        #endregion

        #endregion

        private void Fusion_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChilds == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();
            if (Operator.Comparator == null) SetDefaultPropertiesValues();
            if (Operator.Modes == null) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Neighbouroud : set by DGV cell event

            // Comparators : set by clicking on buttons

            // Modes : set by clicking on button

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }


    }
}
