﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;

namespace GUI.L2Operations
{
    public partial class Intersection : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public List<string> Labels { get; set; }
        public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CIntersect Operator { get; set; }

        public Intersection()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeChilds = new string[] { this.Labels[0], this.Labels[0] };
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.EOCTypeParent = this.Labels[0];
            Operator.Coefficients = new double[1][];
            Operator.Coefficients[0] = new double[NbDimensions];
            Operator.Neighbouroud = null;
            Operator.Modes = CFeatureMode.BuildArray(NbDimensions);
        }
        private void SetControlsValues()
        {
            // Labels
            CFormUtilities.SetComboBoxLabel(cbChildLabel0, Labels, Operator.EOCTypeChilds[0]);
            CFormUtilities.SetComboBoxLabel(cbChildLabel1, Labels, Operator.EOCTypeChilds[1]);
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);

            // Coefficients
            CFormUtilities.SetDGVMatrixOfDouble(
                dgvCoefficients, true,
                Operator.Coefficients);

        }

        #region Controls' events

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }

        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }

        private void cbChildLabel0_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChilds[0] = String.Copy(cbChildLabel0.SelectedItem.ToString());
        }
        private void cbChildLabel1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChilds[1] = String.Copy(cbChildLabel1.SelectedItem.ToString());
        }
        #region Cell validation

        private void dgvStandard_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            DataGridView dgv = (DataGridView)sender;
            string content = e.FormattedValue.ToString();
            double value;
            if (double.TryParse(content, out value) != true)
            {
                dgv.Rows[e.RowIndex].ErrorText = "Please enter a number (or press ESC) !";
                e.Cancel = true;
            }
            else
            {
                DataGridViewColumn dgvCol = dgv.Columns[e.ColumnIndex];
                double[] data = (double[])dgvCol.Tag;
                data[e.RowIndex] = double.Parse(e.FormattedValue.ToString());
            }
        }

        private void dgvStandard_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            // Clear the row error in case the user presses ESC.   
            DataGridView dgv = (DataGridView)sender;
            dgv.Rows[e.RowIndex].ErrorText = String.Empty;
        }

        private void dgvStandard_DefaultValuesNeeded(object sender, DataGridViewRowEventArgs e)
        {
            foreach (DataGridViewCell cell in e.Row.Cells)
            {
                cell.Value = ((double)(0.0)).ToString();
            }
        }
        #endregion           

        #endregion

        private void Intersection_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChilds == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();
            if (Operator.Coefficients == null) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Coefficients : set by DGV cell event

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

    }
}
