﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;
using Agora.Model.OperatorLevel3;

namespace GUI.L3Operations
{
    public partial class EuclidianFeatures : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public Bitmap OriginalImage { get;  set; }
        public List<string> Labels { get;  set; }
        //public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CEuclidianFeatures Operator { get; set; }

        public EuclidianFeatures()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeChild = this.Labels[0];
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.EOCTypeParent = this.Labels[0];
            Operator.UseForm = false;
            Operator.UsePosition = false;
            Operator.UseSize = false;
            Operator.PositionXMin = 0;
            Operator.PositionXMax = 0;
            Operator.PositionYMin = 0;
            Operator.PositionYMax = 0;
            Operator.Replace = false;
        }

        #region Controls' events

        private void cbxActivePosition_CheckedChanged(object sender, EventArgs e)
        {
            grpbxPosition.Enabled = cbxActivePosition.Checked;
            Operator.UsePosition = cbxActivePosition.Checked;
        }
        private void cbxActiveSize_CheckedChanged(object sender, EventArgs e)
        {
            grpbxSize.Enabled = cbxActiveSize.Checked;
            Operator.UseSize = cbxActiveSize.Checked;
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }
        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }
        private void cbChildLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChild = String.Copy(cbChildLabel.SelectedItem.ToString());
        }

        /// <summary>
        /// Generic update from a textbox change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="val"></param>
        private double tbDoubleValueTextChanged(TextBox sender, double val)
        {
            double result = val;
            if (Double.TryParse(sender.Text, out result) == true)
            {
                val = result;
            }
            return result;
        }

        private void tbPositionXMin_TextChanged(object sender, EventArgs e)
        {
            Operator.PositionXMin = this.tbDoubleValueTextChanged((TextBox)sender, Operator.PositionXMin);
            this.SetControlsValues();
        }
        private void tbPositionXMax_TextChanged(object sender, EventArgs e)
        {
            Operator.PositionXMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.PositionXMax);
            this.SetControlsValues();
        }
        private void tbPositionYMin_TextChanged(object sender, EventArgs e)
        {
            Operator.PositionYMin = this.tbDoubleValueTextChanged((TextBox)sender, Operator.PositionYMin);
            this.SetControlsValues();
        }
        private void tbPositionYMax_TextChanged(object sender, EventArgs e)
        {
            Operator.PositionYMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.PositionYMax);
            this.SetControlsValues();
        }
        private void tbSizeX_TextChanged(object sender, EventArgs e)
        {
            Operator.SizeX = this.tbDoubleValueTextChanged((TextBox)sender, Operator.SizeX);
            this.SetControlsValues();
        }
        private void tbSizeY_TextChanged(object sender, EventArgs e)
        {
            Operator.SizeY = this.tbDoubleValueTextChanged((TextBox)sender, Operator.SizeY);
            this.SetControlsValues();
        }
        private void tbSizeXTolerance_TextChanged(object sender, EventArgs e)
        {
            Operator.SizeXTolerance = this.tbDoubleValueTextChanged((TextBox)sender, Operator.SizeXTolerance);
            this.SetControlsValues();
        }
        private void tbSizeYTolerance_TextChanged(object sender, EventArgs e)
        {
            Operator.SizeYTolerance = this.tbDoubleValueTextChanged((TextBox)sender, Operator.SizeYTolerance);
            this.SetControlsValues();
        }

        private void rbMaximumSize_CheckedChanged(object sender, EventArgs e)
        {
            if (rbMaximumSize.Checked == true)
            {
                Operator.SizeType = CEuclidianFeatures.SizeClassificationType.Maximum;
                this.SetControlsValues();
            }
        }
        private void rbMinimumSize_CheckedChanged(object sender, EventArgs e)
        {
            if (rbMinimumSize.Checked == true)
            {
                Operator.SizeType = CEuclidianFeatures.SizeClassificationType.Minimum;
                this.SetControlsValues();
            }
        }
        private void rbExactSize_CheckedChanged(object sender, EventArgs e)
        {
            if (rbExactSize.Checked == true)
            {
                Operator.SizeType = CEuclidianFeatures.SizeClassificationType.Exact;
                this.SetControlsValues();
            }
        }

        private void btnPositionSelectsOnImage_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.PositionXMin = frm.Selection.Left;
                Operator.PositionXMax = frm.Selection.Right;
                Operator.PositionYMin = frm.Selection.Top;
                Operator.PositionYMax = frm.Selection.Bottom;
            }
            this.SetControlsValues();
        }
        private void btnSizeSelectsOnImage_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.SizeX = frm.Selection.Width;
                Operator.SizeY = frm.Selection.Height;
            }
            this.SetControlsValues();
        }
        #endregion

        private void SetControlsValues()
        {
            this.SuspendLayout();
            
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);
            CFormUtilities.SetComboBoxLabel(cbChildLabel, Labels, Operator.EOCTypeChild);

            // Position tab
            cbxActivePosition.Checked = Operator.UsePosition;
            tbPositionXMin.Text = Operator.PositionXMin.ToString();
            tbPositionXMax.Text = Operator.PositionXMax.ToString();
            tbPositionYMin.Text = Operator.PositionYMin.ToString();
            tbPositionYMax.Text = Operator.PositionYMax.ToString();

            // Size Tab
            cbxActiveSize.Checked = Operator.UseSize;
            
            if (Operator.SizeType == CEuclidianFeatures.SizeClassificationType.Maximum)
                rbMaximumSize.Checked = true;
            else
                rbMaximumSize.Checked = false;
            
            if (Operator.SizeType == CEuclidianFeatures.SizeClassificationType.Minimum)
                rbMinimumSize.Checked = true;
            else
                rbMinimumSize.Checked = false;

            if (Operator.SizeType == CEuclidianFeatures.SizeClassificationType.Exact)
            {
                rbExactSize.Checked = true;
                grpbxTolerances.Enabled = true;
            }
            else
            {
                rbExactSize.Checked = false;
                grpbxTolerances.Enabled = false;
            }

            tbSizeX.Text = Operator.SizeX.ToString();
            tbSizeXTolerance.Text = Operator.SizeXTolerance.ToString();
            tbSizeY.Text = Operator.SizeY.ToString();
            tbSizeYTolerance.Text = Operator.SizeYTolerance.ToString();

            this.ResumeLayout(false);
            this.PerformLayout();
        }

        private void EuclidianFeatures_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChild == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Values : set by Textbox event

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

    }
}
