﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Agora.Model.ImageAnalysis;
using Agora.Model.Utils;
using Agora.Model.OperatorLevel0;
using Agora.Model.OperatorLevel0.Pairing;
using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel2;
using Agora.Model.OperatorLevel3;

namespace GUI.L3Operations
{
    public partial class EuclidianSpacing : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public Bitmap OriginalImage { get; set; }
        public List<string> Labels { get; set; }
        public int NbDimensions { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CEuclidianSpacing Operator { get; set; }

        public EuclidianSpacing()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
            Labels = new List<string>();
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeParent = this.Labels[0];
            Operator.EOCTypeChild0 = this.Labels[0];
            Operator.EOCTypeChild1 = this.Labels[0];
            Operator.EOCTypeNewEOC = this.Labels[0];
            Operator.Horizontal = true;
            Operator.Type = CEuclidianSpacing.SpacingType.Maximum;
            Operator.XSpacing = 0;
            Operator.XTolerance = 0;
            Operator.YSpacing = 0;
            Operator.YTolerance = 0;
            Operator.Replace = false;
        }

        #region Controls' events
        private void rbModeHorizontal_CheckedChanged(object sender, EventArgs e)
        {
            Operator.Horizontal = rbModeHorizontal.Checked;
            this.SetControlsValues();
        }

        private void rbModeVertical_CheckedChanged(object sender, EventArgs e)
        {
            Operator.Horizontal = !rbModeVertical.Checked;
            this.SetControlsValues();
        }

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }
        private void cbNewEOCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOC = String.Copy(cbNewEOCLabel.SelectedItem.ToString());
        }
        private void cbChild0Label_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChild0 = String.Copy(((ComboBox)sender).SelectedItem.ToString());
        }
        private void cbChild1Label_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeChild1 = String.Copy(((ComboBox)sender).SelectedItem.ToString());
        }
        /// <summary>
        /// Generic update from a textbox change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="val"></param>
        private double tbDoubleValueTextChanged(TextBox sender, double val)
        {
            double result = val;
            if (Double.TryParse(sender.Text, out result) == true)
            {
                val = result;
            }
            return result;
        }

        private void tbXSpacing_TextChanged(object sender, EventArgs e)
        {
            Operator.XSpacing = this.tbDoubleValueTextChanged((TextBox)sender, Operator.XSpacing);
            this.SetControlsValues();
        }
        private void tbXTolerance_TextChanged(object sender, EventArgs e)
        {
            Operator.XTolerance = this.tbDoubleValueTextChanged((TextBox)sender, Operator.XTolerance);
            this.SetControlsValues();
        }
        private void tbYSpacing_TextChanged(object sender, EventArgs e)
        {
            Operator.YSpacing = this.tbDoubleValueTextChanged((TextBox)sender, Operator.YSpacing);
            this.SetControlsValues();
        }
        private void tbYTolerance_TextChanged(object sender, EventArgs e)
        {
            Operator.YTolerance = this.tbDoubleValueTextChanged((TextBox)sender, Operator.YTolerance);
            this.SetControlsValues();
        }
        private void btnSelectsOnImage_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                if (Operator.Horizontal == true)
                {
                    Operator.XSpacing = frm.Selection.Width;
                    Operator.YTolerance = frm.Selection.Height;
                }
                else
                {
                    Operator.XTolerance = frm.Selection.Width;
                    Operator.YSpacing = frm.Selection.Height;
                }
            }
            this.SetControlsValues();
        }
        #endregion

        private void SetControlsValues()
        {
            this.SuspendLayout();

            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbNewEOCLabel, Labels, Operator.EOCTypeNewEOC);

            if (Operator.Horizontal == true)
            {
                // Enable horizontal controls:
                grpbxHorizontalMode.Enabled = true;
                pnlHorizontalZone.Visible = true;
                pnlLeftEOC.Visible = true;
                pnlRightEOC.Visible = true;

                // Disable vertical controls:
                grpbxVerticalMode.Enabled = false;
                pnlVerticalZone.Visible = false;
                pnlTopEOC.Visible = false;
                pnlBottomEOC.Visible = false;
            }
            else
            {
                // Disable horizontal controls:
                grpbxHorizontalMode.Enabled = false;
                pnlHorizontalZone.Visible = false;
                pnlLeftEOC.Visible = false;
                pnlRightEOC.Visible = false;

                // Enable vertical controls:
                grpbxVerticalMode.Enabled = true;
                pnlVerticalZone.Visible = true;
                pnlTopEOC.Visible = true;
                pnlBottomEOC.Visible = true;
            }

            // Horizontal mode controls :
            CFormUtilities.SetComboBoxLabel(cbRightEOCLabel, Labels, Operator.EOCTypeChild0);
            CFormUtilities.SetComboBoxLabel(cbLeftEOCLabel, Labels, Operator.EOCTypeChild1);
            tbHorizontalXSpacing.Text = Operator.XSpacing.ToString();
            tbHorizontalXTolerance.Text = Operator.XTolerance.ToString();
            tbHorizontalYTolerance.Text = Operator.YTolerance.ToString();

            // Vertical mode controls :
            CFormUtilities.SetComboBoxLabel(cbTopEOCLabel, Labels, Operator.EOCTypeChild0);
            CFormUtilities.SetComboBoxLabel(cbBottomEOCLabel, Labels, Operator.EOCTypeChild1);
            tbVerticalYSpacing.Text = Operator.YSpacing.ToString();
            tbVerticalYTolerance.Text = Operator.YTolerance.ToString();
            tbVerticalXTolerance.Text = Operator.XTolerance.ToString();

            this.ResumeLayout(false);
            this.PerformLayout();
        }
        private void EuclidianSpacing_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeChild0 == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeChild1 == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeNewEOC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Values : set by Textbox event

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }



    }
}
