﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

//using Agora.Model.ImageAnalysis;
//using Agora.Model.Utils;
//using Agora.Model.OperatorLevel0;
//using Agora.Model.OperatorLevel0.Pairing;
//using Agora.Model.OperatorLevel1;
using Agora.Model.OperatorLevel3;

namespace GUI.L3Operations
{
    public partial class TextSegmentation : Form
    {
        /// <summary>
        /// Properties to set before showing form
        /// </summary>
        public Bitmap OriginalImage { get; set; }
        public List<string> Labels { get; set; }

        /// <summary>
        /// Properties to get after closing form
        /// </summary>
        public CTextSegmentation Operator { get; set; }

        public TextSegmentation()
        {
            InitializeComponent();
            Labels = new List<string>();
            this.DialogResult = DialogResult.None;
        }
        private void SetDefaultPropertiesValues()
        {
            Operator.EOCTypeParent = this.Labels[0];
            Operator.EOCTypeCC = this.Labels[0];
            Operator.EOCTypeNewEOCPrefix = String.Empty;
            Operator.NoiseWidthMax = 0.0;
            Operator.NoiseHeightMax = 0.0;
            Operator.CharAccentYSpacingMax = 0.0;
            Operator.CharWidthMax = 0.0;
            Operator.CharHeightMax = 0.0;
            Operator.CharADWidthMax = 0.0;
            Operator.CharADHeightMax = 0.0;
            Operator.WordLeftRangemapXMax = 0.0;
            Operator.WordRightRangemapXMin = 0.0;
            Operator.WordCharSpacingXMax = 0.0;
            Operator.BlockLineSpacingYMax = 0.0;
            Operator.DocumentBlockSpacingXMin = 0.0;
        }
        private void SetControlsValues()
        {
            // Labels
            CFormUtilities.SetComboBoxLabel(cbParentLabel, Labels, Operator.EOCTypeParent);
            CFormUtilities.SetComboBoxLabel(cbCCLabel, Labels, Operator.EOCTypeCC);
            tbNewEOCLabelPrefix.Text = String.Copy(Operator.EOCTypeNewEOCPrefix);
            tbNoiseWidthMax.Text = Operator.NoiseWidthMax.ToString();
            tbNoiseHeightMax.Text = Operator.NoiseHeightMax.ToString();
            tbAccentWidthMax.Text = Operator.AccentWidthMax.ToString();
            tbAccentHeightMax.Text = Operator.AccentHeightMax.ToString();
            tbCharWidthMax.Text = Operator.CharWidthMax.ToString();
            tbCharHeightMax.Text = Operator.CharHeightMax.ToString();
            tbCharADWidthMax.Text = Operator.CharADWidthMax.ToString();
            tbCharADHeightMax.Text = Operator.CharADHeightMax.ToString();
            tbCharNoiseYSpacingMax.Text = Operator.CharAccentYSpacingMax.ToString();
            tbWordLeftRangemapXMax.Text = Operator.WordLeftRangemapXMax.ToString();
            tbWordRightRangemapXMin.Text = Operator.WordRightRangemapXMin.ToString();
            tbWordCharSpacingXMax.Text = Operator.WordCharSpacingXMax.ToString();
            tbBlockLineSpacingYMax.Text = Operator.BlockLineSpacingYMax.ToString();
            tbDocumentBlockSpacingXMin.Text = Operator.DocumentBlockSpacingXMin.ToString();
        }

        #region Controls's events

        private void cbParentLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeParent = String.Copy(cbParentLabel.SelectedItem.ToString());
        }
        private void cbCCLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeCC = String.Copy(cbCCLabel.SelectedItem.ToString());
        }
        private void tbNewEOCLabelPrefix_TextChanged(object sender, EventArgs e)
        {
            Operator.EOCTypeNewEOCPrefix = String.Copy(tbNewEOCLabelPrefix.Text);
        }

        /// <summary>
        /// Generic update from a textbox change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="val"></param>
        private double tbDoubleValueTextChanged(TextBox sender, double val)
        {
            double result = val;
            if (Double.TryParse(sender.Text, out result) == true)
            {
                val = result;
            }
            return result;
        }

        private void tbNoiseWidthMax_TextChanged(object sender, EventArgs e)
        {
            Operator.NoiseWidthMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.NoiseWidthMax);
            this.SetControlsValues();
        }
        private void tbNoiseHeightMax_TextChanged(object sender, EventArgs e)
        {
            Operator.NoiseHeightMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.NoiseHeightMax);
            this.SetControlsValues();
        }
        private void tbAccentWidthMax_TextChanged(object sender, EventArgs e)
        {
            Operator.AccentWidthMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.AccentWidthMax);
            this.SetControlsValues();
        }
        private void tbAccentHeightMax_TextChanged(object sender, EventArgs e)
        {
            Operator.AccentHeightMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.AccentHeightMax);
            this.SetControlsValues();
        }
        private void tbCharWidthMax_TextChanged(object sender, EventArgs e)
        {
            Operator.CharWidthMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.CharWidthMax);
            this.SetControlsValues();
        }
        private void tbCharHeightMax_TextChanged(object sender, EventArgs e)
        {
            Operator.CharHeightMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.CharHeightMax);
            this.SetControlsValues();
        }
        private void tbCharADWidthMax_TextChanged(object sender, EventArgs e)
        {
            Operator.CharADWidthMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.CharADWidthMax);
            this.SetControlsValues();
        }
        private void tbCharADHeightMax_TextChanged(object sender, EventArgs e)
        {
            Operator.CharADHeightMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.CharADHeightMax);
            this.SetControlsValues();
        }
        private void tbCharNoiseYSpacingMax_TextChanged(object sender, EventArgs e)
        {
            Operator.CharAccentYSpacingMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.CharAccentYSpacingMax);
            this.SetControlsValues();
        }
        private void tbWordCharSpacingXMax_TextChanged(object sender, EventArgs e)
        {
            Operator.WordCharSpacingXMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.WordCharSpacingXMax);
            this.SetControlsValues();
        }
        private void tbWordLeftRangemapXMax_TextChanged(object sender, EventArgs e)
        {
            Operator.WordLeftRangemapXMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.WordLeftRangemapXMax);
            this.SetControlsValues();
        }
        private void tbWordRightRangemapXMin_TextChanged(object sender, EventArgs e)
        {
            Operator.WordRightRangemapXMin = this.tbDoubleValueTextChanged((TextBox)sender, Operator.WordRightRangemapXMin);
            this.SetControlsValues();
        }
        private void tbBlockLineSpacingYMax_TextChanged(object sender, EventArgs e)
        {
            Operator.BlockLineSpacingYMax = this.tbDoubleValueTextChanged((TextBox)sender, Operator.BlockLineSpacingYMax);
            this.SetControlsValues();
        }
        private void tbDocumentBlockSpacingXMin_TextChanged(object sender, EventArgs e)
        {
            Operator.DocumentBlockSpacingXMin = this.tbDoubleValueTextChanged((TextBox)sender, Operator.DocumentBlockSpacingXMin);
            this.SetControlsValues();
        }

        #endregion

        private void TextSegmentation_Shown(object sender, EventArgs e)
        {
            if (Operator.EOCTypeCC == null) SetDefaultPropertiesValues();
            if (Operator.EOCTypeParent == null) SetDefaultPropertiesValues();

            this.SetControlsValues();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;

            // Labels : set by Combobox event

            // Values : set by Textbox event

            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void buttonReset_Click(object sender, EventArgs e)
        {
            this.SetDefaultPropertiesValues();
            this.SetControlsValues();
        }

        private void btnSelectOnImageNoise_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.NoiseWidthMax = frm.Selection.Width;
                Operator.NoiseHeightMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageAccent_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.AccentWidthMax = frm.Selection.Width;
                Operator.AccentHeightMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageCharSize_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.CharWidthMax = frm.Selection.Width;
                Operator.CharHeightMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageCharAccentYSpacing_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.CharAccentYSpacingMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageCharAD_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.CharADWidthMax = frm.Selection.Width;
                Operator.CharADHeightMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageBlockYSpacing_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.BlockLineSpacingYMax = frm.Selection.Height;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageWordXSpacing_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.WordCharSpacingXMax = frm.Selection.Width;
            }
            this.SetControlsValues();
        }

        private void btnSelectOnImageDocumentXSpacing_Click(object sender, EventArgs e)
        {
            GUI.SelectionInPictureForm frm = new SelectionInPictureForm();
            frm.OriginalImage = this.OriginalImage;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Operator.DocumentBlockSpacingXMin = frm.Selection.Width;
            }
            this.SetControlsValues();
        }


    }
}
