﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Configuration;
using System.Diagnostics;

using AForge.Imaging.Filters;

using Agora;
using Agora.Model.Utils;
using Agora.ViewModel;

using Settings = GUI.Properties.Settings;

namespace GUI
{
    public partial class MainForm : Form
    {
        private Agora.ViewModel.CViewModel _ViewModel = null;
        private BackgroundWorker _Worker = null;

        public MainForm()
        {
            InitializeComponent();
            this.GenerateLabelColors();
            this.CreateColoredImageList();
            //pbxImage.MouseWheel += new MouseEventHandler(pbxImage_MouseWheel);
            pnlImage.MouseWheel += new MouseEventHandler(pbxImage_MouseWheel);
        }
        private void MainForm_Load(object sender, EventArgs e)
        {
            string name = new StackTrace().GetFrame(0).GetMethod().ReflectedType.Name;
            DefaultTraceListener dtl = (DefaultTraceListener)Trace.Listeners[0];
            dtl.AssertUiEnabled = true;
            dtl.LogFileName = "logfile.txt";

            TextBoxTraceListener tbtl = new TextBoxTraceListener(tbLog);
            tbtl.IndentSize = 4;
            Trace.Listeners.Add(tbtl);
            Trace.WriteLine(DateTime.Now.ToString() + ":START", name);

            _ViewModel = new CViewModel();
            this.Text = "<New Project>";
        }
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Settings.Default.Save();
        }

        #region Miscellaneous

        private void DisplayError(string msg)
        {
            MessageBox.Show(msg, msg, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private KnownColor[] _UsedColors =
            {
                KnownColor.Black,
                KnownColor.Red,
                KnownColor.Green,
                KnownColor.Blue,
                KnownColor.Cyan,
                KnownColor.Magenta,
                KnownColor.DarkGoldenrod,
                KnownColor.Salmon,
                KnownColor.Orange,
                KnownColor.Aquamarine,
            };

        private KnownColor[] _LabelColors = null;

        private void GenerateLabelColors()
        {
            _LabelColors = new KnownColor[10 * _UsedColors.Length];
            for (int i = 0; i < _LabelColors.Length; i = i + 10)
            {
                for (int j = 0; j < _UsedColors.Length; j++)
                {
                    _LabelColors[i + j] = _UsedColors[j];
                }
            }
        }

        private void CreateColoredImageList()
        {
            foreach (KnownColor kc in _LabelColors)
            {
                Bitmap image = new Bitmap(lvImageList.ImageSize.Width, lvImageList.ImageSize.Height);
                Graphics graphics = Graphics.FromImage(image);
                SolidBrush brush = new SolidBrush(Color.FromKnownColor(kc));
                graphics.FillRectangle(brush, 0, 0, image.Width, image.Height);
                lvImageList.Images.Add(image);
            }
        }
        #endregion

        #region Management of background processing

        private void timer1_Tick(object sender, EventArgs e)
        {
            toolStripProgressBar1.PerformStep();
            if (toolStripProgressBar1.Value >= toolStripProgressBar1.Maximum)
                toolStripProgressBar1.Value = 0;
        }

        private void Background_DisableControls()
        {
            menuBar.Enabled = false;
            tvGraph.Enabled = false;
            lvLabels.Enabled = false;
            pbxImage.Enabled = false;
            toolStripProgressBar1.Value = 0;
            timer1.Enabled = true;
            Trace.WriteLine("Background thread started...");
        }
        private void Background_EnableControls()
        {
            Trace.WriteLine("Background thread done.");
            menuBar.Enabled = true;
            tvGraph.Enabled = true;
            lvLabels.Enabled = true;
            pbxImage.Enabled = true;
            timer1.Enabled = false;
            toolStripProgressBar1.Value = 0;
        }
        private void Background_WorkStart(DoWorkEventHandler target, object parameter)
        {
            this.Background_DisableControls();
            _Worker = new BackgroundWorker();
            _Worker.WorkerReportsProgress = false;
            _Worker.WorkerSupportsCancellation = false;
            _Worker.RunWorkerCompleted += this.Background_WorkCompleted;
            _Worker.DoWork += target;
            _Worker.RunWorkerAsync(parameter);
        }

        private void Background_WorkCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            this.Background_EnableControls();
            _LabelListChanged = true;
            this.ShowAnalysis();
        }

        #endregion

        #region Management of menu "File"

        private void Background_ProjectOpen(object sender, DoWorkEventArgs e)
        {
            _ViewModel.ProjectOpen((string)e.Argument);
        }
        private void fileOpenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = Settings.Default.projectDirectory;
            openFileDialog1.InitialDirectory = Settings.Default.projectDirectory;
            openFileDialog1.Filter = "Agora Project (*.xml)|*.xml";
            openFileDialog1.FileName = Settings.Default.projectFilename;
            DialogResult res = openFileDialog1.ShowDialog();
            string filename = openFileDialog1.FileName;
            Settings.Default.projectDirectory = System.IO.Path.GetDirectoryName(filename);
            Settings.Default.projectFilename = System.IO.Path.GetFileName(filename);
            if (res == DialogResult.OK)
            {
                try
                {
                    this.Text = Settings.Default.projectFilename;
                    //_ViewModel.ProjectOpen(filename);
                    //this.ShowAnalysis();
                    Background_WorkStart(new DoWorkEventHandler(Background_ProjectOpen), filename);
                    _CurrentImageChanged = true;
                }
                catch (CExceptionViewModel ex)
                {
                    DisplayError(ex.Message);
                }
            }
        }

        private void fileNewToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                _ViewModel.ProjectCreateNew();
                this.Text = "<New Project>";
            }
            catch (CExceptionViewModel ex)
            {
                DisplayError(ex.Message);
            }
        }

        private void fileSaveToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Title = Settings.Default.projectDirectory;
            saveFileDialog1.InitialDirectory = Settings.Default.projectDirectory;
            saveFileDialog1.FileName = Settings.Default.projectFilename;
            saveFileDialog1.AddExtension = true;
            saveFileDialog1.Filter = "Agora Project (*.xml)|*.xml";
            DialogResult res = saveFileDialog1.ShowDialog();
            string filename = saveFileDialog1.FileName;
            Settings.Default.projectDirectory = System.IO.Path.GetDirectoryName(filename);
            Settings.Default.projectFilename = System.IO.Path.GetFileName(filename);
            if (res == DialogResult.OK)
            {
                try
                {
                    _ViewModel.ProjectSave(filename);
                    this.Text = Settings.Default.projectFilename;
                }
                catch (CExceptionViewModel ex)
                {
                    DisplayError(ex.Message);
                }
            }
        }

        private void fileQuitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                _ViewModel.ProjectClose();
                this.Close();
            }
            catch (CExceptionViewModel ex)
            {
                DisplayError(ex.Message);
            }
        }

        #endregion

        #region Management of menu "Project"

        private void projectSelectDirectoryToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            folderBrowserDialog1.Description = Settings.Default.projectDirectory;
            folderBrowserDialog1.SelectedPath = Settings.Default.projectDirectory;
            DialogResult res = folderBrowserDialog1.ShowDialog();
            if (res == DialogResult.OK)
            {
                string folder = folderBrowserDialog1.SelectedPath;
                Settings.Default.projectDirectory = folder;
                _ViewModel.ProjectSetProjectDirectory(folder);
            }
        }

        private void projectSelectImagesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //string subDirPath = Settings.Default.projectImagesSubDirectory;
            //string[] subDirs = subDirPath.Split(System.IO.Path.DirectorySeparatorChar);
            //string subDir = subDirs[subDirs.Length - 1];
            folderBrowserDialog1.Description = Settings.Default.projectImagesSubDirectory;
            folderBrowserDialog1.SelectedPath = Settings.Default.projectDirectory;
            DialogResult res = folderBrowserDialog1.ShowDialog();
            if (res == DialogResult.OK)
            {
                string folder = folderBrowserDialog1.SelectedPath;
                Settings.Default.projectImagesSubDirectory = folder;
                _ViewModel.ProjectSetImagesSubDirectory(folder);
            }
        }

        private void Background_ProjectSelectCurrentImage(object sender, DoWorkEventArgs e)
        {
            _ViewModel.ProjectSelectCurrentImage((string)e.Argument);
        }
        private void projectSelectCurrentImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            /*
            Project.SelectCurrentImage frm = new GUI.Project.SelectCurrentImage();
            frm.SetImageFilenames(_ViewModel.ProjectGetImages());
            DialogResult res = frm.ShowDialog();
            if (res == DialogResult.OK)
            {
                if (frm.lvSelectCurrentImage.SelectedItems.Count > 0)
                {
                    string filename = frm.lvSelectCurrentImage.SelectedItems[0].SubItems[1].Text;
                    try
                    {
                        //_ViewModel.ProjectSelectCurrentImage(filename);
                        //this.ShowAnalysis();
                        Background_WorkStart(new DoWorkEventHandler(Background_ProjectSelectCurrentImage), filename);
                        _CurrentImageChanged = true;
                    }
                    catch (CExceptionViewModel ex)
                    {
                        DisplayError(ex.Message);
                    }
                }
            }
            */
            
            openFileDialog1.Title = _ViewModel.ProjectGetProjectDirectory();
            openFileDialog1.InitialDirectory = _ViewModel.ProjectGetProjectDirectory();
            openFileDialog1.FileName = "";
            openFileDialog1.Multiselect = false;
            string filter = "";
            string[] extensionsList = GLOBALDATA.IMAGE_INPUT_FILE_EXT.Split(new char[] { ',' });
            foreach (string extension in extensionsList)
            {
                filter += "*." + extension + ";";
            }
            openFileDialog1.Filter = "Source Images (" + filter + ")|" + filter;
            DialogResult res = openFileDialog1.ShowDialog();
            if (res == DialogResult.OK)
            {
                string filename = openFileDialog1.FileName;
                try
                {
                    //_ViewModel.ProjectSelectCurrentImage(filename);
                    //this.ShowAnalysis();
                    Background_WorkStart(new DoWorkEventHandler(Background_ProjectSelectCurrentImage), filename);
                    _CurrentImageChanged = true;
                }
                catch (CExceptionViewModel ex)
                {
                    DisplayError(ex.Message);
                }
            }
        }
        private void projectCleanDirectoriesToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            DialogResult res = MessageBox.Show("WARNING : Project's directories will be cleaned !", "WARNING",
                MessageBoxButtons.OKCancel, MessageBoxIcon.Warning);
            if (res == DialogResult.OK)
            {
                GLOBALDATA.CleanDestinationDirectories();
            }
        }

        private void Background_ProjectScenarioLoad(object sender, DoWorkEventArgs e)
        {
            _ViewModel.ProjectScenarioLoad((string)e.Argument);
        }
        private void projectLoadScenarioToolStripMenuItem_Click(object sender, EventArgs e)
        {
            openFileDialog1.Title = Settings.Default.projectDirectory;
            openFileDialog1.InitialDirectory = Settings.Default.projectDirectory;
            openFileDialog1.Filter = "Agora Scenario (*.xml)|*.xml";
            openFileDialog1.FileName = Settings.Default.scenarioFilename;
            DialogResult res = openFileDialog1.ShowDialog();
            string filename = openFileDialog1.FileName;
            Settings.Default.scenarioFilename = System.IO.Path.GetFileName(filename);
            if (res == DialogResult.OK)
            {
                
                try
                {
                    //_ViewModel.ProjectScenarioLoad(filename);
                    //this.ShowAnalysis();
                    Background_WorkStart(new DoWorkEventHandler(Background_ProjectScenarioLoad), filename);
                    _CurrentImageChanged = true;
                }
                catch (CExceptionViewModel ex)
                {
                    DisplayError(ex.Message);
                }
                
            }
        }

        private void projectSaveScenarioToolStripMenuItem_Click(object sender, EventArgs e)
        {
            saveFileDialog1.Title = Settings.Default.projectDirectory;
            saveFileDialog1.InitialDirectory = Settings.Default.projectDirectory;
            saveFileDialog1.FileName = Settings.Default.scenarioFilename;
            saveFileDialog1.AddExtension = true;
            saveFileDialog1.Filter = "Agora Scenario (*.xml)|*.xml";
            DialogResult res = saveFileDialog1.ShowDialog();
            string filename = saveFileDialog1.FileName;
            Settings.Default.scenarioFilename = System.IO.Path.GetFileName(filename);
            if (res == DialogResult.OK)
            {
                try
                {
                    _ViewModel.ProjectScenarioSave(filename);
                }
                catch (CExceptionViewModel ex)
                {
                    DisplayError(ex.Message);
                }
            }
        }

        private void Background_ProjectApply(object sender, DoWorkEventArgs e)
        {
            _ViewModel.ProjectApply();
        }
        private void applyScenarioToAllImagesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show(
                "Directories will be cleaned, and the whole exportation process may take very long time...", "AGORA",
                MessageBoxButtons.OKCancel, MessageBoxIcon.Exclamation);
            if (result == DialogResult.OK)
            {
                //_ViewModel.ProjectApply();
                Background_WorkStart(new DoWorkEventHandler(Background_ProjectApply), null);
            }
        }

        #endregion

        #region Management of menu "Scenario"

        /*
        private void Background_ScenarioSetDimensions(object sender, DoWorkEventArgs e)
        {
            this.ScenarioSetDimensions((GUI.Project.Dimensions)e.Argument);
        }
        private void ScenarioSetDimensions(GUI.Project.Dimensions frm)
        {
            int nbDimensions = frm.clbDimensions.CheckedIndices.Count;
            int[] dimensions = new int[nbDimensions];
            for (int i = 0; i < nbDimensions; i++)
            {
                int index = frm.clbDimensions.CheckedIndices[i];
                switch (index)
                {
                    case 0: dimensions[i] = Agora.Model.ImageAnalysis.CDimensionType.DIM_EUCLIDIAN_X; break;
                    case 1: dimensions[i] = Agora.Model.ImageAnalysis.CDimensionType.DIM_EUCLIDIAN_Y; break;
                    case 2: dimensions[i] = Agora.Model.ImageAnalysis.CDimensionType.DIM_RANGEMAP_X; break;
                    case 3: dimensions[i] = Agora.Model.ImageAnalysis.CDimensionType.DIM_RANGEMAP_Y; break;
                }
            }
            _ViewModel.ScenarioSetDimensions(dimensions);
        }
        private void scenarioSetDimensionsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.Project.Dimensions frm = new GUI.Project.Dimensions();
            DialogResult res = frm.ShowDialog();
            if (res == DialogResult.OK)
            {
                //this.ScenarioSetDimensions(frm);
                //this.ShowAnalysis();
                Background_WorkStart(new DoWorkEventHandler(Background_ScenarioSetDimensions), frm);
            }
        }
        */

        private void scenarioClearToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            _ViewModel.ProjectScenarioClear();
            this.ShowAnalysis();
        }

        private void scenarioUndoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            _ViewModel.ScenarioRemoveLastStep();
            this.ShowAnalysis();
        }

        #endregion

        #region Management of menu "Misc. Operations"

        private void labelCreationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.MiscOperations.LabelCreation frm = new GUI.MiscOperations.LabelCreation();
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                _ViewModel.MiscOperatorCreateLabel.EOCTypeName = frm.LabelName;
                _ViewModel.MiscCreateLabel();
                this.ShowAnalysis();
            }
        }

        private void labelDeletionToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            GUI.MiscOperations.LabelDeletion frm = new GUI.MiscOperations.LabelDeletion();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels()); 
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                _ViewModel.MiscOperatorDeleteLabel.EOCTypeName = frm.LabelName;
                _ViewModel.MiscDeleteLabel();
                this.ShowAnalysis();
            }
        }

        private void Background_MiscExportResults(object sender, DoWorkEventArgs e)
        {
            this.MiscExportResults((GUI.MiscOperations.ExportResults)e.Argument);
        }
        private void MiscExportResults(GUI.MiscOperations.ExportResults frm)
        {
            _ViewModel.MiscOperatorExportResults.AltoGenerateThumbnails = frm.AltoGenerateThumbnails;
            _ViewModel.MiscOperatorExportResults.AltoThumbnailsJPEGFormat = frm.AltoThumbnailsJPEGFormat;
            _ViewModel.MiscOperatorExportResults.AltoThumbnailsPrefixOriginalFilename = frm.AltoThumbnailsPrefixOriginalFilename;
            _ViewModel.MiscOperatorExportResults.AltoIllustrationEOCTypes = frm.LabelsAltoIllustration;
            _ViewModel.MiscOperatorExportResults.AltoStringEOCTypes = frm.LabelsAltoString;
            _ViewModel.MiscOperatorExportResults.AltoTextblockEOCTypes = frm.LabelsAltoBlock;
            _ViewModel.MiscOperatorExportResults.AltoTextlineEOCTypes = frm.LabelsAltoLine;
            _ViewModel.MiscOperatorExportResults.AltoWordEOCTypes = frm.LabelsAltoWord;
            _ViewModel.MiscExportResults();
        }
        private void exportResultsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.MiscOperations.ExportResults frm = new GUI.MiscOperations.ExportResults();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                //this.MiscExportResults(frm);
                //this.ShowAnalysis();
                Background_WorkStart(new DoWorkEventHandler(Background_MiscExportResults), frm);
            }
        }

        #endregion

        #region Management of menu "L1 Operations"

        private void Background_FindCC(object sender, DoWorkEventArgs e)
        {
            this.FindCC((GUI.L1Operations.FindCC)e.Argument);
        }
        private void FindCC(GUI.L1Operations.FindCC frm)
        {
            _ViewModel.L1FindCC();
        }
        private void findCCToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.FindCC frm = new GUI.L1Operations.FindCC();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.Operator = _ViewModel.L1OperatorFindCC;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_FindCC), frm);
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.Delete frm = new GUI.L1Operations.Delete();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.Operator = _ViewModel.L1OperatorDelete;
            frm.ShowDialog();
            if (frm.DialogResult  == DialogResult.OK)
            {
                _ViewModel.L1Delete();
                this.ShowAnalysis();
            }
        }

        private void setFeaturesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.SetFeatures frm = new GUI.L1Operations.SetFeatures();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.Operator = _ViewModel.L1OperatorSetFeatures;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                _ViewModel.L1SetFeatures();
                this.ShowAnalysis();
            }
        }

        private void Background_L1Insert(object sender, DoWorkEventArgs e)
        {
            this.L1Insert((GUI.L1Operations.Insert)e.Argument);
        }
        private void L1Insert(GUI.L1Operations.Insert frm)
        {
            _ViewModel.L1Insert();
        }
        private void insertToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L1Operations.Insert frm = new GUI.L1Operations.Insert();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.Operator = _ViewModel.L1OperatorInsert;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L1Insert), frm);
            }
        }

        #endregion

        #region Management of menu "L2 Operations"

        private void Background_L2ClassifyByFeatures(object sender, DoWorkEventArgs e)
        {
            this.L2ClassifyByFeatures((GUI.L2Operations.ClassifyByFeatures)e.Argument);
        }
        private void L2ClassifyByFeatures(GUI.L2Operations.ClassifyByFeatures frm)
        {
            _ViewModel.L2ClassifyByFeatures();
        }
        private void classifyByFeaturesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L2Operations.ClassifyByFeatures frm = new GUI.L2Operations.ClassifyByFeatures();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.Operator = _ViewModel.L2OperatorClassifyByFeatures;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L2ClassifyByFeatures), frm);
            }
        }
        
        private void Background_L2OperationsFusion(object sender, DoWorkEventArgs e)
        {
            this.L2OperationsFusion((GUI.L2Operations.Fusion)e.Argument);
        }
        private void L2OperationsFusion(GUI.L2Operations.Fusion frm)
        {
            _ViewModel.L2Fusion();
        }
        private void fusionL2ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L2Operations.Fusion frm = new GUI.L2Operations.Fusion();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.Operator = _ViewModel.L2OperatorFusion;
            frm.ShowDialog();
            if (frm.DialogResult  == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L2OperationsFusion), frm);
            }
        }

        private void Background_L2OperationsIntersection(object sender, DoWorkEventArgs e)
        {
            this.L2OperationsIntersection((GUI.L2Operations.Intersection)e.Argument);
        }
        private void L2OperationsIntersection(GUI.L2Operations.Intersection frm)
        {
            _ViewModel.L2Intersect();
        }
        private void intersectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L2Operations.Intersection frm = new GUI.L2Operations.Intersection();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.Operator = _ViewModel.L2OperatorIntersect;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L2OperationsIntersection), frm);
            }
        }

        #endregion

        #region Management of menu "L3 Operations"

        private void textSegmentationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L3Operations.TextSegmentation frm = new GUI.L3Operations.TextSegmentation();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
            frm.Operator = _ViewModel.L3OperatorTextSegmentation;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L3OperationsTextSegmentation), frm);
            }
        }
        private void Background_L3OperationsTextSegmentation(object sender, DoWorkEventArgs e)
        {
            this.L3OperationTextSegmentation((GUI.L3Operations.TextSegmentation)e.Argument);
        }
        private void L3OperationTextSegmentation(GUI.L3Operations.TextSegmentation frm)
        {
            _ViewModel.L3TextSegmentation();
        }

        private void identifyEuclidianFeaturesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            GUI.L3Operations.EuclidianFeatures frm = new GUI.L3Operations.EuclidianFeatures();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            //frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
            frm.Operator = _ViewModel.L3OperatorEuclidianFeatures;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L3OperationsEuclidianFeatures), frm);
            }
        }
        private void Background_L3OperationsEuclidianFeatures(object sender, DoWorkEventArgs e)
        {
            this.L3OperationEuclidianFeatures((GUI.L3Operations.EuclidianFeatures)e.Argument);
        }
        private void L3OperationEuclidianFeatures(GUI.L3Operations.EuclidianFeatures frm)
        {
            _ViewModel.L3EuclidianFeatures();
        }

        private void identifyEuclidianSpacingToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            GUI.L3Operations.EuclidianSpacing frm = new GUI.L3Operations.EuclidianSpacing();
            frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
            frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
            frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
            frm.Operator = _ViewModel.L3OperatorEuclidianSpacing;
            frm.ShowDialog();
            if (frm.DialogResult == DialogResult.OK)
            {
                Background_WorkStart(new DoWorkEventHandler(Background_L3OperationsEuclidianSpacing), frm);
            }
        }
        private void Background_L3OperationsEuclidianSpacing(object sender, DoWorkEventArgs e)
        {
            this.L3OperationEuclidianSpacing((GUI.L3Operations.EuclidianSpacing)e.Argument);
        }
        private void L3OperationEuclidianSpacing(GUI.L3Operations.EuclidianSpacing frm)
        {
            _ViewModel.L3EuclidianSpacing();
        }

        #endregion

        #region Management of menu "View"

        private void zoomFitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.FitToScreen();
            this.ShowAnalysisImage();
            this.RefreshTBZoomValue();
        }

        private void zoomInToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.ScaleIncrement();
            this.ShowAnalysisImage();
            this.RefreshTBZoomValue();
        }

        private void zoomOutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.ScaleDecrement();
            this.ShowAnalysisImage();
            this.RefreshTBZoomValue();
        }

        private void RefreshTBZoomValue()
        {
            tbZoomValue.Text = (_Scale * 100.0).ToString() + "%";
        }
        private void tbZoomValue_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)
            {
                try
                {
                    double scale = Double.Parse(tbZoomValue.Text);
                    _Scale = scale / 100.0;
                    this.ScaleAdjust();
                    this.ShowAnalysisImage();
                    this.RefreshTBZoomValue();
                }
                catch (FormatException ex)
                {
                    this.FitToScreen();
                    this.ShowAnalysisImage();
                    this.RefreshTBZoomValue();
                }
            }
        }

        #endregion

        /// <summary>
        /// Top level method for viewing all analysis informations.
        /// This method calls other methods for displaying specific informations
        /// </summary>
        private void ShowAnalysis()
        {
            this.ShowAnalysisImage();
            this.ShowAnalysisLabels();
            this.ShowAnalysisGraph();
            this.ShowScenario();
        }

        #region Management of displayed image (Zoom, etc.)

        private Bitmap _OriginalImage = null;
        private Bitmap _ScaledImage = null;
        private Bitmap _DisplayedImage = null;
        private double _Scale = 1.0;
        //private double _PreviousScale = 0.0;
        private bool _CurrentImageChanged = true;
        private bool _ScaleChanged = true;
        private bool _LabelListChanged = true;
        private bool _GraphChanged = true;

        private double[] _Scales = new double[] {
            0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 2.0, 3.0, 4.0, 5.0
            };
        private void ScaleIncrement()
        {
            for (int i = 0; i < _Scales.Length - 1; i++)
            {
                if (_Scale == _Scales[i])
                {
                    _Scale = _Scales[i + 1];
                    break;
                }
            }
            _ScaleChanged = true;
        }
        private void ScaleDecrement()
        {
            for (int i = 1; i < _Scales.Length; i++)
            {
                if (_Scale == _Scales[i])
                {
                    _Scale = _Scales[i - 1];
                    break;
                }
            }
            _ScaleChanged = true;
        }
        private void ScaleAdjust()
        {
            for (int i = 0; i < _Scales.Length - 1; i++)
            {
                if ((_Scales[i] <= _Scale) && (_Scale < _Scales[i + 1]))
                {
                    _Scale = _Scales[i];
                    break;
                }
            }
            _ScaleChanged = true;
        }
        private void ShowAnalysisImage()
        {
            _OriginalImage = _ViewModel.AnalysisGetCurrentImage();
            this.ComputeViewedImage();
        }

        private void FitToScreen()
        {
            if (_OriginalImage == null) return;
            _Scale = Math.Min(
                (double)pbxImage.Parent.Width / (double)_OriginalImage.Width,
                (double)pbxImage.Parent.Height / (double)_OriginalImage.Height);
            this.ScaleAdjust();
        }

        private void ComputeViewedImage()
        {
            if (_OriginalImage == null) return;

            if (_CurrentImageChanged == true)
            {
                _CurrentImageChanged = false;
                _ScaleChanged = true; // To recompute image below
            }

            // Scale management:
            if (_ScaleChanged == true)
            {
                _ScaleChanged = false;

                int newW = Convert.ToInt32((double)_OriginalImage.Width * _Scale);
                int newH = Convert.ToInt32((double)_OriginalImage.Height * _Scale);

                ResizeNearestNeighbor filter = new ResizeNearestNeighbor(newW, newH);
                //ResizeBicubic filter = new ResizeBicubic(newW, newH);
                //ResizeBilinear filter = new ResizeBilinear(newW, newH);
                _ScaledImage = filter.Apply(_OriginalImage);
                _LabelListChanged = true; // To repaint EOC below
            }

            // Paiting EOC of selected labels over image
            if (_LabelListChanged == true)
            {
                _LabelListChanged = false;

                List<string> labels = _ViewModel.AnalysisGetLabels();
                _DisplayedImage = (Bitmap)_ScaledImage.Clone();
                Graphics graphics = Graphics.FromImage(_DisplayedImage);
                foreach (ListViewItem item in lvLabels.SelectedItems)
                {
                    string label = item.Text;
                    if (! labels.Contains(label)) continue;
                    int labelNum = labels.IndexOf(label);
                    Pen pen = new Pen(Color.FromKnownColor(_LabelColors[labelNum]));
                    pen.Width = 3;
                    List<Rectangle> boundingBoxes = _ViewModel.AnalysisGetBoundingBoxes(label);
                    foreach (Rectangle rect in boundingBoxes)
                    {
                        Rectangle scaledRect = new Rectangle(
                            (int)((double)rect.X * _Scale),
                            (int)((double)rect.Y * _Scale),
                            (int)((double)rect.Width * _Scale) - 1,
                            (int)((double)rect.Height * _Scale - 1)
                            );
                        graphics.DrawRectangle(pen, scaledRect);
                    }
                }
            }

            // Painting EOC selected in graph
            if (_GraphChanged == true)
            {
                _GraphChanged = false;

                _DisplayedImage = (Bitmap)_ScaledImage.Clone();
                Graphics graphics = Graphics.FromImage(_DisplayedImage);
                TreeNode node = tvGraph.SelectedNode;
                if (node != null)
                {
                    int labelNum = node.ImageIndex;
                    Rectangle rect = _ViewModel.AnalysisGraphGetBoundingBox(node.Tag);
                    Pen pen = new Pen(Color.FromKnownColor(_LabelColors[labelNum]));
                    pen.Width = 3;
                    Rectangle scaledRect = new Rectangle(
                        (int)((double)rect.X * _Scale),
                        (int)((double)rect.Y * _Scale),
                        (int)((double)rect.Width * _Scale) - 1,
                        (int)((double)rect.Height * _Scale - 1)
                        );
                    graphics.DrawRectangle(pen, scaledRect);
                }
            }

            pbxImage.Image = _DisplayedImage;
        }

        /*
        private void button3_Click(object sender, EventArgs e)
        {
            //_Scale = Convert.ToDouble(textBox1.Text) / 100.0;
            this.ComputeViewedImage();
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            //Matrix mx = new Matrix(1, 0, 0, 1, AutoScrollPosition.X, AutoScrollPosition.Y);
            //e.Graphics.Transform = mx;
            this.ComputeViewedImage();
        }
        */

        private void pbxImage_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                int xMouse = e.X;
                int yMouse = e.Y;
                int xImage = Convert.ToInt32(Convert.ToDouble(xMouse) / _Scale);
                int yImage = Convert.ToInt32(Convert.ToDouble(yMouse) / _Scale);
                toolStripStatusLabel1.Text = "X=" + xImage.ToString() + " Y=" + yImage.ToString();
                if (lvLabels.SelectedItems.Count > 0)
                {
                    string label = lvLabels.SelectedItems[0].Text;
                    TreeNode nearestNode = _ViewModel.SearchNearestTreeNode(tvGraph.Nodes[0], tvGraph.Nodes[0], label, xImage, yImage);
                    tvGraph.SelectedNode = nearestNode;
                }
            }
        }
        private void pbxImage_MouseHover(object sender, EventArgs e)
        {

        }
        private void pbxImage_MouseEnter(object sender, EventArgs e)
        {
            //pbxImage.Focus();
            pnlImage.Focus();
        }

        private bool _MouseButtonRightPressed = false;

        private void pbxImage_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                _MouseButtonRightPressed = true;
            }
        }

        private void pbxImage_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
                _MouseButtonRightPressed = false;
            }
        }

        private void pbxImage_MouseWheel(object sender, MouseEventArgs e)
        {
            //MessageBox.Show(e.Delta.ToString());
            if (_MouseButtonRightPressed == true)
            {
                if (e.Delta < 0)
                {
                    this.ScaleIncrement();
                }
                else
                {
                    this.ScaleDecrement();
                }
                this.ShowAnalysisImage();
                this.RefreshTBZoomValue();
                HandledMouseEventArgs ev = (HandledMouseEventArgs)e;
                ev.Handled = true;
            }
        }

        private void CenterOnSelectedEOC()
        {
            TreeNode node = tvGraph.SelectedNode;
            if (node != null)
            {
                Rectangle rect = _ViewModel.AnalysisGraphGetBoundingBox(node.Tag);
                int centerX = rect.X + rect.Width / 2;
                int centerY = rect.Y + rect.Height / 2;
                int scaledCenterX = (int)((double)centerX * _Scale);
                int scaledCenterY = (int)((double)centerY * _Scale);
                pnlImage.AutoScrollHPos = scaledCenterX - pnlImage.HorizontalScroll.LargeChange / 2;
                pnlImage.performScrollHorizontal(ScrollEventType.ThumbPosition);
                pnlImage.AutoScrollVPos = scaledCenterY - pnlImage.VerticalScroll.LargeChange / 2;
                pnlImage.performScrollVertical(ScrollEventType.ThumbPosition);
            }
        }
        private void centerOnSelectedEOCToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.CenterOnSelectedEOC();
        }

        #endregion (Zoom, etc.)

        #region Management of displayed labels

        private void ShowAnalysisLabels()
        {
            List<string> labels = _ViewModel.AnalysisGetLabels();
            lvLabels.Items.Clear();
            for (int i = 0; i < labels.Count; i++)
            {
                string label = labels[i];
                /*
                ListViewItem item = new ListViewItem();
                item.Text = label;
                item.ImageIndex = 0;
                item.SubItems.Add("Color");
                item.SubItems[1].BackColor = Color.Blue;
                */
                lvLabels.Items.Add(label, i);
            }
            //lbxLabels.DataSource = _ViewModel.AnalysisGetLabels();
            lvLabels.AutoResizeColumn(0, ColumnHeaderAutoResizeStyle.ColumnContent);
        }

        private void lvLabels_SelectedIndexChanged(object sender, EventArgs e)
        {
            _LabelListChanged = true;
            this.ShowAnalysisImage();
        }

        private void lvLabels_MouseClick(object sender, MouseEventArgs e)
        {
            _LabelListChanged = true;
            this.ShowAnalysisImage();
        }

        #endregion

        #region Management of displayed graph

        private void ShowAnalysisGraph()
        {
            tvGraph.Visible = false;
            tvGraph.Nodes.Clear();
            tvGraph.Nodes.Add(_ViewModel.AnalysisGetGraph());
            tvGraph.ExpandAll();
            tvGraph.Visible = true;
        }
        private void tvGraph_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
                this.CenterOnSelectedEOC();
        }
        private void tvGraph_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            //this.CenterOnSelectedEOC();
        }
        private void tvGraph_AfterSelect(object sender, TreeViewEventArgs e)
        {
            _GraphChanged = true;
            //this.CenterOnSelectedEOC();
            this.ShowAnalysisImage();
        }

        #endregion

        #region Management of displayed scenario
        private void ShowScenario()
        {
            lbHistory.DataSource = _ViewModel.ScenarioGetStepStrings();
        }

        private void lbHistory_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            int n = lbHistory.SelectedIndex;
            List<Agora.Model.Project.CStepOperator> stepOperators = _ViewModel.ScenarioGetStepOperators();
            Agora.Model.Project.CStepOperator stepOperator = stepOperators[n];
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel1.CFindCC))
            {
                GUI.L1Operations.FindCC frm = new GUI.L1Operations.FindCC();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.Operator = (Agora.Model.OperatorLevel1.CFindCC)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel1.CDelete))
            {
                GUI.L1Operations.Delete frm = new GUI.L1Operations.Delete();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.Operator = (Agora.Model.OperatorLevel1.CDelete)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel1.CInsert))
            {
                GUI.L1Operations.Insert frm = new GUI.L1Operations.Insert();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.Operator = (Agora.Model.OperatorLevel1.CInsert)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel1.CSetFeatures))
            {
                GUI.L1Operations.SetFeatures frm = new GUI.L1Operations.SetFeatures();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.Operator = (Agora.Model.OperatorLevel1.CSetFeatures)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel2.CClassifyByFeatures))
            {
                GUI.L2Operations.ClassifyByFeatures frm = new GUI.L2Operations.ClassifyByFeatures();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.Operator = (Agora.Model.OperatorLevel2.CClassifyByFeatures)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel2.CFusion))
            {
                GUI.L2Operations.Fusion frm = new GUI.L2Operations.Fusion();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.Operator = (Agora.Model.OperatorLevel2.CFusion)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel2.CIntersect))
            {
                GUI.L2Operations.Intersection frm = new GUI.L2Operations.Intersection();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.Operator = (Agora.Model.OperatorLevel2.CIntersect)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel3.CEuclidianFeatures))
            {
                GUI.L3Operations.EuclidianFeatures frm = new GUI.L3Operations.EuclidianFeatures();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                //frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
                frm.Operator = (Agora.Model.OperatorLevel3.CEuclidianFeatures)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel3.CEuclidianSpacing))
            {
                GUI.L3Operations.EuclidianSpacing frm = new GUI.L3Operations.EuclidianSpacing();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                //frm.NbDimensions = _ViewModel.AnalysisGetDimensions().Length;
                frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
                frm.Operator = (Agora.Model.OperatorLevel3.CEuclidianSpacing)stepOperator;
                frm.ShowDialog();
            }
            if (stepOperator.GetType() == typeof(Agora.Model.OperatorLevel3.CTextSegmentation))
            {
                GUI.L3Operations.TextSegmentation frm = new GUI.L3Operations.TextSegmentation();
                frm.Labels.AddRange(_ViewModel.AnalysisGetLabels());
                frm.OriginalImage = _ViewModel.AnalysisGetCurrentImage();
                frm.Operator = (Agora.Model.OperatorLevel3.CTextSegmentation)stepOperator;
                frm.ShowDialog();
            }
        }

        #endregion


    }
}
