﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace GUI.Project
{
    public partial class SelectCurrentImage : Form
    {
        private string _CurrentImageFilename = "";

        public SelectCurrentImage()
        {
            InitializeComponent();
        }

        public void SetImageFilenames(List<string> filenames)
        {
            ImageList imageList = new ImageList();
            lvSelectCurrentImage.SmallImageList = imageList;
            lvSelectCurrentImage.LargeImageList = imageList;
            imageList.ColorDepth = ColorDepth.Depth24Bit;
            imageList.TransparentColor = Color.Transparent;
            imageList.ImageSize = new Size(128, 128);

            foreach (string filename in filenames)
            {
                string name = System.IO.Path.GetFileName(filename);
                ListViewItem item = new ListViewItem(name, name);
                item.SubItems.Add(filename);
                lvSelectCurrentImage.Items.Add(item);

                Bitmap img = new Bitmap(filename);
                Bitmap bmp = new Bitmap(imageList.ImageSize.Width, imageList.ImageSize.Height);
                int posX = 0;
                int posY = 0;
                int h = bmp.Height;
                int w = bmp.Width;
                if (img.Height < img.Width)
                {
                    w = h * img.Height / img.Width;
                    posY = (bmp.Width - w) / 2;
                }
                else
                {
                    h = w * img.Width / img.Height;
                    posX = (bmp.Height - h) / 2;
                }
                Graphics g = Graphics.FromImage(bmp);
                Pen pen = new Pen(Color.LightGray, 1);
                g.DrawRectangle(pen, 0, 0, bmp.Width - 1, bmp.Height - 1);

                AForge.Imaging.Filters.ResizeBilinear f = new AForge.Imaging.Filters.ResizeBilinear(
                    h - 2, w - 2);
                img = f.Apply(img);

                g.DrawImage(img, new Rectangle(posX + 1, posY + 1, h - 2, w - 2));

                imageList.Images.Add(name, bmp);
            }
            lvSelectCurrentImage.Refresh();
        }

        private void lvSelectCurrentImage_ItemSelectionChanged(object sender, ListViewItemSelectionChangedEventArgs e)
        {
            _CurrentImageFilename = e.Item.SubItems[1].Text;
            pbCurrentImage.Refresh();
        }

        private void pbCurrentImage_Paint(object sender, PaintEventArgs e)
        {
            if (System.IO.File.Exists(_CurrentImageFilename) == true)
            {
                Graphics g = e.Graphics;
                Bitmap img = new Bitmap(_CurrentImageFilename);
                double wScale = (double)img.Width / (double)pbCurrentImage.Width;
                double hScale = (double)img.Height / (double)pbCurrentImage.Height;
                double scale = Math.Max(wScale, hScale);
                int w = (int)((double)img.Width / scale);
                int h = (int)((double)img.Height / scale);
                AForge.Imaging.Filters.ResizeBilinear f =
                    new AForge.Imaging.Filters.ResizeBilinear(w, h);
                img = f.Apply(img);
                g.DrawImage(img, new Rectangle(
                    (pbCurrentImage.Width - w) / 2,
                    (pbCurrentImage.Height - h) / 2,
                    w, h));
            }
        }
    }
}
