﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using AForge.Imaging.Filters;

namespace GUI
{
    public partial class SelectionInPictureForm : Form
    {
        private Bitmap _OriginalImage = null;
        public Bitmap OriginalImage
        {
            set { _OriginalImage = value; }
        }

        private Rectangle _Selection = Rectangle.Empty;
        public Rectangle Selection
        {
            get { return _Selection; }
        }

        public SelectionInPictureForm()
        {
            InitializeComponent();
            this.DialogResult = DialogResult.None;
        }

        private void SelectionInPictureForm_Shown(object sender, EventArgs e)
        {
            this.ComputeDisplayedImage();
            this.SetControlsValues();
        }

        private void SetControlsValues()
        {
            this.SuspendLayout();

            lblSelectionInfo.Text =
                "{" +
                "L=" + Selection.Left.ToString() + "," +
                "T=" + Selection.Top.ToString() + "," +
                "R=" + Selection.Right.ToString() + "," +
                "B=" + Selection.Bottom.ToString() +
                "}{" +
                "W=" + Selection.Width.ToString() + "," +
                "H=" + Selection.Height.ToString() +
                "}";

            this.ResumeLayout(false);
            this.PerformLayout();
        }

        #region SCALE MANAGMENT

        private double _Scale = 1.0;
        private double[] _Scales = new double[] {
            0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 2.0, 3.0, 4.0, 5.0
            };

        private void ScaleIncrement()
        {
            for (int i = 0; i < _Scales.Length - 1; i++)
            {
                if (_Scale == _Scales[i])
                {
                    _Scale = _Scales[i + 1];
                    break;
                }
            }
        }
        private void ScaleDecrement()
        {
            for (int i = 1; i < _Scales.Length; i++)
            {
                if (_Scale == _Scales[i])
                {
                    _Scale = _Scales[i - 1];
                    break;
                }
            }
        }
        #endregion

        #region DISPLAYED IMAGE MANAGMENT

        private Bitmap _DisplayedImage = null;

        private void ComputeDisplayedImage()
        {
            if (_OriginalImage == null) return;

            // Scaling
            int newW = Convert.ToInt32((double)_OriginalImage.Width * _Scale);
            int newH = Convert.ToInt32((double)_OriginalImage.Height * _Scale);

            ResizeNearestNeighbor filter = new ResizeNearestNeighbor(newW, newH);
            _DisplayedImage = filter.Apply(_OriginalImage);
            pictureBox1.Image = _DisplayedImage;
        }

        private void pictureBox1_Paint(object sender, PaintEventArgs e)
        {
            if (this._Selecting)
            {
                Graphics g = e.Graphics;
                //Graphics g = pictureBox1.CreateGraphics();
                Pen pen = new Pen(Color.Black);
                Brush selectionBrush = new SolidBrush(Color.FromArgb(128, 72, 145, 220));
                pen.DashStyle = DashStyle.Dash;
                //g.DrawRectangle(pen, this._selectionRect);
                g.FillRectangle(selectionBrush, this._SelectionRect);
            }
        }

        #endregion

        #region SELECTION MANAGMENT

        private bool _Selecting = false;
        private Point _SelectionRectStartPoint = Point.Empty;
        private Point _SelectionRectEndPoint = Point.Empty;
        private Rectangle _SelectionRect = Rectangle.Empty;

        private const FrameStyle SELECTION_STYLE = FrameStyle.Dashed;

        private void pictureBox1_MouseDown(object sender, MouseEventArgs e)
        {
            //base.OnMouseDown(e);

            if (e.Button == MouseButtons.Left)
            {
                this._Selecting = true;
                int hScrollBarValue = panel1.HorizontalScroll.Value;
                int vScrollBarValue = panel1.VerticalScroll.Value;

                this._SelectionRectStartPoint = this.PointToScreen(
                    new Point(
                        Convert.ToInt32((e.X - hScrollBarValue) ),
                        Convert.ToInt32((e.Y - vScrollBarValue) )
                        )
                       );
                this._SelectionRectEndPoint = Point.Empty;
                this._SelectionRect = Rectangle.Empty;

                this._Selection.X = Convert.ToInt32(e.X / _Scale);
                this._Selection.Y = Convert.ToInt32(e.Y / _Scale);

                pictureBox1.Invalidate();
                this.SetControlsValues();
            }
        }

        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            //base.OnMouseMove(e);

            if (this._Selecting)
            {
                // efface le rectangle de sélection précédent
                ControlPaint.DrawReversibleFrame(this._SelectionRect, this.BackColor, SELECTION_STYLE);

                int hScrollBarValue = panel1.HorizontalScroll.Value;
                int vScrollBarValue = panel1.VerticalScroll.Value;

                // calcul du nouveau rectangle
                this._SelectionRectEndPoint = this.PointToScreen(
                    new Point(
                        Convert.ToInt32((e.X - hScrollBarValue) ),
                        Convert.ToInt32((e.Y - vScrollBarValue) ))
                    );
                this._SelectionRect = new Rectangle(
                    this._SelectionRectStartPoint.X,
                    this._SelectionRectStartPoint.Y,
                    this._SelectionRectEndPoint.X - this._SelectionRectStartPoint.X,
                    this._SelectionRectEndPoint.Y - this._SelectionRectStartPoint.Y
                    );

                // dessin du nouveau rectangle
                ControlPaint.DrawReversibleFrame(this._SelectionRect, this.BackColor, SELECTION_STYLE);
                Invalidate(this.ClientRectangle); // Efface la sélection.
                //pictureBox1.Invalidate();

                this._Selection.Width = Convert.ToInt32(e.X / _Scale) - this._Selection.X;
                this._Selection.Height = Convert.ToInt32(e.Y / _Scale) - this._Selection.Y;
                this.SetControlsValues();
            }
        }

        private void pictureBox1_MouseUp(object sender, MouseEventArgs e)
        {
            //base.OnMouseUp(e);

            if (this._Selecting)
            {
                // efface le rectangle de sélection
                //ControlPaint.DrawReversibleFrame(this._selectionRect, this.BackColor, SELECTION_STYLE);
                //Invalidate(this.ClientRectangle); // Efface la sélection.
                //pictureBox1.Refresh();
            }

            this._Selecting = false;
            //MessageBox.Show(this._Selection.ToString());
            this.SetControlsValues();
        }

        #endregion

        private void buttonZoomPlus_Click(object sender, EventArgs e)
        {
            this.ScaleIncrement();
            this.ComputeDisplayedImage();
            lblZoomFactorValue.Text = (_Scale * 100).ToString() + "%";
        }

        private void buttonZoomMinus_Click(object sender, EventArgs e)
        {
            this.ScaleDecrement();
            this.ComputeDisplayedImage();
            lblZoomFactorValue.Text = (_Scale * 100).ToString() + "%";
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

    }
}
