﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;

namespace GUI
{
    public class TextBoxTraceListener : TraceListener
    {
        private System.Windows.Forms.TextBox _target = null;
        private delegate void StringSendDelegate(string msg);
        private StringSendDelegate _invokeWrite = null;

        public TextBoxTraceListener(System.Windows.Forms.TextBox tb)
        {
            _target = tb;
            _invokeWrite = new StringSendDelegate(SendString);
        }

        public override void Write(string message)
        {
            if (_target != null)
                _target.Invoke(_invokeWrite, new object[] { message });
        }
        public override void WriteLine(string message)
        {
            if (_target != null)
                _target.Invoke(_invokeWrite, new object[] { message + Environment.NewLine });
        }

        private void SendString(string msg)
        {
            if (_target != null)
            {
                _target.Text += msg;
                _target.SelectionStart = _target.Text.Length - 1;
                _target.SelectionLength = 1;
                _target.ScrollToCaret();
            }
        }
    }
}
